@@ -9,10 +9,13 @@
 """Debusine worker controller."""
 
 import logging
+import os
 import shutil
+import socket
 from pathlib import Path
 
 from utils import common
+from utils.server import DebusineServer
 from utils.waiter import Waiter
 
 logger = logging.getLogger(__name__)
@@ -23,9 +26,10 @@ class Worker:
 
     CONFIG_DIRECTORY = Path('/etc/debusine/worker')
     TOKEN_FILE = CONFIG_DIRECTORY / 'token'
+    ACTIVATION_TOKEN_FILE = CONFIG_DIRECTORY / 'activation-token'
 
     @classmethod
-    def set_up(cls) -> None:
+    def set_up(cls, use_activation_token: bool = True) -> None:
         """
         Set up worker and waits for the token file.
 
@@ -34,8 +38,12 @@ class Worker:
         - stop a possible debusine-worker
         - copy the debusine-worker's config.ini file to the appropriate
           directory
+        - if use_activation_token is true:
+            - create the worker in the server's database with an activation
+              token
+            - configure the worker using that activation token
         - start debusine-worker
-        - wait for the token file to appear
+        - wait for the (non-activation) token file to appear
         """
         common.run(['systemctl', 'stop', 'debusine-worker'])
 
@@ -45,10 +53,30 @@ class Worker:
         cls.CONFIG_DIRECTORY.mkdir(parents=True)
         shutil.chown(cls.CONFIG_DIRECTORY, 'debusine-worker')
 
-        shutil.copy(
-            '/usr/share/doc/debusine-worker/examples/config.ini',
-            cls.CONFIG_DIRECTORY,
-        )
+        with (
+            open(
+                "/usr/share/doc/debusine-worker/examples/config.ini"
+            ) as orig_config_file,
+            open(cls.CONFIG_DIRECTORY / "config.ini", "w") as config_file,
+        ):
+            for line in orig_config_file:
+                if line.startswith("api-url = "):
+                    config_file.write(
+                        f"api-url = https://{socket.getfqdn()}/api\n"
+                    )
+                else:
+                    config_file.write(line)
+
+        if use_activation_token:
+            created_worker = DebusineServer.execute_command(
+                "worker", "create", socket.getfqdn()
+            )
+
+            with open(cls.ACTIVATION_TOKEN_FILE, "w") as token_file:
+                os.fchmod(token_file.fileno(), 0o600)
+                token_file.write(created_worker.stdout)
+            shutil.chown(cls.ACTIVATION_TOKEN_FILE, "debusine-worker")
+
         common.run(['systemctl', 'start', 'debusine-worker'])
 
         cls.wait_for_token_file()
