@@ -18,6 +18,7 @@ from debusine.artifacts.local_artifact import (
     LintianArtifact,
     LocalArtifact,
     PackageBuildLog,
+    RepositoryIndex,
     SigningInputArtifact,
     SigningOutputArtifact,
     SourcePackage,
@@ -35,6 +36,7 @@ __all__ = [
     "LintianArtifact",
     "LocalArtifact",
     "PackageBuildLog",
+    "RepositoryIndex",
     "SigningInputArtifact",
     "SigningOutputArtifact",
     "SourcePackage",
