@@ -763,6 +763,17 @@ class DebDiff(ArtifactData):
         return f"debdiff {self.original} {self.new}"
 
 
+class DebianRepositoryIndex(ArtifactData):
+    """Data for debian:repository-index artifacts."""
+
+    # New artifacts have this set; old ones may not.
+    path: str | None = None
+
+    def get_label(self) -> str | None:
+        """Return a short human-readable label for the artifact."""
+        return self.path
+
+
 def get_source_package_name(
     artifact_data: (
         DebianSourcePackage
