@@ -1588,28 +1588,11 @@ class RepositoryIndexTests(TestCase):
     def test_create(self) -> None:
         """Create a valid RepositoryIndex."""
         directory = self.create_temporary_directory()
-        (release := directory / "Release").write_text("A Release file\n")
-
-        artifact = RepositoryIndex.create(file=release)
-
-        artifact.validate_model()
-        self.assertEqual(
-            artifact,
-            RepositoryIndex(
-                category=RepositoryIndex._category,
-                files={"Release": release},
-                data=data_models.EmptyArtifactData(),
-            ),
-        )
-
-    def test_create_override_name(self) -> None:
-        """Override the file name when creating a RepositoryIndex."""
-        directory = self.create_temporary_directory()
         (
             release := directory / "deb.debian.org_debian_dists_sid_Release"
         ).write_text("A Release file\n")
 
-        artifact = RepositoryIndex.create(file=release, override_name="Release")
+        artifact = RepositoryIndex.create(file=release, path="Release")
 
         artifact.validate_model()
         self.assertEqual(
@@ -1617,6 +1600,6 @@ class RepositoryIndexTests(TestCase):
             RepositoryIndex(
                 category=RepositoryIndex._category,
                 files={"Release": release},
-                data=data_models.EmptyArtifactData(),
+                data=data_models.DebianRepositoryIndex(path="Release"),
             ),
         )
