@@ -677,6 +677,14 @@ class DebDiffTest(TestCase):
         )
 
 
+class DebianRepositoryIndexTests(TestCase):
+    """Test the DebianRepositoryIndex model."""
+
+    def test_get_label(self) -> None:
+        data = data_models.DebianRepositoryIndex(path="main/source/Sources.xz")
+        self.assertEqual(data.get_label(), "main/source/Sources.xz")
+
+
 class GetSourcePackageNameTests(TestCase):
     """Tests for function get_source_package_name."""
 
