@@ -72,6 +72,7 @@ from debusine.client.models import (
     WorkRequestResponse,
     WorkflowTemplateRequest,
     WorkflowTemplateResponse,
+    WorkspaceInheritanceChain,
     model_to_json_serializable_dict,
 )
 
@@ -881,6 +882,25 @@ class Debusine:
             expected_class=TaskConfigurationCollectionUpdateResults,
         )
 
+    def get_workspace_inheritance(
+        self, workspace: str
+    ) -> WorkspaceInheritanceChain:
+        """Get a workspace inheritance chain."""
+        return self._debusine_http_client.get(
+            f"/workspace/{workspace}/inheritance/",
+            expected_class=WorkspaceInheritanceChain,
+        )
+
+    def set_workspace_inheritance(
+        self, workspace: str, *, chain: WorkspaceInheritanceChain
+    ) -> WorkspaceInheritanceChain:
+        """Set a workspace inheritance chain."""
+        return self._debusine_http_client.post(
+            f"/workspace/{workspace}/inheritance/",
+            data=model_to_json_serializable_dict(chain),
+            expected_class=WorkspaceInheritanceChain,
+        )
+
     def _log_tenacity_exception(
         self, retry_state: tenacity.RetryCallState
     ) -> None:
