@@ -11,11 +11,11 @@
 
 import json
 import re
-from collections.abc import Iterator, Sequence
+from collections.abc import Collection, Iterator, Sequence
 from datetime import datetime
 from enum import StrEnum
 from pathlib import Path
-from typing import Any, Generic, Literal, NewType, TypeVar
+from typing import Any, Generic, Literal, NewType, Self, TypeVar
 
 try:
     import pydantic.v1 as pydantic
@@ -469,6 +469,75 @@ class TaskConfigurationCollectionUpdateResults(StrictBaseModel):
     unchanged: int
 
 
+class WorkspaceInheritanceChainElement(StrictBaseModel):
+    """An element in an inheritance chain."""
+
+    id: int | None = None
+    scope: str | None = None
+    workspace: str | None = None
+
+    @pydantic.root_validator
+    @classmethod
+    def not_empty(cls, values: dict[str, Any]) -> dict[str, Any]:
+        """Ensure at least one of id or workspace is set."""
+        if values.get("id") is None and values.get("workspace") is None:
+            raise ValueError("at least id or workspace need to be set")
+        return values
+
+    def matches(self, element: "WorkspaceInheritanceChainElement") -> bool:
+        """Check if the components of element that are set match this one."""
+        return (
+            (element.id is None or element.id == self.id)
+            and (element.scope is None or element.scope == self.scope)
+            and (
+                element.workspace is None or element.workspace == self.workspace
+            )
+        )
+
+    @classmethod
+    def from_string(cls, value: str) -> Self:
+        """Resolve a command line argument to a WorkspaceInheritanceChain."""
+        if value.isdigit():
+            return cls(id=int(value))
+        elif "/" in value:
+            scope, workspace = value.split("/", 1)
+            return cls(scope=scope, workspace=workspace)
+        else:
+            return cls(workspace=value)
+
+
+class WorkspaceInheritanceChain(StrictBaseModel):
+    """Ordered list of workspaces inherited by a workspace."""
+
+    chain: list[WorkspaceInheritanceChainElement] = []
+
+    def __add__(self, other: "WorkspaceInheritanceChain") -> Self:
+        """Return the concatenation of this chain and another."""
+        return self.__class__(chain=self.chain + other.chain)
+
+    def __sub__(self, other: "WorkspaceInheritanceChain") -> Self:
+        """Return a copy of this chain without the elements in other."""
+        filtered: list[WorkspaceInheritanceChainElement] = []
+        for element in self.chain:
+            for to_remove in other.chain:
+                if element.matches(to_remove):
+                    break
+            else:
+                filtered.append(element)
+
+        return self.__class__(chain=filtered)
+
+    @classmethod
+    def from_strings(cls, values: Collection[str]) -> Self:
+        """Build an inheritance chain from a list of user-provided strings."""
+        return cls(
+            chain=[
+                WorkspaceInheritanceChainElement.from_string(value)
+                for value in values
+            ]
+        )
+
+
 def model_to_json_serializable_dict(
     model: pydantic.BaseModel, exclude_unset: bool = False
 ) -> dict[Any, Any]:
