@@ -72,6 +72,8 @@ from debusine.client.models import (
     WorkRequestExternalDebsignRequest,
     WorkRequestRequest,
     WorkflowTemplateRequest,
+    WorkspaceInheritanceChain,
+    WorkspaceInheritanceChainElement,
     model_to_json_serializable_dict,
 )
 from debusine.client.task_configuration import (
@@ -1753,6 +1755,34 @@ class DebusineTests(TestCase):
         assert responses.calls[1].request.body is not None
         self.assertTrue(json.loads(responses.calls[1].request.body)["dry_run"])
 
+    @responses.activate
+    def test_get_workspace_inheritance(self) -> None:
+        url = f"{self.debusine.api_url}/workspace/System/inheritance/"
+        chain = [
+            WorkspaceInheritanceChainElement(
+                id=3, scope="scope1", workspace="workspace1"
+            ),
+            WorkspaceInheritanceChainElement(
+                id=4, scope="scope2", workspace="workspace2"
+            ),
+        ]
+        result = WorkspaceInheritanceChain(chain=chain)
+        responses.add(responses.GET, url, json=result.dict())
+        res = self.debusine.get_workspace_inheritance("System")
+        self.assertEqual(res.chain, chain)
+        responses.assert_call_count(url, 1)
+
+    @responses.activate
+    def test_set_workspace_inheritance(self) -> None:
+        url = f"{self.debusine.api_url}/workspace/System/inheritance/"
+        result = WorkspaceInheritanceChain(chain=[])
+        responses.add(responses.POST, url, json=result.dict())
+        res = self.debusine.set_workspace_inheritance(
+            "System", chain=WorkspaceInheritanceChain(chain=[])
+        )
+        self.assertEqual(res.chain, [])
+        responses.assert_call_count(url, 1)
+
 
 class DebusineAsyncTests(server.DebusineAioHTTPTestCase, TestCase):
     """
