@@ -12,6 +12,7 @@
 import json
 from collections import deque
 from importlib.metadata import version
+from io import BytesIO
 from itertools import count
 from typing import Any
 from unittest import mock, skipIf
@@ -19,6 +20,11 @@ from unittest import mock, skipIf
 import requests
 import responses
 
+# Exported from top-level urllib3 module in >= 2.0.0, but that isn't in
+# bookworm.
+from urllib3.response import HTTPHeaderDict  # type: ignore[attr-defined]
+from urllib3.response import HTTPResponse
+
 from debusine.client import exceptions
 from debusine.client.debusine_http_client import DebusineHttpClient
 from debusine.client.models import StrictBaseModel
@@ -30,11 +36,9 @@ except ImportError:
     OrderedRegistry = None  # type: ignore
 
 
-class TestResponse(StrictBaseModel):
+class SampleResponse(StrictBaseModel):
     """A dummy model for test cases."""
 
-    __test__ = False
-
     id: int
     page: int | None = None
 
@@ -44,6 +48,7 @@ class DebusineHttpClientTests(TestCase):
 
     def setUp(self) -> None:
         """Initialize member variables for the tests."""
+        super().setUp()
         self.api_url = "https://debusine.debian.org/api/1.0"
         self.token = "token-for-server"
 
@@ -94,7 +99,7 @@ class DebusineHttpClientTests(TestCase):
         base_url: str, *, length: int = 4, page_size: int = 3
     ) -> None:
         """Add a set of responses for paginated listing requests."""
-        objects = deque(TestResponse(id=i) for i in range(length))
+        objects = deque(SampleResponse(id=i) for i in range(length))
 
         url = base_url
         for pageno in count(1):
@@ -137,7 +142,7 @@ class DebusineHttpClientTests(TestCase):
             ValueError, "^data argument not allowed with HTTP GET$"
         ):
             debusine_client._api_request(
-                "GET", self.api_url, TestResponse, data={}
+                "GET", self.api_url, SampleResponse, data={}
             )
 
     @responses.activate
@@ -168,7 +173,7 @@ class DebusineHttpClientTests(TestCase):
         with self.assertRaisesRegex(
             ValueError, "^Method must be one of: GET, POST, PUT, PATCH$"
         ):
-            debusine_client._api_request("BAD-METHOD", "", TestResponse)
+            debusine_client._api_request("BAD-METHOD", "", SampleResponse)
 
     @responses.activate
     def test_api_request_not_found_raise_exception(self) -> None:
@@ -193,7 +198,7 @@ class DebusineHttpClientTests(TestCase):
 
         debusine_client = DebusineHttpClient(self.api_url, self.token)
         with self.assertRaises(exceptions.DebusineError) as debusine_error:
-            debusine_client._api_request("GET", self.api_url, TestResponse)
+            debusine_client._api_request("GET", self.api_url, SampleResponse)
 
         exception = debusine_error.exception
 
@@ -213,7 +218,7 @@ class DebusineHttpClientTests(TestCase):
             exceptions.ClientForbiddenError,
             rf"^HTTP 403. Token \({token}\) is invalid or disabled$",
         ):
-            debusine_client._api_request("GET", self.api_url, TestResponse)
+            debusine_client._api_request("GET", self.api_url, SampleResponse)
 
     @responses.activate
     def test_api_request_raise_debusine_error_404(self) -> None:
@@ -230,7 +235,7 @@ class DebusineHttpClientTests(TestCase):
         )
         debusine_client = DebusineHttpClient(self.api_url, self.token)
         with self.assertRaises(exceptions.DebusineError) as debusine_error:
-            debusine_client._api_request("POST", self.api_url, TestResponse)
+            debusine_client._api_request("POST", self.api_url, SampleResponse)
 
         exception = debusine_error.exception
 
@@ -255,7 +260,7 @@ class DebusineHttpClientTests(TestCase):
 
         debusine_client = DebusineHttpClient(self.api_url, self.token)
         with self.assertRaises(exceptions.DebusineError) as debusine_error:
-            debusine_client._api_request("GET", self.api_url, TestResponse)
+            debusine_client._api_request("GET", self.api_url, SampleResponse)
 
         exception = debusine_error.exception
 
@@ -274,7 +279,7 @@ class DebusineHttpClientTests(TestCase):
             r"^Server returned unexpected status code: 418 "
             rf"\({self.api_url}\)$",
         ):
-            debusine_client._api_request("GET", self.api_url, TestResponse)
+            debusine_client._api_request("GET", self.api_url, SampleResponse)
 
         self.assert_token_key_included_in_all_requests(self.token)
 
@@ -290,7 +295,7 @@ class DebusineHttpClientTests(TestCase):
             "^Server returned unexpected status code: 400 "
             fr"\({self.api_url}\)$",
         ):
-            debusine_client._api_request("GET", self.api_url, TestResponse)
+            debusine_client._api_request("GET", self.api_url, SampleResponse)
 
     @responses.activate
     def test_api_request_raise_unexpected_response_400_no_json(self) -> None:
@@ -307,7 +312,7 @@ class DebusineHttpClientTests(TestCase):
             "^Server returned unexpected status code: 400 "
             fr"\({self.api_url}\)$",
         ):
-            debusine_client._api_request("GET", self.api_url, TestResponse)
+            debusine_client._api_request("GET", self.api_url, SampleResponse)
 
     @skipIf(
         [int(part) for part in version("responses").split(".")] < [0, 21],
@@ -337,7 +342,7 @@ class DebusineHttpClientTests(TestCase):
                         method=method, status=requests.codes.ok
                     )
                     debusine_client._api_request(
-                        method, self.api_url, TestResponse
+                        method, self.api_url, SampleResponse
                     )
 
     @skipIf(
@@ -367,7 +372,7 @@ class DebusineHttpClientTests(TestCase):
                     )
                 ):
                     debusine_client._api_request(
-                        responses.GET, self.api_url, TestResponse
+                        responses.GET, self.api_url, SampleResponse
                     )
 
     @responses.activate
@@ -383,7 +388,7 @@ class DebusineHttpClientTests(TestCase):
             exceptions.UnexpectedResponseError,
             fr"^Server \({self.api_url}\) did not return valid object. Error: ",
         ):
-            debusine_client._api_request("GET", self.api_url, TestResponse)
+            debusine_client._api_request("GET", self.api_url, SampleResponse)
 
         self.assert_token_key_included_in_all_requests(self.token)
 
@@ -401,7 +406,7 @@ class DebusineHttpClientTests(TestCase):
 
         # Make a call (the HTTP GET is done, but it failed)
         with self.assertRaises(exceptions.UnexpectedResponseError):
-            debusine_client._api_request("GET", self.api_url, TestResponse)
+            debusine_client._api_request("GET", self.api_url, SampleResponse)
 
         session_mock.assert_called_once()
         session_mock.return_value.get.assert_called_once_with(
@@ -411,7 +416,7 @@ class DebusineHttpClientTests(TestCase):
         session = debusine_client._session
         # Make a call (the HTTP GET is done, but it failed)
         with self.assertRaises(exceptions.UnexpectedResponseError):
-            debusine_client._api_request("GET", self.api_url, TestResponse)
+            debusine_client._api_request("GET", self.api_url, SampleResponse)
 
         # Same session reused
         self.assertIs(debusine_client._session, session)
@@ -426,7 +431,7 @@ class DebusineHttpClientTests(TestCase):
 
         # Make a call (the HTTP GET is done, but it failed)
         with self.assertRaises(exceptions.UnexpectedResponseError):
-            debusine_client._api_request("GET", self.api_url, TestResponse)
+            debusine_client._api_request("GET", self.api_url, SampleResponse)
 
         session_mock.assert_called_once()
         session_mock.return_value.get.assert_called_once_with(
@@ -446,7 +451,7 @@ class DebusineHttpClientTests(TestCase):
 
         # Make a call (the HTTP GET is done, but it failed)
         with self.assertRaises(exceptions.UnexpectedResponseError):
-            debusine_client._api_request("GET", self.api_url, TestResponse)
+            debusine_client._api_request("GET", self.api_url, SampleResponse)
 
         session_mock.assert_called_once()
         session_mock.return_value.get.assert_called_once_with(
@@ -476,7 +481,7 @@ class DebusineHttpClientTests(TestCase):
             debusine_client._api_request(
                 "POST",
                 self.api_url,
-                TestResponse,
+                SampleResponse,
                 data=data,
                 expected_statuses=expected_statuses,
             )
@@ -485,6 +490,37 @@ class DebusineHttpClientTests(TestCase):
             assert responses.calls[0].request.body is not None
             self.assertEqual(json.loads(responses.calls[0].request.body), data)
 
+    @responses.activate
+    def test_api_request_ok_no_content_type(self) -> None:
+        """_api_request() handles a 200 response with no Content-Type."""
+
+        class EmptyResponse(responses.BaseResponse):
+            """A response without even a Content-Type."""
+
+            def get_response(
+                self, request: requests.PreparedRequest
+            ) -> HTTPResponse:
+                headers = HTTPHeaderDict()
+                body = BytesIO()
+                # See comments in responses._form_response for the mypy
+                # suppressions here.
+                orig_response = HTTPResponse(
+                    body=body,
+                    msg=headers,  # type: ignore[arg-type]
+                )
+                return HTTPResponse(
+                    status=requests.codes.ok,
+                    body=body,
+                    headers=headers,
+                    original_response=orig_response,  # type: ignore[arg-type]
+                    request_method=request.method,
+                )
+
+        debusine_client = DebusineHttpClient(self.api_url, self.token)
+        responses.add(EmptyResponse("PUT", self.api_url))
+
+        debusine_client._api_request("PUT", self.api_url, None)
+
     @responses.activate
     def test_debusine_error_details_combinations_everything(self) -> None:
         """Test combinations of status and body for _debusine_error_details."""
@@ -542,7 +578,7 @@ class DebusineHttpClientTests(TestCase):
                     f"^Cannot connect to {self.api_url}. Error: $",
                 ):
                     debusine_client._api_request(
-                        "GET", self.api_url, TestResponse
+                        "GET", self.api_url, SampleResponse
                     )
 
     @responses.activate
@@ -553,7 +589,7 @@ class DebusineHttpClientTests(TestCase):
             self.api_url, length=4, page_size=3
         )
 
-        iter_ = debusine_client.iter_paginated_get("", TestResponse)
+        iter_ = debusine_client.iter_paginated_get("", SampleResponse)
         for i in range(4):
             obj = next(iter_)
             self.assertEqual(obj.id, i)
@@ -567,14 +603,14 @@ class DebusineHttpClientTests(TestCase):
             self.api_url, length=4, page_size=3
         )
 
-        response = list(debusine_client.iter_paginated_get("", TestResponse))
+        response = list(debusine_client.iter_paginated_get("", SampleResponse))
         self.assertEqual(
             response,
             [
-                TestResponse(id=0, page=1),
-                TestResponse(id=1, page=1),
-                TestResponse(id=2, page=1),
-                TestResponse(id=3, page=2),
+                SampleResponse(id=0, page=1),
+                SampleResponse(id=1, page=1),
+                SampleResponse(id=2, page=1),
+                SampleResponse(id=3, page=2),
             ],
         )
 
@@ -586,7 +622,7 @@ class DebusineHttpClientTests(TestCase):
             self.api_url, length=0, page_size=3
         )
 
-        response = list(debusine_client.iter_paginated_get("", TestResponse))
+        response = list(debusine_client.iter_paginated_get("", SampleResponse))
         self.assertEqual(response, [])
 
     @responses.activate
@@ -602,7 +638,7 @@ class DebusineHttpClientTests(TestCase):
             exceptions.UnexpectedResponseError,
             fr"^Server \({self.api_url}\) did not return valid object. Error: ",
         ):
-            next(debusine_client.iter_paginated_get("", TestResponse))
+            next(debusine_client.iter_paginated_get("", SampleResponse))
 
     @responses.activate
     def test_iter_paginated_get_invalid_response(self) -> None:
@@ -620,4 +656,4 @@ class DebusineHttpClientTests(TestCase):
             exceptions.UnexpectedResponseError,
             fr"^Server \({self.api_url}\) did not return valid object. Error: ",
         ):
-            next(debusine_client.iter_paginated_get("", TestResponse))
+            next(debusine_client.iter_paginated_get("", SampleResponse))
