@@ -26,6 +26,7 @@ from debusine.artifacts.models import ArtifactCategory, ArtifactData
 from debusine.db.context import context
 from debusine.db.models.files import File
 from debusine.db.models.permissions import (
+    Allow,
     PermissionUser,
     permission_check,
     permission_filter,
@@ -88,7 +89,7 @@ class ArtifactQuerySet(QuerySet["Artifact", A], Generic[A]):
         """Filter to artifacts in the current workspace."""
         return self.filter(workspace=context.require_workspace())
 
-    @permission_filter
+    @permission_filter(workers=Allow.ALWAYS, anonymous=Allow.PASS)
     def can_display(self, user: PermissionUser) -> "ArtifactQuerySet[A]":
         """Keep only Artifacts that can be displayed."""
         # Delegate to workspace can_display check
@@ -107,6 +108,77 @@ class ArtifactQuerySet(QuerySet["Artifact", A], Generic[A]):
             )
         )
 
+    def not_expired(self, at: datetime) -> "ArtifactQuerySet[Any]":
+        """
+        Return queryset with artifacts that have not expired.
+
+        :param at: datetime to check if the artifacts are not expired.
+        :return: artifacts that expire_at is None (do not expire) or
+          expire_at is after the given datetime.
+        """
+        return self.annotate(
+            effective_expiration_delay=Coalesce(
+                "expiration_delay",
+                "workspace__default_expiration_delay",
+            )
+        ).filter(
+            Q(effective_expiration_delay=timedelta(0))
+            | Q(
+                # https://github.com/typeddjango/django-stubs/issues/1548
+                created_at__gt=(
+                    at - F("effective_expiration_delay")  # type: ignore[operator] # noqa: E501
+                )
+            )
+        )
+
+    def expired(self, at: datetime) -> "ArtifactQuerySet[Any]":
+        """
+        Return queryset with artifacts that have expired.
+
+        :param at: datetime to check if the artifacts are expired.
+        :return: artifacts that expire_at is before the given datetime.
+        """
+        return (
+            self.annotate(
+                effective_expiration_delay=Coalesce(
+                    "expiration_delay",
+                    "workspace__default_expiration_delay",
+                )
+            )
+            .exclude(effective_expiration_delay=timedelta(0))
+            .filter(
+                # https://github.com/typeddjango/django-stubs/issues/1548
+                created_at__lte=(
+                    at - F("effective_expiration_delay")  # type: ignore[operator] # noqa: E501
+                )
+            )
+        )
+
+    def part_of_collection_with_retains_artifacts(
+        self,
+    ) -> "ArtifactQuerySet[Any]":
+        """
+        Return Artifacts where the parent_collections has retains_artifacts set.
+
+        :return: Artifacts that are part of a retains_artifacts collection.
+        """
+        # Import here to prevent circular imports
+        from debusine.db.models.collections import Collection
+        from debusine.db.models.work_requests import WorkRequest
+
+        RetainsArtifacts = Collection.RetainsArtifacts
+        return self.filter(
+            Q(parent_collections__retains_artifacts=RetainsArtifacts.ALWAYS)
+            | Q(
+                parent_collections__retains_artifacts=RetainsArtifacts.WORKFLOW,
+                parent_collections__workflow__status__in={
+                    WorkRequest.Statuses.PENDING,
+                    WorkRequest.Statuses.RUNNING,
+                    WorkRequest.Statuses.BLOCKED,
+                },
+            )
+        )
+
 
 class ArtifactManager(models.Manager["Artifact"]):
     """Manager for the Artifact model."""
@@ -123,7 +195,7 @@ class ArtifactManager(models.Manager["Artifact"]):
         *,
         created_by_work_request: Optional["WorkRequest"] = None,
     ) -> "Artifact":
-        """Return a new Artifact based on a :class:`LocalArtifact`."""
+        """Return a new Artifact based on a :py:class:`LocalArtifact`."""
         artifact = Artifact.objects.create(
             category=local_artifact.category,
             workspace=workspace,
@@ -136,85 +208,15 @@ class ArtifactManager(models.Manager["Artifact"]):
             file_backend = workspace.scope.upload_file_backend(file)
             file_backend.add_file(local_path, fileobj=file)
             FileInArtifact.objects.create(
-                artifact=artifact, path=artifact_path, file=file, complete=True
+                artifact=artifact,
+                path=artifact_path,
+                file=file,
+                complete=True,
+                content_type=local_artifact.content_types.get(artifact_path),
             )
 
         return artifact
 
-    def not_expired(self, at: datetime) -> QuerySet["Artifact"]:
-        """
-        Return queryset with artifacts that have not expired.
-
-        :param at: datetime to check if the artifacts are not expired.
-        :return: artifacts that expire_at is None (do not expire) or
-          expire_at is after the given datetime.
-        """
-        return (
-            self.get_queryset()
-            .annotate(
-                effective_expiration_delay=Coalesce(
-                    "expiration_delay",
-                    "workspace__default_expiration_delay",
-                )
-            )
-            .filter(
-                Q(effective_expiration_delay=timedelta(0))
-                | Q(
-                    # https://github.com/typeddjango/django-stubs/issues/1548
-                    created_at__gt=(
-                        at - F("effective_expiration_delay")  # type: ignore[operator] # noqa: E501
-                    )
-                )
-            )
-        )
-
-    def expired(self, at: datetime) -> QuerySet["Artifact"]:
-        """
-        Return queryset with artifacts that have expired.
-
-        :param at: datetime to check if the artifacts are expired.
-        :return: artifacts that expire_at is before the given datetime.
-        """
-        return (
-            self.get_queryset()
-            .annotate(
-                effective_expiration_delay=Coalesce(
-                    "expiration_delay",
-                    "workspace__default_expiration_delay",
-                )
-            )
-            .exclude(effective_expiration_delay=timedelta(0))
-            .filter(
-                # https://github.com/typeddjango/django-stubs/issues/1548
-                created_at__lte=(
-                    at - F("effective_expiration_delay")  # type: ignore[operator] # noqa: E501
-                )
-            )
-        )
-
-    def part_of_collection_with_retains_artifacts(self) -> QuerySet["Artifact"]:
-        """
-        Return Artifacts where the parent_collections has retains_artifacts set.
-
-        :return: Artifacts that are part of a retains_artifacts collection.
-        """
-        # Import here to prevent circular imports
-        from debusine.db.models.collections import Collection
-        from debusine.db.models.work_requests import WorkRequest
-
-        RetainsArtifacts = Collection.RetainsArtifacts
-        return self.get_queryset().filter(
-            Q(parent_collections__retains_artifacts=RetainsArtifacts.ALWAYS)
-            | Q(
-                parent_collections__retains_artifacts=RetainsArtifacts.WORKFLOW,
-                parent_collections__workflow__status__in={
-                    WorkRequest.Statuses.PENDING,
-                    WorkRequest.Statuses.RUNNING,
-                    WorkRequest.Statuses.BLOCKED,
-                },
-            )
-        )
-
 
 class Artifact(models.Model):
     """Artifact model."""
@@ -260,7 +262,11 @@ class Artifact(models.Model):
                 {"data": f"invalid artifact data: {e}"}
             ) from e
 
-    @permission_check("{user} cannot display artifact {resource}")
+    @permission_check(
+        "{user} cannot display artifact {resource}",
+        workers=Allow.ALWAYS,
+        anonymous=Allow.PASS,
+    )
     def can_display(self, user: PermissionUser) -> bool:
         """Check if the artifact can be displayed."""
         return self.workspace.can_display(user)
@@ -333,17 +339,23 @@ class Artifact(models.Model):
 
     def get_absolute_url(self) -> str:
         """Return the canonical URL to display the artifact."""
-        return reverse(
-            "workspaces:artifacts:detail",
-            kwargs={"wname": self.workspace.name, "artifact_id": self.id},
-        )
+        from debusine.server.scopes import urlconf_scope
+
+        with urlconf_scope(self.workspace.scope.name):
+            return reverse(
+                "workspaces:artifacts:detail",
+                kwargs={"wname": self.workspace.name, "artifact_id": self.id},
+            )
 
     def get_absolute_url_download(self) -> str:
         """Return the canonical URL to download the artifact."""
-        return reverse(
-            "workspaces:artifacts:download",
-            kwargs={"wname": self.workspace.name, "artifact_id": self.id},
-        )
+        from debusine.server.scopes import urlconf_scope
+
+        with urlconf_scope(self.workspace.scope.name):
+            return reverse(
+                "workspaces:artifacts:download",
+                kwargs={"wname": self.workspace.name, "artifact_id": self.id},
+            )
 
     def __str__(self) -> str:
         """Return basic information of Artifact."""
@@ -366,6 +378,7 @@ class FileInArtifact(models.Model):
     # added.  (debusine.debian.net only had a single workspace at the time
     # of this migration, so that's not a problem there.)
     complete = models.BooleanField(default=False)
+    content_type = models.CharField(blank=True, null=True)
 
     class Meta(TypedModelMeta):
         constraints = [
@@ -384,27 +397,45 @@ class FileInArtifact(models.Model):
 
     def get_absolute_url(self) -> str:
         """Return an absolute URL to view this file."""
-        return reverse(
-            "workspaces:artifacts:detail-file",
-            kwargs={
-                "wname": self.artifact.workspace.name,
-                "artifact_id": self.artifact.id,
-                "file_in_artifact_id": self.id,
-                "path": self.path,
-            },
-        )
+        from debusine.server.scopes import urlconf_scope
+
+        with urlconf_scope(self.artifact.workspace.scope.name):
+            return reverse(
+                "workspaces:artifacts:file-detail",
+                kwargs={
+                    "wname": self.artifact.workspace.name,
+                    "artifact_id": self.artifact.id,
+                    "path": self.path,
+                },
+            )
 
     def get_absolute_url_raw(self) -> str:
         """Return an absolute URL to view this file as raw."""
-        return reverse(
-            "workspaces:artifacts:detail-file-raw",
-            kwargs={
-                "wname": self.artifact.workspace.name,
-                "artifact_id": self.artifact.id,
-                "file_in_artifact_id": self.id,
-                "path": self.path,
-            },
-        )
+        from debusine.server.scopes import urlconf_scope
+
+        with urlconf_scope(self.artifact.workspace.scope.name):
+            return reverse(
+                "workspaces:artifacts:file-raw",
+                kwargs={
+                    "wname": self.artifact.workspace.name,
+                    "artifact_id": self.artifact.id,
+                    "path": self.path,
+                },
+            )
+
+    def get_absolute_url_download(self) -> str:
+        """Return an absolute URL to download this file."""
+        from debusine.server.scopes import urlconf_scope
+
+        with urlconf_scope(self.artifact.workspace.scope.name):
+            return reverse(
+                "workspaces:artifacts:file-download",
+                kwargs={
+                    "wname": self.artifact.workspace.name,
+                    "artifact_id": self.artifact.id,
+                    "path": self.path,
+                },
+            )
 
 
 class FileUpload(models.Model):
@@ -500,7 +531,7 @@ class ArtifactRelationQuerySet(QuerySet["ArtifactRelation", A], Generic[A]):
             artifact__workspace__scope=scope, target__workspace__scope=scope
         )
 
-    @permission_filter
+    @permission_filter(workers=Allow.ALWAYS, anonymous=Allow.PASS)
     def can_display(
         self, user: PermissionUser
     ) -> "ArtifactRelationQuerySet[A]":
@@ -550,7 +581,11 @@ class ArtifactRelation(models.Model):
             )
         ]
 
-    @permission_check("{user} cannot display artifact relation {resource}")
+    @permission_check(
+        "{user} cannot display artifact relation {resource}",
+        workers=Allow.ALWAYS,
+        anonymous=Allow.PASS,
+    )
     def can_display(self, user: PermissionUser) -> bool:
         """Check if the artifact can be displayed."""
         return self.artifact.workspace.can_display(
