@@ -15,14 +15,16 @@ from collections.abc import Mapping
 from pathlib import Path
 from typing import Any, TYPE_CHECKING
 
+import pgtrigger
 from django.core.exceptions import ValidationError
 from django.db import models
-from django.db.models import CheckConstraint, Q, UniqueConstraint
+from django.db.models import CheckConstraint, F, Q, UniqueConstraint
 
 from debusine.server.file_backend.models import (
     ExternalDebianSuiteBackendConfiguration,
     LocalFileBackendConfiguration,
     MemoryFileBackendConfiguration,
+    S3FileBackendConfiguration,
 )
 from debusine.utils import calculate_hash
 
@@ -131,6 +133,7 @@ class _FileStoreBackendChoices(models.TextChoices):
     LOCAL = "Local", "Local"
     EXTERNAL_DEBIAN_SUITE = "ExternalDebianSuite", "ExternalDebianSuite"
     MEMORY = "Memory", "Memory"
+    S3 = "S3", "S3"
 
 
 DEFAULT_FILE_STORE_NAME = "Default"
@@ -159,12 +162,46 @@ class FileStore(models.Model):
             ExternalDebianSuiteBackendConfiguration
         ),
         BackendChoices.MEMORY: MemoryFileBackendConfiguration,
+        BackendChoices.S3: S3FileBackendConfiguration,
     }
 
     name = models.CharField(max_length=255, unique=True)
     backend = models.CharField(max_length=255, choices=BackendChoices.choices)
     configuration = models.JSONField(default=dict, blank=True)
     files = models.ManyToManyField(File, through="db.FileInStore")
+    instance_wide = models.BooleanField(default=True)
+    total_size = models.BigIntegerField(default=0, editable=False)
+    soft_max_size = models.BigIntegerField(blank=True, null=True)
+    max_size = models.BigIntegerField(blank=True, null=True)
+    provider_account = models.ForeignKey(
+        "Asset", blank=True, null=True, on_delete=models.PROTECT
+    )
+
+    class Meta(TypedModelMeta):
+        triggers = [
+            pgtrigger.Trigger(
+                name="db_filestore_sync_instance_wide",
+                operation=pgtrigger.Insert | pgtrigger.Update,
+                when=pgtrigger.After,
+                func=" ".join(
+                    """
+                    UPDATE db_filestoreinscope
+                        SET file_store_instance_wide = NEW.instance_wide
+                        WHERE
+                            db_filestoreinscope.file_store_id = NEW.id
+                            AND db_filestoreinscope.file_store_instance_wide
+                                != NEW.instance_wide;
+                    RETURN NEW;
+                    """.split()
+                ),
+            )
+        ]
+        constraints = [
+            CheckConstraint(
+                name="%(app_label)s_%(class)s_consistent_max_sizes",
+                check=~Q(soft_max_size__gt=F("max_size")),
+            )
+        ]
 
     @staticmethod
     def default() -> "FileStore":
@@ -214,6 +251,10 @@ class FileStore(models.Model):
                 )
 
                 return ExternalDebianSuiteFileBackend(self)
+            case _FileStoreBackendChoices.S3:
+                from debusine.server.file_backend.s3 import S3FileBackend
+
+                return S3FileBackend(self)
             case _ as unreachable:
                 raise NotImplementedError(
                     f"backend {unreachable!r} not supported"
@@ -232,6 +273,42 @@ class FileInStore(models.Model):
     data = models.JSONField(default=dict, blank=True)
 
     class Meta(TypedModelMeta):
+        triggers = [
+            # Maintain FileStore.total_size: adding a file to a store
+            # increases it, and removing a file from a store decreases it.
+            pgtrigger.Trigger(
+                name="db_fileinstore_total_size_insert",
+                operation=pgtrigger.Insert,
+                when=pgtrigger.After,
+                func=" ".join(
+                    """
+                    UPDATE db_filestore
+                        SET total_size = total_size + db_file.size
+                        FROM db_file
+                        WHERE
+                            db_filestore.id = NEW.store_id
+                            AND db_file.id = NEW.file_id;
+                    RETURN NULL;
+                    """.split()
+                ),
+            ),
+            pgtrigger.Trigger(
+                name="db_fileinstore_total_size_delete",
+                operation=pgtrigger.Delete,
+                when=pgtrigger.After,
+                func=" ".join(
+                    """
+                    UPDATE db_filestore
+                        SET total_size = total_size - db_file.size
+                        FROM db_file
+                        WHERE
+                            db_filestore.id = OLD.store_id
+                            AND db_file.id = OLD.file_id;
+                    RETURN NULL;
+                    """.split()
+                ),
+            ),
+        ]
         constraints = [
             UniqueConstraint(
                 fields=["store", "file"],
