@@ -9,15 +9,22 @@
 
 """Unit tests for auth-related models."""
 
+import contextlib
+import re
+from collections.abc import Generator
 from datetime import timedelta
 from typing import ClassVar
+from unittest import mock
 
 from asgiref.sync import sync_to_async
 from django.contrib.auth import get_user_model
 from django.contrib.auth.models import AnonymousUser
+from django.core.exceptions import ImproperlyConfigured
 from django.db import IntegrityError
 from django.utils import timezone
 
+import debusine.db.models.auth as auth
+from debusine.db.context import ContextConsistencyError, context
 from debusine.db.models import (
     Group,
     Identity,
@@ -27,7 +34,15 @@ from debusine.db.models import (
     Worker,
     Workspace,
 )
-from debusine.db.models.auth import GroupMembership, system_user
+from debusine.db.models.auth import (
+    ClientEnroll,
+    GroupAuditLog,
+    GroupMembership,
+    GroupRoleBase,
+    GroupRoles,
+    system_user,
+)
+from debusine.db.models.tests.utils import RolesTestCase
 from debusine.db.playground import scenarios
 from debusine.test.django import ChannelsHelpersMixin, TestCase
 
@@ -213,6 +228,16 @@ class TokenManagerTests(TestCase):
             [],
         )
 
+    def test_get_tokens_include_expired(self) -> None:
+        """get_tokens only includes expired tokens if requested."""
+        self.token_john.expire_at = timezone.now() - timedelta(seconds=1)
+        self.token_john.save()
+        self.assertQuerySetEqual(Token.objects.get_tokens(username="John"), [])
+        self.assertQuerySetEqual(
+            Token.objects.get_tokens(username="John", include_expired=True),
+            [self.token_john],
+        )
+
     def test_expired(self) -> None:
         """``expired`` returns only the expired tokens."""
         now = timezone.now()
@@ -307,9 +332,59 @@ class IdentityTests(TestCase):
         self.assertEqual(str(ident), "salsa:test@debian.org")
 
 
+class GroupRolesTests(RolesTestCase[GroupRoles]):
+    """Tests for the GroupRoles class."""
+
+    scenario = scenarios.DefaultContext()
+    roles_class = GroupRoles
+
+    def test_invariants(self) -> None:
+        self.assertRolesInvariants()
+
+    def test_choices(self) -> None:
+        self.assertChoices()
+
+    def test_init(self) -> None:
+        r = GroupRoleBase("test")
+        r._setup()
+        self.assertEqual(r.implied_by_scope_roles, frozenset())
+        self.assertEqual(r.implied_by_group_roles, frozenset([r]))
+
+    def test_init_invalid_implication(self) -> None:
+        r = GroupRoleBase("test", implied_by=[Workspace.Roles.OWNER])
+        with self.assertRaisesRegex(
+            ImproperlyConfigured,
+            r"Group roles do not support implications"
+            r" by <enum 'WorkspaceRoles'>",
+        ):
+            r._setup()
+
+    def test_values(self) -> None:
+        self.assertEqual(Group.Roles.ADMIN.label, "Admin")
+        self.assertEqual(
+            Group.Roles.ADMIN.implied_by_scope_roles,
+            frozenset([Scope.Roles.OWNER]),
+        )
+        self.assertEqual(
+            Group.Roles.ADMIN.implied_by_group_roles,
+            frozenset([Group.Roles.ADMIN]),
+        )
+
+        self.assertEqual(Group.Roles.MEMBER.label, "Member")
+        self.assertEqual(
+            Group.Roles.MEMBER.implied_by_scope_roles,
+            frozenset([Scope.Roles.OWNER]),
+        )
+        self.assertEqual(
+            Group.Roles.MEMBER.implied_by_group_roles,
+            frozenset([Group.Roles.ADMIN, Group.Roles.MEMBER]),
+        )
+
+
 class GroupManagerTests(TestCase):
     """Test for GroupManager class."""
 
+    scenario = scenarios.DefaultScopeUser()
     scope1: ClassVar[Scope]
     scope2: ClassVar[Scope]
     group1: ClassVar[Group]
@@ -344,6 +419,7 @@ class GroupManagerTests(TestCase):
 
     def test_create_ephemeral(self) -> None:
         """Test create_ephemeral."""
+        self.scenario.set_current()
         user = self.playground.get_default_user()
         group = Group.objects.create_ephemeral(
             scope=self.scope1, name="test", owner=user
@@ -355,20 +431,42 @@ class GroupManagerTests(TestCase):
         membership = GroupMembership.objects.get(group=group, user=user)
         self.assertEqual(membership.role, Group.Roles.ADMIN)
 
+        audit_logs = list(GroupAuditLog.objects.order_by("created_at"))
+        self.assertEqual(len(audit_logs), 2)
+        # Group creation audit log
+        audit_log = audit_logs[0]
+        self.assertEqual(audit_log.group, group)
+        self.assertEqual(audit_log.actor, self.scenario.user)
+        changes_add = auth.GroupAuditLogCreated.create(group)
+        changes_add.actor = self.scenario.user.username
+        self.assertEqual(audit_log.changes, changes_add.dict())
+        # Admin added audit log
+        audit_log = audit_logs[1]
+        self.assertEqual(audit_log.group, group)
+        self.assertEqual(audit_log.actor, self.scenario.user)
+        changes_memb = auth.GroupAuditLogMemberAdded(
+            user=self.scenario.user.username,
+            role=Group.Roles.ADMIN,
+            actor=self.scenario.user.username,
+        )
+        self.assertEqual(audit_log.changes, changes_memb.dict())
+
     def test_unused_ephemeral_unused(self) -> None:
         """Test unused_ephemeral."""
         self.assertQuerySetEqual(Group.objects.unused_ephemeral(), [])
 
-        self.group1.ephemeral = True
-        self.group1.save()
+        with context.disable_permission_checks():
+            self.group1.ephemeral = True
+            self.group1.save()
         self.assertQuerySetEqual(
             Group.objects.unused_ephemeral(), [self.group1]
         )
 
     def test_unused_ephemeral_used_by_scope(self) -> None:
         """Test unused_ephemeral for a group used by a Scope."""
-        self.group1.ephemeral = True
-        self.group1.save()
+        with context.disable_permission_checks():
+            self.group1.ephemeral = True
+            self.group1.save()
         self.assertQuerySetEqual(
             Group.objects.unused_ephemeral(), [self.group1]
         )
@@ -378,8 +476,9 @@ class GroupManagerTests(TestCase):
 
     def test_unused_ephemeral_used_by_workspace(self) -> None:
         """Test unused_ephemeral for a group used by a Workspace."""
-        self.group1.ephemeral = True
-        self.group1.save()
+        with context.disable_permission_checks():
+            self.group1.ephemeral = True
+            self.group1.save()
         self.assertQuerySetEqual(
             Group.objects.unused_ephemeral(), [self.group1]
         )
@@ -390,8 +489,9 @@ class GroupManagerTests(TestCase):
 
     def test_unused_ephemeral_used_by_asset(self) -> None:
         """Test unused_ephemeral for a group used by an Asset."""
-        self.group1.ephemeral = True
-        self.group1.save()
+        with context.disable_permission_checks():
+            self.group1.ephemeral = True
+            self.group1.save()
         self.assertQuerySetEqual(
             Group.objects.unused_ephemeral(), [self.group1]
         )
@@ -403,8 +503,9 @@ class GroupManagerTests(TestCase):
 
     def test_unused_ephemeral_used_by_asset_usage(self) -> None:
         """Test unused_ephemeral for a group used by an AssetUsage."""
-        self.group1.ephemeral = True
-        self.group1.save()
+        with context.disable_permission_checks():
+            self.group1.ephemeral = True
+            self.group1.save()
         self.assertQuerySetEqual(
             Group.objects.unused_ephemeral(), [self.group1]
         )
@@ -463,6 +564,24 @@ class GroupTests(TestCase):
             else:
                 self.assertEqual(membership.role, role)
 
+    @contextlib.contextmanager
+    def assertGroupAuditLog(
+        self,
+    ) -> Generator[list[auth.GroupAuditLogEntry], None, None]:
+        """
+        Ensure add_audit_log is called.
+
+        Generates a list populated with the messages logged.
+        """
+        changes: list[auth.GroupAuditLogEntry] = []
+        with mock.patch(
+            "debusine.db.models.auth.Group.add_audit_log"
+        ) as add_audit_log:
+            yield changes
+        add_audit_log.assert_called()
+        for call in add_audit_log.call_args_list:
+            changes.append(call.args[0])
+
     def test_assertRole(self) -> None:
         """Test assertRole."""
         self.scenario.set_current()
@@ -474,7 +593,7 @@ class GroupTests(TestCase):
         self.group.add_user(self.scenario.user, Group.Roles.ADMIN)
         self.assertRole(self.group, self.scenario.user, Group.Roles.ADMIN)
         with self.assertRaisesRegex(
-            AssertionError, "'admin' != GroupRoles.MEMBER"
+            AssertionError, re.escape(f"'admin' != {Group.Roles.MEMBER!r}")
         ):
             self.assertRole(self.group, self.scenario.user, Group.Roles.MEMBER)
         with self.assertRaisesRegex(
@@ -588,7 +707,10 @@ class GroupTests(TestCase):
         """Try assigning a role using the wrong enum."""
         with self.assertRaisesRegex(
             TypeError,
-            r"ScopeRoles.OWNER cannot be converted to <enum 'WorkspaceRoles'>",
+            re.escape(
+                f"{Scope.Roles.OWNER!r} cannot be converted to"
+                " <enum 'WorkspaceRoles'>"
+            ),
         ):
             self.group.assign_role(self.scenario.workspace, Scope.Roles.OWNER)
 
@@ -626,18 +748,38 @@ class GroupTests(TestCase):
 
     def test_set_user_role(self) -> None:
         """Set the role of a user on a group."""
+        self.scenario.set_current()
         self.group.add_user(self.scenario.user)
         self.assertEqual(
             self.group.get_user_role(self.scenario.user), Group.Roles.MEMBER
         )
-        self.group.set_user_role(self.scenario.user, Group.Roles.ADMIN)
+        with self.assertGroupAuditLog() as changes:
+            self.group.set_user_role(self.scenario.user, Group.Roles.ADMIN)
         self.assertEqual(
             self.group.get_user_role(self.scenario.user), Group.Roles.ADMIN
         )
-        self.group.set_user_role(self.scenario.user, Group.Roles.MEMBER)
+        self.assertEqual(
+            changes,
+            [
+                auth.GroupAuditLogMemberRoleChanged(
+                    user=self.scenario.user.username, role=Group.Roles.ADMIN
+                )
+            ],
+        )
+
+        with self.assertGroupAuditLog() as changes:
+            self.group.set_user_role(self.scenario.user, Group.Roles.MEMBER)
         self.assertEqual(
             self.group.get_user_role(self.scenario.user), Group.Roles.MEMBER
         )
+        self.assertEqual(
+            changes,
+            [
+                auth.GroupAuditLogMemberRoleChanged(
+                    user=self.scenario.user.username, role=Group.Roles.MEMBER
+                )
+            ],
+        )
 
     def test_set_user_role_not_a_member(self) -> None:
         """Set the role of a user not a member of the group."""
@@ -650,17 +792,36 @@ class GroupTests(TestCase):
     def test_add_user(self) -> None:
         """Add a nonexisting user to a group."""
         self.assertRole(self.group, self.scenario.user, None)
-        self.assertIsInstance(
-            self.group.add_user(self.scenario.user), GroupMembership
-        )
+        with self.assertGroupAuditLog() as changes:
+            self.assertIsInstance(
+                self.group.add_user(self.scenario.user), GroupMembership
+            )
         self.assertRole(self.group, self.scenario.user, Group.Roles.MEMBER)
+        self.assertEqual(
+            changes,
+            [
+                auth.GroupAuditLogMemberAdded(
+                    user=self.scenario.user.username, role=Group.Roles.MEMBER
+                )
+            ],
+        )
 
     def test_add_user_role(self) -> None:
         """Add a user to a group with a custom role."""
         self.assertRole(self.group, self.scenario.user, None)
-        self.group.add_user(self.scenario.user, Group.Roles.ADMIN)
+        with self.assertGroupAuditLog() as changes:
+            self.group.add_user(self.scenario.user, Group.Roles.ADMIN)
         self.assertRole(self.group, self.scenario.user, Group.Roles.ADMIN)
 
+        self.assertEqual(
+            changes,
+            [
+                auth.GroupAuditLogMemberAdded(
+                    user=self.scenario.user.username, role=Group.Roles.ADMIN
+                )
+            ],
+        )
+
     def test_add_user_already_member(self) -> None:
         """Cannot add a user that is already a member."""
         self.scenario.set_current()
@@ -675,10 +836,15 @@ class GroupTests(TestCase):
 
     def test_remove_user(self) -> None:
         """Remove a user from a group."""
-        self.group.add_user(self.scenario.user)
+        self.playground.add_user(self.group, self.scenario.user)
         self.assertRole(self.group, self.scenario.user, Group.Roles.MEMBER)
-        self.group.remove_user(self.scenario.user)
+        with self.assertGroupAuditLog() as changes:
+            self.group.remove_user(self.scenario.user)
         self.assertRole(self.group, self.scenario.user, None)
+        self.assertEqual(
+            changes,
+            [auth.GroupAuditLogMemberRemoved(user=self.scenario.user.username)],
+        )
 
     def test_remove_user_not_member(self) -> None:
         """Removing a user not in the group fails."""
@@ -690,6 +856,34 @@ class GroupTests(TestCase):
             self.group.remove_user(self.scenario.user)
         self.assertRole(self.group, self.scenario.user, None)
 
+    def test_add_audit_log_no_user(self) -> None:
+        """Ensure add_audit_log fails if there is no user in context."""
+        with self.assertRaisesRegex(
+            ContextConsistencyError, r"add_audit_log requires a valid user"
+        ):
+            self.group.add_audit_log(
+                auth.GroupAuditLogMemberAdded(
+                    user="test", role=Group.Roles.MEMBER
+                )
+            )
+
+    def test_add_audit_log(self) -> None:
+        """Ensure add_audit_log fails if there is no user in context."""
+        self.scenario.set_current()
+        changes = auth.GroupAuditLogMemberAdded(
+            user=self.scenario.user.username, role=Group.Roles.ADMIN
+        )
+        entry = self.group.add_audit_log(changes.copy())
+        assert entry is not None
+        changes.actor = self.scenario.user.username
+        self.assertEqual(entry.group, self.group)
+        self.assertEqual(entry.actor, self.scenario.user)
+        self.assertEqual(entry.changes, changes.dict())
+        self.assertIsInstance(
+            entry.changes_model, auth.GroupAuditLogMemberAdded
+        )
+        self.assertEqual(entry.changes_model, changes)
+
     def test_can_display_anonymous(self) -> None:
         """Deny can_display for anonymous users."""
         self.assertPermission(
@@ -706,7 +900,7 @@ class GroupTests(TestCase):
 
         self.assertPermission(
             "can_display",
-            users=(None, AnonymousUser(), self.scenario.user),
+            users=[None],
             denied=[self.group, self.other_group],
             token=self.playground.create_worker_token(),
         )
@@ -730,6 +924,48 @@ class GroupTests(TestCase):
             allowed=[self.group, owner_group, self.other_group],
         )
 
+    def test_can_display_audit_log_anonymous(self) -> None:
+        """Deny can_display_audit_log for anonymous users."""
+        self.assertPermission(
+            "can_display_audit_log",
+            users=AnonymousUser(),
+            denied=[self.group, self.other_group],
+        )
+
+    def test_can_display_audit_log_workers(self) -> None:
+        """Deny can_display_audit_log for workers."""
+        GroupMembership.objects.create(
+            group=self.group, user=self.scenario.user, role=Group.Roles.ADMIN
+        )
+
+        self.assertPermission(
+            "can_display_audit_log",
+            users=[None],
+            denied=[self.group, self.other_group],
+            token=self.playground.create_worker_token(),
+        )
+
+    def test_can_display_audit_log(self) -> None:
+        """can_display_audit_log is True for any user."""
+        owner_group = self.playground.create_group_role(
+            self.scenario.scope, "owner", [self.scenario.user]
+        )
+        admin = self.playground.create_user("admin")
+        GroupMembership.objects.create(
+            group=self.group, user=admin, role=Group.Roles.ADMIN
+        )
+
+        member = self.playground.create_user("member")
+        self.group.users.add(member)
+
+        other = self.playground.create_user("other")
+
+        self.assertPermission(
+            "can_display_audit_log",
+            users=[self.scenario.user, admin, member, other],
+            allowed=[self.group, owner_group, self.other_group],
+        )
+
     def test_can_manage_anonymous(self) -> None:
         """Deny can_manage for anonymous users."""
         self.assertPermission(
@@ -746,7 +982,7 @@ class GroupTests(TestCase):
 
         self.assertPermission(
             "can_manage",
-            users=(None, AnonymousUser(), self.scenario.user),
+            users=[None],
             denied=[self.group, self.other_group],
             token=self.playground.create_worker_token(),
         )
@@ -805,3 +1041,129 @@ class GroupMembershipTests(TestCase):
     def test_str(self) -> None:
         """Test stringification."""
         self.assertEqual(str(self.membership), "debusine/group:playground")
+
+
+class GroupAuditLogEntryTests(TestCase):
+    """Test GroupAuditLogEntry models."""
+
+    scenario = scenarios.DefaultScopeUser()
+
+    def test_created_create(self) -> None:
+        """Test GroupAuditLogCreated.create."""
+        group = Group(name="test", scope=self.scenario.scope, ephemeral=True)
+        entry = auth.GroupAuditLogCreated.create(group)
+        self.assertEqual(
+            entry,
+            auth.GroupAuditLogCreated(
+                name="test",
+                scope="debusine",
+                ephemeral=True,
+            ),
+        )
+
+    def test_created_description(self) -> None:
+        """Test GroupAuditLogCreated.description."""
+        for group, description in (
+            (
+                Group(name="test", scope=self.scenario.scope, ephemeral=True),
+                "actor created ephemeral group test in scope debusine",
+            ),
+            (
+                Group(name="test", scope=self.scenario.scope, ephemeral=False),
+                "actor created group test in scope debusine",
+            ),
+        ):
+            with self.subTest(group=group):
+                entry = auth.GroupAuditLogCreated.create(group)
+                entry.actor = "actor"
+                self.assertEqual(
+                    entry.description(),
+                    description,
+                )
+
+    def test_updated_create_diff(self) -> None:
+        """Test GroupAuditLogUpdated.create_diff."""
+        scope1 = self.playground.get_or_create_scope("scope1")
+        group1 = Group(name="test", scope=self.scenario.scope, ephemeral=False)
+        group2 = Group(name="test1", scope=scope1, ephemeral=True)
+        entry = auth.GroupAuditLogUpdated.create_diff(group1, group2)
+        self.assertEqual(
+            entry,
+            auth.GroupAuditLogUpdated(
+                name=("test", "test1"),
+                scope=("debusine", "scope1"),
+                ephemeral=(False, True),
+            ),
+        )
+
+    def test_updated_description(self) -> None:
+        """Test GroupAuditLogUpdated.description."""
+        group = Group(name="test", scope=self.scenario.scope, ephemeral=False)
+        scope1 = self.playground.get_or_create_scope("scope1")
+        for other_group, expected in (
+            (
+                Group(name="test1", scope=scope1, ephemeral=True),
+                "actor updated name: 'test'→'test1';"
+                " scope: 'debusine'→'scope1';"
+                " ephemeral: False→True",
+            ),
+            (
+                Group(name="test1", scope=self.scenario.scope, ephemeral=True),
+                "actor updated name: 'test'→'test1'; ephemeral: False→True",
+            ),
+            (
+                Group(name="test", scope=self.scenario.scope, ephemeral=True),
+                "actor updated ephemeral: False→True",
+            ),
+            (
+                group,
+                "actor updated group, no changes detected",
+            ),
+        ):
+            with self.subTest(other_group=other_group):
+                entry = auth.GroupAuditLogUpdated.create_diff(
+                    group, other_group
+                )
+                entry.actor = "actor"
+                self.assertEqual(entry.description(), expected)
+
+    def test_member_added_description(self) -> None:
+        """Test GroupAuditLogMemberAdded.description."""
+        entry = auth.GroupAuditLogMemberAdded(
+            actor="actor", user="test", role=Group.Roles.ADMIN
+        )
+        self.assertEqual(entry.description(), "actor added test as admin")
+
+    def test_role_changed_description(self) -> None:
+        """Test GroupAuditLogMemberRoleChanged.description."""
+        entry = auth.GroupAuditLogMemberRoleChanged(
+            actor="actor", user="test", role=Group.Roles.ADMIN
+        )
+        self.assertEqual(entry.description(), "actor set test's role as admin")
+
+    def test_removed_description(self) -> None:
+        """Test GroupAuditLogMemberRemoved.description."""
+        entry = auth.GroupAuditLogMemberRemoved(actor="actor", user="test")
+        self.assertEqual(entry.description(), "actor removed test")
+
+
+class ClientEnrollTests(TestCase):
+    """Tests for :py:class:`ClientEnroll`."""
+
+    def test_fields(self) -> None:
+        """Test basic model fields."""
+        ce = ClientEnroll.objects.create(
+            nonce="12345678", payload={"foo": "bar"}
+        )
+        self.assertEqual(ce.nonce, "12345678")
+        self.assertEqual(ce.payload, {"foo": "bar"})
+
+    def test_nonce_unique(self) -> None:
+        """Nonce must be unique."""
+        ClientEnroll.objects.create(nonce="12345678", payload={"foo": "bar"})
+        with self.assertRaisesRegex(
+            IntegrityError, "db_clientenroll_nonce_key"
+        ):
+            ClientEnroll.objects.create(
+                nonce="12345678", payload={"foo": "baz"}
+            )
