@@ -26,22 +26,86 @@ class WorkerManagerTests(TestCase):
     def test_connected(self) -> None:
         """WorkerManager.connected() return the connected Workers."""
         worker_connected = Worker.objects.create_with_fqdn(
-            'connected-worker', Token.objects.create()
+            'connected-worker', token=Token.objects.create()
         )
 
         worker_connected.mark_connected()
 
         Worker.objects.create_with_fqdn(
             'not-connected-worker',
-            Token.objects.create(),
+            token=Token.objects.create(),
         )
 
         self.assertQuerySetEqual(Worker.objects.connected(), [worker_connected])
 
+    def test_active(self) -> None:
+        """Test active() filter."""
+        worker_pool = self.playground.create_worker_pool()
+        workers: dict[str, Worker] = {}
+
+        workers["static_disconnected"] = self.playground.create_worker(
+            fqdn="disconnected.static.lan"
+        )
+
+        workers["static_connected_idle"] = self.playground.create_worker(
+            fqdn="idle.connected.static.lan"
+        )
+
+        workers["dynamic_terminated"] = self.playground.create_worker(
+            fqdn="disconnected.dynamic.lan", worker_pool=worker_pool
+        )
+        workers["dynamic_terminated"].instance_created_at = None
+        workers["dynamic_terminated"].save()
+
+        workers["dynamic_disconnected"] = self.playground.create_worker(
+            fqdn="disconnected.dynamic.lan", worker_pool=worker_pool
+        )
+
+        workers["dynamic_connected_idle"] = self.playground.create_worker(
+            fqdn="idle.connected.dynamic.lan", worker_pool=worker_pool
+        )
+        workers["dynamic_connected_idle"].connected_at = timezone.now()
+        workers["dynamic_connected_idle"].save()
+
+        workers["dynamic_connected_busy"] = self.playground.create_worker(
+            fqdn="busy.connected.dynamic.lan", worker_pool=worker_pool
+        )
+        workers["dynamic_connected_busy"].connected_at = timezone.now()
+        workers["dynamic_connected_busy"].save()
+        wr = self.playground.create_work_request()
+        wr.assign_worker(workers["dynamic_connected_busy"])
+        wr.mark_running()
+
+        workers["dynamic_connected_now_idle"] = self.playground.create_worker(
+            fqdn="now_idle.connected.dynamic.lan", worker_pool=worker_pool
+        )
+        workers["dynamic_connected_now_idle"].connected_at = timezone.now()
+        workers["dynamic_connected_now_idle"].save()
+        wr = self.playground.create_work_request()
+        wr.assign_worker(workers["dynamic_connected_now_idle"])
+        wr.mark_completed(WorkRequest.Results.SUCCESS)
+
+        for worker, included in (
+            ("static_disconnected", True),
+            ("static_connected_idle", True),
+            ("dynamic_terminated", False),
+            ("dynamic_disconnected", True),
+            ("dynamic_connected_idle", True),
+            ("dynamic_connected_busy", True),
+            ("dynamic_connected_now_idle", True),
+        ):
+            with self.subTest(worker=worker):
+                self.assertEqual(
+                    Worker.objects.active()
+                    .filter(pk=workers[worker].pk)
+                    .exists(),
+                    included,
+                )
+
     def test_waiting_for_work_request(self) -> None:
         """Test WorkerManager.waiting_for_work_request() return a Worker."""
         worker = Worker.objects.create_with_fqdn(
-            'worker-a', Token.objects.create(enabled=True)
+            'worker-a', token=Token.objects.create(enabled=True)
         )
 
         # WorkerManagement.waiting_for_work_request: returns no workers because
@@ -115,12 +179,12 @@ class WorkerManagerTests(TestCase):
     def test_waiting_for_work_request_no_return_disabled_workers(self) -> None:
         """Test WorkerManager.waiting_for_work_request() no return disabled."""
         worker_enabled = Worker.objects.create_with_fqdn(
-            "worker-enabled", Token.objects.create(enabled=True)
+            "worker-enabled", token=Token.objects.create(enabled=True)
         )
         worker_enabled.mark_connected()
 
         worker_disabled = Worker.objects.create_with_fqdn(
-            "worker-disabled", Token.objects.create(enabled=False)
+            "worker-disabled", token=Token.objects.create(enabled=False)
         )
         worker_disabled.mark_connected()
 
@@ -131,11 +195,11 @@ class WorkerManagerTests(TestCase):
     def test_waiting_for_work_request_static_first(self) -> None:
         """Ensure waiting_for_work_request() returns static workers first."""
         worker_static = Worker.objects.create_with_fqdn(
-            "worker-static", Token.objects.create(enabled=True)
+            "worker-static", token=Token.objects.create(enabled=True)
         )
 
         worker_cloud = Worker.objects.create_with_fqdn(
-            "worker-cloud", Token.objects.create(enabled=True)
+            "worker-cloud", token=Token.objects.create(enabled=True)
         )
         worker_cloud.worker_pool = WorkerPool.objects.create(
             name="test",
@@ -247,7 +311,7 @@ class WorkerManagerTests(TestCase):
         """WorkerManager.get_worker_by_token_or_none() return the Worker."""
         token = Token.objects.create()
 
-        worker = Worker.objects.create_with_fqdn('worker-a', token)
+        worker = Worker.objects.create_with_fqdn('worker-a', token=token)
 
         self.assertEqual(
             Worker.objects.get_worker_by_token_key_or_none(token.key), worker
@@ -257,7 +321,7 @@ class WorkerManagerTests(TestCase):
         """WorkerManager.get_worker_or_none() return the Worker."""
         token = Token.objects.create()
 
-        worker = Worker.objects.create_with_fqdn('worker-a', token)
+        worker = Worker.objects.create_with_fqdn('worker-a', token=token)
 
         self.assertEqual(Worker.objects.get_worker_or_none('worker-a'), worker)
 
@@ -268,21 +332,21 @@ class WorkerManagerTests(TestCase):
     def test_with_idle_time_no_work_requests_not_connected(self) -> None:
         """Test idle_time with no work requests assigned and never connected."""
         worker = Worker.objects.create_with_fqdn(
-            'worker-a', Token.objects.create(enabled=True)
+            'worker-a', token=Token.objects.create(enabled=True)
         )
         worker.registered_at = timezone.now() - datetime.timedelta(days=1)
         worker.save()
 
         idle_time = (
             Worker.objects.with_idle_time().get(pk=worker.pk)
-        ).idle_time  # type: ignore[attr-defined]
+        ).idle_time
         self.assertGreaterEqual(idle_time, 86400)
         self.assertLessEqual(idle_time, 86400 + 3600)
 
     def test_with_idle_time_no_work_requests_connected(self) -> None:
         """Compute idle_time with no work requests assigned and connected."""
         worker = Worker.objects.create_with_fqdn(
-            'worker-a', Token.objects.create(enabled=True)
+            'worker-a', token=Token.objects.create(enabled=True)
         )
         worker.registered_at = timezone.now() - datetime.timedelta(days=2)
         worker.instance_created_at = timezone.now() - datetime.timedelta(days=1)
@@ -290,14 +354,14 @@ class WorkerManagerTests(TestCase):
 
         idle_time = (
             Worker.objects.with_idle_time().get(pk=worker.pk)
-        ).idle_time  # type: ignore[attr-defined]
+        ).idle_time
         self.assertGreaterEqual(idle_time, 86400)
         self.assertLessEqual(idle_time, 86400 + 3600)
 
     def test_with_idle_time_running_task(self) -> None:
         """Compute idle_time with a running task."""
         worker = Worker.objects.create_with_fqdn(
-            'worker-a', Token.objects.create(enabled=True)
+            'worker-a', token=Token.objects.create(enabled=True)
         )
         worker.registered_at = timezone.now() - datetime.timedelta(days=2)
         worker.instance_created_at = timezone.now() - datetime.timedelta(days=1)
@@ -309,13 +373,13 @@ class WorkerManagerTests(TestCase):
 
         idle_time = (
             Worker.objects.with_idle_time().get(pk=worker.pk)
-        ).idle_time  # type: ignore[attr-defined]
+        ).idle_time
         self.assertGreaterEqual(idle_time, 0)
 
     def test_with_idle_time_completed_task(self) -> None:
         """Compute idle_time with a running task."""
         worker = Worker.objects.create_with_fqdn(
-            'worker-a', Token.objects.create(enabled=True)
+            'worker-a', token=Token.objects.create(enabled=True)
         )
         worker.registered_at = timezone.now() - datetime.timedelta(days=2)
         worker.instance_created_at = timezone.now() - datetime.timedelta(days=1)
@@ -328,10 +392,42 @@ class WorkerManagerTests(TestCase):
 
         idle_time = (
             Worker.objects.with_idle_time().get(pk=worker.pk)
-        ).idle_time  # type: ignore[attr-defined]
+        ).idle_time
         self.assertGreaterEqual(idle_time, 6 * 3600)
         self.assertLessEqual(idle_time, 7 * 3600)
 
+    def test_mark_all_disconnected(self) -> None:
+        """WorkerManager.mark_all_disconnected() affects non-Celery workers."""
+        worker_connected_external = self.playground.create_worker(
+            fqdn="connected-external"
+        )
+        worker_connected_external.mark_connected()
+        worker_connected_signing = self.playground.create_worker(
+            worker_type=WorkerType.SIGNING, fqdn="connected-signing"
+        )
+        worker_connected_signing.mark_connected()
+        worker_connected_celery = Worker.objects.get_or_create_celery()
+        worker_connected_celery.mark_connected()
+        worker_disconnected_external = self.playground.create_worker(
+            fqdn="disconnected-external"
+        )
+        worker_disconnected_signing = self.playground.create_worker(
+            worker_type=WorkerType.SIGNING, fqdn="disconnected-signing"
+        )
+
+        Worker.objects.mark_all_disconnected()
+
+        worker_connected_celery.refresh_from_db()
+        self.assertTrue(worker_connected_celery.connected())
+        for disconnected in (
+            worker_connected_external,
+            worker_connected_signing,
+            worker_disconnected_external,
+            worker_disconnected_signing,
+        ):
+            disconnected.refresh_from_db()
+            self.assertFalse(disconnected.connected())
+
 
 class WorkerTests(TestCase):
     """Tests for the Worker model."""
@@ -343,7 +439,7 @@ class WorkerTests(TestCase):
         """Set up the Worker for the tests."""
         super().setUpTestData()
         cls.worker = Worker.objects.create_with_fqdn(
-            "computer.lan", Token.objects.create()
+            "computer.lan", token=Token.objects.create()
         )
         cls.worker.static_metadata = {"os": "debian"}
         cls.worker.set_dynamic_metadata({"cpu_cores": "4"})
@@ -402,18 +498,19 @@ class WorkerTests(TestCase):
 
         self.assertFalse(worker.is_busy())
 
-    def test_is_busy_concurrency(self) -> None:
-        """The is_busy method handles workers with concurrency > 1."""
+    def test_is_at_capacity(self) -> None:
+        """The is_at_capacity method handles workers with concurrency > 1."""
         worker = Worker.objects.get_or_create_celery()
         worker.concurrency = 3
         worker.save()
+        self.assertFalse(worker.is_at_capacity())
         for _ in range(3):
-            self.assertFalse(worker.is_busy())
+            self.assertFalse(worker.is_at_capacity())
             work_request = self.playground.create_work_request(worker=worker)
             work_request.assign_worker(worker)
-        self.assertTrue(worker.is_busy())
+        self.assertTrue(worker.is_at_capacity())
         work_request.mark_aborted()
-        self.assertFalse(worker.is_busy())
+        self.assertFalse(worker.is_at_capacity())
 
     def test_running_work_requests(self) -> None:
         """The running_work_requests return expected work requests."""
@@ -488,3 +585,10 @@ class WorkerTests(TestCase):
             self.worker.__str__(),
             f"Id: {self.worker.id} Name: {self.worker.name}",
         )
+
+    def test_get_absolute_url(self) -> None:
+        """Test the get_absolute_url method."""
+        self.assertEqual(
+            self.worker.get_absolute_url(),
+            f"/-/status/workers/{self.worker.name}/",
+        )
