@@ -16,6 +16,7 @@ from typing import TYPE_CHECKING
 from django.core.exceptions import ValidationError
 from django.db import models, transaction
 from django.db.models import JSONField, QuerySet
+from django.urls import reverse
 from django.utils import timezone
 
 from debusine.db.context import context
@@ -76,6 +77,10 @@ class WorkerPool(models.Model):
         """Return the id and name of the WorkerPool."""
         return f"Id: {self.id} Name: {self.name}"
 
+    def get_absolute_url(self) -> str:
+        """Return an absolute URL to view this WorkerPool."""
+        return reverse("worker-pools:detail", kwargs={"name": self.name})
+
     @property
     def limits_model(self) -> WorkerPoolLimits:
         """Return the pydantic model for limits."""
