@@ -892,7 +892,7 @@ class Playground(ArtifactPlayground):
 
         return FileUpload.objects.create(
             file_in_artifact=file_in_artifact,
-            path="temp_file_aaaa",
+            path=f"temp_file_{time.monotonic_ns()}",
         )
 
     @context.disable_permission_checks()
