@@ -42,6 +42,7 @@ from debusine.db.models import (
     Artifact,
     ArtifactRelation,
     Asset,
+    Collection,
     DEFAULT_FILE_STORE_NAME,
     FileInArtifact,
     FileStore,
@@ -825,6 +826,11 @@ class PlaygroundTest(django.test.TestCase, TestCase):
         playground = Playground()
         source = playground.create_source_artifact()
         wr = playground.simulate_package_build(source, architecture="amd64")
+        task_config_collection = Collection.objects.get(
+            workspace=playground.get_default_workspace(),
+            name="default",
+            category=CollectionCategory.TASK_CONFIGURATION,
+        )
 
         binaries: list[Artifact] = []
         buildlogs: list[Artifact] = []
@@ -880,6 +886,7 @@ class PlaygroundTest(django.test.TestCase, TestCase):
                 subject="hello",
                 runtime_context="any:amd64:amd64",
                 configuration_context="bookworm",
+                task_configuration_id=task_config_collection.id,
             ),
         )
         self.assertIsNone(wr.parent)
