@@ -5,7 +5,11 @@ When running in development mode, selected.py should point to this file.
 """
 
 from debusine.project.settings import defaults
-from debusine.project.settings.db_postgresql import DATABASES
+
+try:
+    from debusine.project.settings.db_postgresql import DATABASES
+except (ModuleNotFoundError, PermissionError):
+    from debusine.project.settings.db_postgresql_default import DATABASES
 
 __all__ = [
     'ADMINS',
