@@ -50,6 +50,10 @@ service_urlpatterns: list[URLPattern | URLResolver] = [
         "status/workers/",
         include("debusine.web.urls.workers", namespace="workers"),
     ),
+    path(
+        "status/worker-pools/",
+        include("debusine.web.urls.worker_pools", namespace="worker-pools"),
+    ),
     path(
         "user/",
         include("debusine.web.urls.user", namespace="user"),
