@@ -15,6 +15,7 @@ from contextlib import ExitStack
 from celery import shared_task
 from django.db import transaction
 
+from debusine.artifacts.models import TaskTypes
 from debusine.db.context import context
 from debusine.db.models import WorkRequest
 from debusine.db.models.work_requests import (
@@ -24,7 +25,7 @@ from debusine.db.models.work_requests import (
 )
 from debusine.server.tasks import BaseServerTask
 from debusine.tasks import TaskConfigError
-from debusine.tasks.models import OutputData, OutputDataError, TaskTypes
+from debusine.tasks.models import OutputData, OutputDataError
 
 logger = logging.getLogger(__name__)
 
@@ -89,7 +90,7 @@ def _run_server_task_or_error(work_request: WorkRequest) -> bool:
 # can't fix that here.
 @shared_task  # type: ignore[misc]
 def run_server_task(work_request_id: int) -> bool:
-    """Run a :class:`BaseTask` via Celery."""
+    """Run a :py:class:`BaseTask` via Celery."""
     try:
         work_request = WorkRequest.objects.get(pk=work_request_id)
     except WorkRequest.DoesNotExist:
@@ -131,6 +132,7 @@ def run_server_task(work_request_id: int) -> bool:
 # mypy complains that celery.shared_task is untyped, which is true, but we
 # can't fix that here.
 @shared_task  # type: ignore[misc]
+@transaction.atomic
 def update_workflows() -> None:
     """
     Update expensive properties of workflows.
@@ -138,9 +140,12 @@ def update_workflows() -> None:
     The ``workflow_last_activity_at`` and ``workflow_runtime_status`` fields
     of workflows are used in the web UI and need to be kept up to date.
     """
-    workflow_ancestor_ids = workflow_ancestors(
-        WorkRequest.objects.filter(workflows_need_update=True)
-    ).values_list("id", flat=True)
+    need_update = WorkRequest.objects.filter(
+        workflows_need_update=True
+    ).select_for_update()
+    workflow_ancestor_ids = workflow_ancestors(need_update).values_list(
+        "id", flat=True
+    )
     for workflow in WorkRequest.objects.filter(
         id__in=workflow_ancestor_ids
     ).select_for_update():
@@ -151,5 +156,5 @@ def update_workflows() -> None:
         workflow.workflow_runtime_status = compute_workflow_runtime_status(
             workflow
         )
-        workflow.workflows_need_update = False
         workflow.save()
+    need_update.update(workflows_need_update=False)
