@@ -9,11 +9,16 @@
 
 """Implementation of external file backends."""
 
+import tempfile
 from collections.abc import Callable, Generator
+from contextlib import contextmanager
 from datetime import datetime
 from pathlib import Path
 from typing import Any, Self
 
+import requests
+from requests_toolbelt.downloadutils.stream import stream_response_to_file
+
 from debusine.db.models import File, FileInStore, FileStore
 from debusine.server.file_backend.interface import (
     FileBackendEntryInterface,
@@ -59,6 +64,21 @@ class ExternalDebianSuiteFileBackendEntry(
         """
         return None
 
+    @contextmanager
+    def get_temporary_local_path(self) -> Generator[Path]:
+        """Yield a temporary local path for the file."""
+        # TODO: Once we can assume Python >= 3.12, it would be better to use
+        # delete_on_close=False here and call temp_file.close() instead of
+        # temp_file.flush().
+        with tempfile.NamedTemporaryFile(
+            prefix="debusine-external-"
+        ) as temp_file:
+            with requests.get(self.get_url(), stream=True) as r:
+                r.raise_for_status()
+                stream_response_to_file(r, temp_file)
+            temp_file.flush()
+            yield Path(temp_file.name)
+
     def get_url(self, content_type: str | None = None) -> str:  # noqa: U100
         """
         Return a URL pointing to the content.
