@@ -58,6 +58,27 @@ class ExternalDebianSuiteFileBackendTests(TestCase):
         )
         self.assertEqual(str(entry), expected_url)
 
+    @responses.activate
+    def test_get_temporary_local_path(self) -> None:
+        """`get_temporary_local_path` yields a temporary local path."""
+        db_file = self.file_backend.add_remote_file(
+            "pool/main/c/coreutils/coreutils_9.1-1_amd64.deb",
+            bytes.fromhex(
+                "61038f857e346e8500adf53a2a0a2085"
+                "9f4d3a3b51570cc876b153a2d51a3091"
+            ),
+            2896560,
+        )
+        expected_url = (
+            "http://ftp.debian.org/debian/"
+            "pool/main/c/coreutils/coreutils_9.1-1_amd64.deb"
+        )
+        mocked_content = b"coreutils.deb_mocked_content"
+        responses.add(responses.GET, expected_url, mocked_content)
+
+        with self.file_backend.get_temporary_local_path(db_file) as temp_path:
+            self.assertEqual(temp_path.read_bytes(), mocked_content)
+
     @responses.activate
     def test_remote_file(self) -> None:  # pragma: no cover
         """Create a remote file."""
