@@ -29,7 +29,7 @@ from debusine.server.file_backend.local import (
     LocalFileBackendEntry,
 )
 from debusine.test.django import TestCase
-from debusine.test.utils import data_generator
+from debusine.test.test_utils import data_generator
 
 
 class LocalFileBackendTests(TestCase):
@@ -161,6 +161,22 @@ class LocalFileBackendTests(TestCase):
             / f"{hash_hex}-{file.size}",
         )
 
+    def test_get_temporary_local_path(self) -> None:
+        """LocalFileBackend.get_temporary_local_path yields the file path."""
+        temp_file_path = self.create_temporary_file()
+        hash_hex = File.calculate_hash(temp_file_path).hex()
+        file = File.from_local_path(temp_file_path)
+
+        with self.file_backend.get_temporary_local_path(file) as temp_path:
+            self.assertEqual(
+                temp_path,
+                Path(self.default_file_store.configuration["base_directory"])
+                / hash_hex[0:2]
+                / hash_hex[2:4]
+                / hash_hex[4:6]
+                / f"{hash_hex}-{file.size}",
+            )
+
     def test_get_url(self) -> None:
         """LocalFileBackend.get_url return None."""
         fileobj = self.playground.create_file_in_backend(self.file_backend)
