@@ -34,7 +34,6 @@ from debusine.db.models.assets import (
     AssetUsageRole,
     AssetUsageRoles,
 )
-from debusine.db.models.permissions import Roles
 from debusine.django.management.debusine_base_command import DebusineBaseCommand
 from debusine.server.management.management_utils import (
     AssetUsageRoles as AssetUsageRolesPrinter,
@@ -188,7 +187,7 @@ class Command(DebusineBaseCommand):
             )
         return group
 
-    def get_role_by_name(self, role_name: str, usage_role: bool) -> Roles:
+    def get_role_by_name(self, role_name: str, usage_role: bool) -> str:
         """Return the Roles object referring to role_name."""
         role_class = AssetUsageRoles if usage_role else AssetRoles
         try:
