@@ -20,19 +20,15 @@ import sys
 from typing import Any, NoReturn
 
 import yaml
+from django.conf import settings
 from django.contrib.auth import get_user_model
 from django.core.management import CommandError, CommandParser
 from django.db import transaction
 
-from debusine.db.models import (
-    DEFAULT_WORKSPACE_NAME,
-    SYSTEM_USER_NAME,
-    User,
-    WorkRequest,
-    Workspace,
-)
+from debusine.artifacts.models import TaskTypes
+from debusine.db.models import SYSTEM_USER_NAME, User, WorkRequest
 from debusine.django.management.debusine_base_command import DebusineBaseCommand
-from debusine.tasks.models import TaskTypes
+from debusine.server.management.management_utils import get_workspace
 
 
 class Command(DebusineBaseCommand):
@@ -58,7 +54,10 @@ class Command(DebusineBaseCommand):
             default=SYSTEM_USER_NAME,
         )
         parser.add_argument(
-            "--workspace", help="Workspace", default=DEFAULT_WORKSPACE_NAME
+            "--workspace",
+            metavar="scope/name",
+            help="Workspace",
+            default=settings.DEBUSINE_DEFAULT_WORKSPACE,
         )
         parser.add_argument(
             "--data",
@@ -87,7 +86,6 @@ class Command(DebusineBaseCommand):
         task_type = TaskTypes[options["task_type"].upper()]
         task_name = options["task_name"]
         created_by_name = options["created_by"]
-        workspace_name = options["workspace"]
         data = self.parse_yaml_data(options["data"].read()) or {}
         if options["event_reactions"] is not None:
             event_reactions = (
@@ -103,12 +101,7 @@ class Command(DebusineBaseCommand):
                 f'User "{created_by_name}" not found', returncode=3
             )
 
-        try:
-            workspace = Workspace.objects.get(name=workspace_name)
-        except Workspace.DoesNotExist:
-            raise CommandError(
-                f'Workspace "{workspace_name}" not found', returncode=3
-            )
+        workspace = get_workspace(options["workspace"])
 
         with transaction.atomic():
             try:
