@@ -12,16 +12,12 @@
 import io
 
 import yaml
+from django.conf import settings
 from django.core.management import CommandError
 from django.test import override_settings
 
 from debusine.artifacts.models import CollectionCategory
-from debusine.db.context import context
-from debusine.db.models import (
-    Collection,
-    DEFAULT_WORKSPACE_NAME,
-    default_workspace,
-)
+from debusine.db.models import Collection, default_workspace
 from debusine.django.management.tests import call_command
 from debusine.test.django import TestCase
 
@@ -47,7 +43,9 @@ class CreateCollectionCommandTests(TestCase):
         self.assertEqual(exit_code, 0)
 
         collection = Collection.objects.get(name=name, category=category)
-        self.assertEqual(collection.workspace.name, DEFAULT_WORKSPACE_NAME)
+        self.assertEqual(
+            collection.workspace.name, settings.DEBUSINE_DEFAULT_WORKSPACE
+        )
         self.assertEqual(collection.data, data)
 
     def test_create_collection_from_stdin(self) -> None:
@@ -67,7 +65,9 @@ class CreateCollectionCommandTests(TestCase):
         self.assertEqual(exit_code, 0)
 
         collection = Collection.objects.get(name=name, category=category)
-        self.assertEqual(collection.workspace.name, DEFAULT_WORKSPACE_NAME)
+        self.assertEqual(
+            collection.workspace.name, settings.DEBUSINE_DEFAULT_WORKSPACE
+        )
         self.assertEqual(collection.data, data)
 
     def test_create_collection_empty_data(self) -> None:
@@ -90,8 +90,7 @@ class CreateCollectionCommandTests(TestCase):
         name = "test"
         category = CollectionCategory.ENVIRONMENTS
         workspace_name = "test-workspace"
-        with context.disable_permission_checks():
-            self.playground.create_workspace(name=workspace_name)
+        workspace = self.playground.create_workspace(name=workspace_name)
         stdout, stderr, exit_code = call_command(
             "create_collection",
             name,
@@ -106,7 +105,32 @@ class CreateCollectionCommandTests(TestCase):
         self.assertEqual(exit_code, 0)
 
         collection = Collection.objects.get(name=name, category=category)
-        self.assertEqual(collection.workspace.name, workspace_name)
+        self.assertEqual(collection.workspace, workspace)
+
+    def test_create_collection_different_scope(self) -> None:
+        """`create_collection` can use a workspace in a different scope."""
+        name = "test"
+        category = CollectionCategory.ENVIRONMENTS
+        scope_name = "test-scope"
+        scope = self.playground.get_or_create_scope(name=scope_name)
+        workspace = self.playground.create_workspace(
+            scope=scope, name=settings.DEBUSINE_DEFAULT_WORKSPACE
+        )
+        stdout, stderr, exit_code = call_command(
+            "create_collection",
+            name,
+            category,
+            "--workspace",
+            f"{scope_name}/{settings.DEBUSINE_DEFAULT_WORKSPACE}",
+            stdin=io.StringIO(),
+        )
+
+        self.assertEqual(stdout, "")
+        self.assertEqual(stderr, "")
+        self.assertEqual(exit_code, 0)
+
+        collection = Collection.objects.get(name=name, category=category)
+        self.assertEqual(collection.workspace, workspace)
 
     def test_create_collection_invalid_data_yaml(self) -> None:
         """`create_collection` returns error: cannot parse YAML data."""
@@ -125,7 +149,9 @@ class CreateCollectionCommandTests(TestCase):
     def test_create_collection_nonexistent_workspace(self) -> None:
         """`create_collection` returns error: workspace not found."""
         with self.assertRaisesRegex(
-            CommandError, r'^Workspace "nonexistent" not found$'
+            CommandError,
+            fr"^Workspace 'nonexistent' not found in scope "
+            fr"'{settings.DEBUSINE_DEFAULT_SCOPE}'$",
         ) as exc:
             call_command(
                 "create_collection",
