@@ -12,16 +12,16 @@
 import io
 
 import yaml
+from django.conf import settings
 from django.core.management import CommandError
 
-from debusine.db.context import context
+from debusine.artifacts.models import TaskTypes
 from debusine.db.models import WorkRequest, default_workspace, system_user
 from debusine.django.management.tests import call_command
 from debusine.tasks.models import (
     ActionSendNotification,
     ActionUpdateCollectionWithArtifacts,
     EventReactions,
-    TaskTypes,
 )
 from debusine.test.django import TestCase
 
@@ -246,8 +246,7 @@ class CreateWorkRequestCommandTests(TestCase):
     def test_different_workspace(self) -> None:
         """`create_work_request` can use a non-default workspace."""
         workspace_name = "test-workspace"
-        with context.disable_permission_checks():
-            self.playground.create_workspace(name=workspace_name)
+        workspace = self.playground.create_workspace(name=workspace_name)
         stdout, stderr, exit_code = call_command(
             "create_work_request",
             "worker",
@@ -260,7 +259,33 @@ class CreateWorkRequestCommandTests(TestCase):
         work_request = WorkRequest.objects.get(
             task_type=TaskTypes.WORKER, task_name="noop"
         )
-        self.assertEqual(work_request.workspace.name, workspace_name)
+        self.assertEqual(work_request.workspace, workspace)
+
+        expected = yaml.safe_dump({"work_request_id": work_request.id})
+        self.assertEqual(stdout, f"---\n{expected}\n")
+        self.assertEqual(stderr, "")
+        self.assertEqual(exit_code, 0)
+
+    def test_different_scope(self) -> None:
+        """`create_work_request` can use a workspace in a different scope."""
+        scope_name = "test-scope"
+        scope = self.playground.get_or_create_scope(name=scope_name)
+        workspace = self.playground.create_workspace(
+            scope=scope, name=settings.DEBUSINE_DEFAULT_WORKSPACE
+        )
+        stdout, stderr, exit_code = call_command(
+            "create_work_request",
+            "worker",
+            "noop",
+            "--workspace",
+            f"{scope_name}/{settings.DEBUSINE_DEFAULT_WORKSPACE}",
+            stdin=io.StringIO(),
+        )
+
+        work_request = WorkRequest.objects.get(
+            task_type=TaskTypes.WORKER, task_name="noop"
+        )
+        self.assertEqual(work_request.workspace, workspace)
 
         expected = yaml.safe_dump({"work_request_id": work_request.id})
         self.assertEqual(stdout, f"---\n{expected}\n")
@@ -329,7 +354,7 @@ class CreateWorkRequestCommandTests(TestCase):
     def test_workspace_not_found(self) -> None:
         """`create_work_request` returns error: workspace not found."""
         with self.assertRaisesRegex(
-            CommandError, r'^Workspace "nonexistent" not found'
+            CommandError, r"^Workspace 'nonexistent' not found"
         ) as exc:
             call_command(
                 "create_work_request",
