@@ -31,7 +31,7 @@ class GroupCommandTests(TabularOutputTests, TestCase):
     def setUpTestData(cls) -> None:
         """Set up common test data."""
         super().setUpTestData()
-        cls.group = Group.objects.create(scope=cls.scenario.scope, name="group")
+        cls.group = cls.playground.create_group("group")
 
     def assertGroup(
         self, scope: Scope, name: str, users: list[User] | None = None
@@ -63,16 +63,6 @@ class GroupCommandTests(TabularOutputTests, TestCase):
 
         self.assertEqual(getattr(exc.exception, "returncode"), 3)
 
-    def test_get_scope(self) -> None:
-        """Test get_scope."""
-        command = Command()
-        self.assertEqual(command.get_scope("debusine"), self.scenario.scope)
-        with self.assertRaisesRegex(
-            CommandError, r"Scope 'test' not found"
-        ) as exc:
-            command.get_scope("test")
-        self.assertEqual(getattr(exc.exception, "returncode"), 3)
-
     def test_get_scope_and_group_name(self) -> None:
         """Test get_scope_and_group_name."""
         command = Command()
@@ -151,7 +141,7 @@ class GroupCommandTests(TabularOutputTests, TestCase):
 
     def test_list(self) -> None:
         """Test group list."""
-        self.group.users.add(self.scenario.user)
+        self.playground.add_user(self.group, self.scenario.user)
         with self.assertPrintsTable() as output:
             stdout, stderr, exit_code = call_command(
                 "group", "list", "debusine"
@@ -183,7 +173,7 @@ class GroupCommandTests(TabularOutputTests, TestCase):
         self.assertEqual(stderr, "")
         self.assertEqual(exit_code, 0)
         group = self.assertGroup(self.scenario.scope, "group1")
-        group.users.add(self.playground.get_default_user())
+        self.playground.add_user(group, self.playground.get_default_user())
 
         stdout, stderr, exit_code = call_command(
             "group", "create", "debusine/group1"
@@ -213,7 +203,7 @@ class GroupCommandTests(TabularOutputTests, TestCase):
 
     def test_rename(self) -> None:
         """Test a successful rename."""
-        self.group.users.add(self.playground.get_default_user())
+        self.playground.add_user(self.group, self.playground.get_default_user())
         stdout, stderr, exit_code = call_command(
             "group", "rename", "debusine/group", "group1"
         )
@@ -227,7 +217,7 @@ class GroupCommandTests(TabularOutputTests, TestCase):
 
     def test_rename_noop(self) -> None:
         """Test renaming to current name."""
-        self.group.users.add(self.playground.get_default_user())
+        self.playground.add_user(self.group, self.playground.get_default_user())
 
         with mock.patch("debusine.db.models.Group.save") as save:
             stdout, stderr, exit_code = call_command(
@@ -256,7 +246,7 @@ class GroupCommandTests(TabularOutputTests, TestCase):
 
     def test_rename_target_exists(self) -> None:
         """Test renaming with a name already in use."""
-        group = Group.objects.create(scope=self.scenario.scope, name="test")
+        group = self.playground.create_group("test")
         stdout, stderr, exit_code = call_command(
             "group", "rename", "debusine/test", "group"
         )
@@ -292,7 +282,7 @@ class GroupCommandTests(TabularOutputTests, TestCase):
 
     def test_delete(self) -> None:
         """Test a successful delete."""
-        self.group.users.add(self.playground.get_default_user())
+        self.playground.add_user(self.group, self.playground.get_default_user())
         stdout, stderr, exit_code = call_command(
             "group",
             "delete",
@@ -308,7 +298,7 @@ class GroupCommandTests(TabularOutputTests, TestCase):
 
     def test_delete_missing(self) -> None:
         """Test idempotence in deleting a missing group."""
-        self.group.users.add(self.playground.get_default_user())
+        self.playground.add_user(self.group, self.playground.get_default_user())
         stdout, stderr, exit_code = call_command(
             "group",
             "delete",
@@ -326,7 +316,7 @@ class GroupCommandTests(TabularOutputTests, TestCase):
         """Test members --set."""
         user1 = self.playground.create_user("user1")
         user2 = self.playground.create_user("user2")
-        self.group.users.add(self.playground.get_default_user())
+        self.playground.add_user(self.group, self.playground.get_default_user())
 
         stdout, stderr, exit_code = call_command(
             "group", "members", "debusine/group", "--set", "user1", "user2"
@@ -341,7 +331,7 @@ class GroupCommandTests(TabularOutputTests, TestCase):
 
     def test_members_set_empty(self) -> None:
         """Test members --set to the empty set."""
-        self.group.users.add(self.playground.get_default_user())
+        self.playground.add_user(self.group, self.playground.get_default_user())
 
         stdout, stderr, exit_code = call_command(
             "group", "members", "debusine/group", "--set"
@@ -355,7 +345,7 @@ class GroupCommandTests(TabularOutputTests, TestCase):
     def test_members_add(self) -> None:
         """Test members --add."""
         user1 = self.playground.create_user("user1")
-        self.group.users.add(self.playground.get_default_user())
+        self.playground.add_user(self.group, self.playground.get_default_user())
 
         stdout, stderr, exit_code = call_command(
             "group",
@@ -377,8 +367,8 @@ class GroupCommandTests(TabularOutputTests, TestCase):
     def test_members_remove(self) -> None:
         """Test members --remove."""
         user1 = self.playground.create_user("user1")
-        self.group.users.add(self.playground.get_default_user())
-        self.group.users.add(user1)
+        self.playground.add_user(self.group, self.playground.get_default_user())
+        self.playground.add_user(self.group, user1)
 
         stdout, stderr, exit_code = call_command(
             "group",
@@ -429,7 +419,7 @@ class GroupCommandTests(TabularOutputTests, TestCase):
 
     def test_members_list(self) -> None:
         """Test group members."""
-        self.group.users.add(self.scenario.user)
+        self.playground.add_user(self.group, self.scenario.user)
         with self.assertPrintsTable() as output:
             stdout, stderr, exit_code = call_command(
                 "group", "members", "debusine/group"
