@@ -298,9 +298,11 @@ class DebusineCollector(Collector):
             "Number of User Identities",
             labels=("active", "issuer"),
         )
-        for stat in Identity.objects.values(
-            "user__is_active", "issuer"
-        ).annotate(count=Count("id")):
+        for stat in (
+            Identity.objects.filter(user__isnull=False)
+            .values("user__is_active", "issuer")
+            .annotate(count=Count("id"))
+        ):
             gauge.add_metric(
                 (str(int(stat["user__is_active"])), stat["issuer"]),
                 stat["count"],
