@@ -10,20 +10,27 @@
 """Schedule WorkRequests to Workers."""
 
 import logging
-from operator import attrgetter
 from typing import Any
 from uuid import uuid4
 
 import django.db
-from celery import shared_task
+from celery import shared_task, states
 from django.conf import settings
 from django.db import connection, transaction
-from django.db.models import DateTimeField, Exists, OuterRef, Q
+from django.db.models import (
+    DateTimeField,
+    Exists,
+    IntegerField,
+    JSONField,
+    OuterRef,
+    Q,
+)
 from django.db.models.fields.json import KT
 from django.db.models.functions import Cast
 from django.db.models.signals import post_save
 from django.dispatch import receiver
 from django.utils import timezone
+from django_celery_results.models import TaskResult
 
 from debusine.artifacts.models import TaskTypes
 from debusine.db.models import WorkRequest, Worker
@@ -211,8 +218,21 @@ def schedule_internal() -> list[WorkRequest]:
     These don't require a worker, but need some kind of action from the
     scheduler.
     """
+    in_progress_task_results = TaskResult.objects.filter(
+        status__in={
+            states.PENDING,
+            states.RECEIVED,
+            states.STARTED,
+            states.RETRY,
+        },
+        task_name=("debusine.server.workflows.celery.run_workflow_task"),
+    ).annotate(
+        task_args_array=Cast("task_args", JSONField()),
+        first_task_arg=Cast("task_args_array__0", IntegerField()),
+    )
     qs = (
-        WorkRequest.objects.pending()
+        WorkRequest.objects.with_workflow_root_id()
+        .pending()
         .filter(
             Q(
                 task_type=TaskTypes.INTERNAL,
@@ -224,10 +244,19 @@ def schedule_internal() -> list[WorkRequest]:
         .exclude(
             Q(task_type=TaskTypes.INTERNAL, task_name="workflow")
             & Exists(
-                WorkRequest.objects.running().filter(
-                    task_type=TaskTypes.INTERNAL,
-                    task_name="workflow",
-                    parent=OuterRef("parent"),
+                in_progress_task_results.filter(
+                    first_task_arg__in=WorkRequest.objects.filter(
+                        parent=OuterRef(OuterRef("parent"))
+                    ),
+                )
+            )
+        )
+        # Only one sub-workflow within a given root workflow may run at once.
+        .exclude(
+            Q(task_type=TaskTypes.WORKFLOW)
+            & Exists(
+                in_progress_task_results.filter(
+                    first_task_arg=OuterRef("workflow_root_id")
                 )
             )
         )
@@ -235,7 +264,7 @@ def schedule_internal() -> list[WorkRequest]:
     )
 
     running_workflow_callback_parents: set[WorkRequest] = set()
-    workflow_roots: set[WorkRequest] = set()
+    workflow_root_ids: set[int] = set()
     result: list[WorkRequest] = []
     for work_request in qs:
         match (work_request.task_type, work_request.task_name):
@@ -269,10 +298,9 @@ def schedule_internal() -> list[WorkRequest]:
                 # called directly from the scheduler, although if a
                 # sub-workflow is pending then its root workflow may deal
                 # with populating it.
-                workflow_root = work_request.get_workflow_root()
                 # Workflows always have a root.
-                assert workflow_root is not None
-                workflow_roots.add(workflow_root)
+                assert work_request.workflow_root_id is not None
+                workflow_root_ids.add(work_request.workflow_root_id)
 
             case (TaskTypes.WAIT, _):
                 # Wait tasks are marked running immediately, but otherwise
@@ -286,7 +314,11 @@ def schedule_internal() -> list[WorkRequest]:
                     f"Unexpected internal task name: {unreachable}"
                 )
 
-    for workflow_root in sorted(workflow_roots, key=attrgetter("id")):
+    for workflow_root in (
+        WorkRequest.objects.filter(id__in=workflow_root_ids)
+        .order_by("id")
+        .select_for_update()
+    ):
         # Workflows are populated by the corresponding workflow
         # orchestrator.  This may take a while, so we delegate it to a
         # Celery task.
