@@ -40,6 +40,7 @@ from debusine.artifacts.models import (
 from debusine.assets import AssetCategory, asset_data_model
 from debusine.client.models import (
     LookupChildType,
+    WorkspaceInheritanceChainElement,
     model_to_json_serializable_dict,
 )
 from debusine.db.context import context
@@ -815,51 +816,75 @@ class TaskConfigurationCollectionUpdateResultsSerializer(
 
 
 class WorkspaceChainItemField(
-    FieldBase[Workspace, int | str, int | str, Workspace]
+    FieldBase[Workspace, dict[str, Any], dict[str, Any], Workspace]
 ):
     """Serializers for a Workspace in a workspace chain."""
 
-    def to_representation(self, value: Workspace) -> int:
+    def to_representation(self, value: Workspace) -> dict[str, Any]:
         """Trivially serialize workspaces with IDs."""
-        return value.id
+        res = WorkspaceInheritanceChainElement(
+            id=value.id,
+            scope=value.scope.name,
+            workspace=value.name,
+        )
+        return model_to_json_serializable_dict(res)
+
+    def _workspace_from_id(
+        self, el: WorkspaceInheritanceChainElement
+    ) -> Workspace:
+        assert el.id is not None
+
+        workspace_qs = Workspace.objects.can_display(
+            context.require_user()
+        ).select_related("scope")
+        try:
+            workspace = workspace_qs.get(pk=el.id)
+        except Workspace.DoesNotExist:
+            raise ValidationError(f"Workspace with id {el.id} does not exist")
+        if el.workspace is not None and workspace.name != el.workspace:
+            raise ValidationError(
+                f"Workspace with id {el.id} is not named {el.workspace!r}"
+            )
+        if el.scope is not None and workspace.scope.name != el.scope:
+            raise ValidationError(
+                f"Workspace with id {el.id} is not in scope {el.scope!r}"
+            )
+        return workspace
+
+    def _workspace_from_scope_workspace(
+        self, el: WorkspaceInheritanceChainElement
+    ) -> Workspace:
+        assert el.workspace is not None
+
+        workspace_qs = Workspace.objects.can_display(
+            context.require_user()
+        ).select_related("scope")
+
+        if el.scope is None:
+            el.scope = context.require_scope().name
+
+        try:
+            return workspace_qs.get(scope__name=el.scope, name=el.workspace)
+        except Workspace.DoesNotExist:
+            raise ValidationError(
+                f"Workspace {el.workspace!r}"
+                f" does not exist in scope {el.scope!r}"
+            )
 
     def to_internal_value(self, data: Any) -> Workspace:
         """Parse a workspace from IDs or ``scope/name`` strings."""
-        workspace_qs = Workspace.objects.can_display(context.require_user())
-        match data:
-            # isinstance(True, int) and isinstance(False, int) are True
-            case bool():
-                raise ValidationError(
-                    "Workspace indicator should be an int or string"
-                )
-            case int():
-                try:
-                    return workspace_qs.get(pk=data)
-                except Workspace.DoesNotExist:
-                    raise ValidationError(
-                        f"Workspace with id {data} does not exist"
-                    )
-            case str():
-                try:
-                    scope_name, workspace_name = data.split("/", 1)
-                except ValueError:
-                    raise ValidationError(
-                        f"{data!r} should be in"
-                        "`scopename/workspacename` format"
-                    )
-                try:
-                    return workspace_qs.get(
-                        scope__name=scope_name, name=workspace_name
-                    )
-                except Workspace.DoesNotExist:
-                    raise ValidationError(
-                        f"Workspace {workspace_name!r}"
-                        f" does not exist in scope {scope_name!r}"
-                    )
-            case _:
-                raise ValidationError(
-                    "Workspace indicator should be an int or string"
-                )
+        if not isinstance(data, dict):
+            raise ValidationError("Workspace indicator should be a dict")
+
+        try:
+            parsed = WorkspaceInheritanceChainElement(**data)
+        except ValueError as e:
+            raise ValidationError(str(e))
+
+        if parsed.id is not None:
+            return self._workspace_from_id(parsed)
+        else:
+            return self._workspace_from_scope_workspace(parsed)
 
 
 class WorkspaceChainSerializer(serializers.Serializer[Never]):
