@@ -679,7 +679,7 @@ class APTMirror(
         assert self.work_request is not None
         assert self.workspace is not None
         index = RepositoryIndex.create(
-            file=Path(paragraph["Filename"]), override_name=PurePath(name).name
+            file=Path(paragraph["Filename"]), path=name
         )
         index_artifact = Artifact.objects.create_from_local_artifact(
             index, self.workspace, created_by_work_request=self.work_request
