@@ -365,7 +365,7 @@ class GenerateSuiteIndexes(
     def _add_artifact(
         self,
         *,
-        path_in_archive: PurePath,
+        path_in_suite: PurePath,
         path: Path,
         newer_item: CollectionItem | None,
     ) -> Artifact:
@@ -373,7 +373,7 @@ class GenerateSuiteIndexes(
         assert self.workspace is not None
 
         artifact = Artifact.objects.create_from_local_artifact(
-            RepositoryIndex.create(file=path),
+            RepositoryIndex.create(file=path, path=path_in_suite.as_posix()),
             self.workspace,
             created_by_work_request=self.work_request,
         )
@@ -381,8 +381,8 @@ class GenerateSuiteIndexes(
             artifact,
             user=self.work_request.created_by,
             workflow=self.work_request.parent,
-            variables={"path": path_in_archive.as_posix()},
-            name=f"index:{path_in_archive.as_posix()}",
+            variables={"path": path_in_suite.as_posix()},
+            name=f"index:{path_in_suite.as_posix()}",
             created_at=self.data.generate_at,
             replaced_by=newer_item,
         )
@@ -391,11 +391,11 @@ class GenerateSuiteIndexes(
     def _add_index_info(
         self,
         *,
-        path_in_archive: PurePath,
+        path_in_suite: PurePath,
         path: Path,
         artifact: Artifact | None = None,
     ) -> None:
-        self.index_info[path_in_archive] = IndexInfo(
+        self.index_info[path_in_suite] = IndexInfo(
             size=path.stat().st_size,
             sha256=calculate_hash(path, "sha256").hex(),
             artifact=artifact,
@@ -609,18 +609,18 @@ class GenerateSuiteIndexes(
             flat=True,
         )
 
-        sources_path_in_archive = PurePath(component, "source", "Sources")
-        sources_path = temp_path / sources_path_in_archive
+        sources_path_in_suite = PurePath(component, "source", "Sources")
+        sources_path = temp_path / sources_path_in_suite
         sources_path.parent.mkdir(parents=True, exist_ok=True)
         with sources_path.open("w") as f:
             for stanza in qs:
                 print(stanza, file=f)
                 print(file=f)
         self._add_index_info(
-            path_in_archive=sources_path_in_archive, path=sources_path
+            path_in_suite=sources_path_in_suite, path=sources_path
         )
 
-        sources_xz_path_in_archive = sources_path_in_archive.with_suffix(".xz")
+        sources_xz_path_in_suite = sources_path_in_suite.with_suffix(".xz")
         sources_xz_path = sources_path.with_suffix(".xz")
         with (
             sources_path.open(mode="rb") as uncompressed,
@@ -630,12 +630,12 @@ class GenerateSuiteIndexes(
         ):
             shutil.copyfileobj(uncompressed, compressed)
         sources_xz_artifact = self._add_artifact(
-            path_in_archive=sources_xz_path_in_archive,
+            path_in_suite=sources_xz_path_in_suite,
             path=sources_xz_path,
             newer_item=newer_item,
         )
         self._add_index_info(
-            path_in_archive=sources_xz_path_in_archive,
+            path_in_suite=sources_xz_path_in_suite,
             path=sources_xz_path,
             artifact=sources_xz_artifact,
         )
@@ -763,22 +763,20 @@ class GenerateSuiteIndexes(
             flat=True,
         )
 
-        packages_path_in_archive = PurePath(
+        packages_path_in_suite = PurePath(
             component, f"binary-{architecture}", "Packages"
         )
-        packages_path = temp_path / packages_path_in_archive
+        packages_path = temp_path / packages_path_in_suite
         packages_path.parent.mkdir(parents=True, exist_ok=True)
         with packages_path.open("w") as f:
             for stanza in qs:
                 print(stanza, file=f)
                 print(file=f)
         self._add_index_info(
-            path_in_archive=packages_path_in_archive, path=packages_path
+            path_in_suite=packages_path_in_suite, path=packages_path
         )
 
-        packages_xz_path_in_archive = packages_path_in_archive.with_suffix(
-            ".xz"
-        )
+        packages_xz_path_in_suite = packages_path_in_suite.with_suffix(".xz")
         packages_xz_path = packages_path.with_suffix(".xz")
         with (
             packages_path.open(mode="rb") as uncompressed,
@@ -788,12 +786,12 @@ class GenerateSuiteIndexes(
         ):
             shutil.copyfileobj(uncompressed, compressed)
         packages_xz_artifact = self._add_artifact(
-            path_in_archive=packages_xz_path_in_archive,
+            path_in_suite=packages_xz_path_in_suite,
             path=packages_xz_path,
             newer_item=newer_item,
         )
         self._add_index_info(
-            path_in_archive=packages_xz_path_in_archive,
+            path_in_suite=packages_xz_path_in_suite,
             path=packages_xz_path,
             artifact=packages_xz_artifact,
         )
@@ -863,12 +861,12 @@ class GenerateSuiteIndexes(
                 related.append(index_info.artifact)
 
         release.sort_fields(key=_release_sort_key)
-        release_path_in_archive = PurePath("Release")
-        release_path = temp_path / release_path_in_archive
+        release_path_in_suite = PurePath("Release")
+        release_path = temp_path / release_path_in_suite
         with release_path.open("wb") as f:
             release.dump(f)
         release_artifact = self._add_artifact(
-            path_in_archive=release_path_in_archive,
+            path_in_suite=release_path_in_suite,
             path=release_path,
             newer_item=newer_item,
         )
