@@ -1847,7 +1847,7 @@ class APTMirrorTests(TestCase):
             sources_item.artifact,
             ArtifactCategory.REPOSITORY_INDEX,
             work_request.workspace,
-            {},
+            {"path": "main/source/Sources.xz"},
             {"Sources.xz": sources_path.read_bytes()},
         )
         self.assertEqual(sources_item.created_by_user, work_request.created_by)
