@@ -11,10 +11,13 @@
 
 from debusine.server.tasks.wait import BaseWaitTask
 from debusine.server.tasks.wait.models import DelayData
+from debusine.tasks import DefaultDynamicData
 from debusine.tasks.models import BaseDynamicTaskData
 
 
-class Delay(BaseWaitTask[DelayData, BaseDynamicTaskData]):
+class Delay(
+    BaseWaitTask[DelayData, BaseDynamicTaskData], DefaultDynamicData[DelayData]
+):
     """Task that delays until a given time is reached."""
 
     TASK_VERSION = 1
