@@ -876,38 +876,63 @@ class WorkspaceChainItemFieldTests(TestCase):
         field = WorkspaceChainItemField()
         self.assertEqual(
             field.to_representation(self.scenario.workspace),
-            self.scenario.workspace.pk,
+            {
+                "id": self.scenario.workspace.pk,
+                "scope": self.scenario.scope.name,
+                "workspace": self.scenario.workspace.name,
+            },
         )
 
-    def test_from_int(self) -> None:
+    def test_from_id(self) -> None:
         field = WorkspaceChainItemField()
         self.assertEqual(
-            field.to_internal_value(self.scenario.workspace.pk),
+            field.to_internal_value({"id": self.scenario.workspace.pk}),
             self.scenario.workspace,
         )
 
-    def test_from_int_cannot_display(self) -> None:
+    def test_from_id_cannot_display(self) -> None:
         field = WorkspaceChainItemField()
         hidden = self.playground.create_workspace(name="hidden")
         with self.assertRaisesRegex(
             ValidationError, r"Workspace with id \d+ does not exist"
         ):
-            field.to_internal_value(hidden.pk),
+            field.to_internal_value({"id": hidden.pk}),
 
-    def test_from_str(self) -> None:
+    def test_from_scope_workspace(self) -> None:
         field = WorkspaceChainItemField()
         self.assertEqual(
-            field.to_internal_value(f"debusine/{self.scenario.workspace.name}"),
+            field.to_internal_value(
+                {
+                    "scope": self.scenario.scope.name,
+                    "workspace": self.scenario.workspace.name,
+                }
+            ),
             self.scenario.workspace,
         )
 
-    def test_from_str_invalid_format(self) -> None:
+    def test_from_workspace_only(self) -> None:
+        field = WorkspaceChainItemField()
+        self.assertEqual(
+            field.to_internal_value(
+                {"workspace": self.scenario.workspace.name}
+            ),
+            self.scenario.workspace,
+        )
+
+    def test_from_mismatch_id_worksapce(self) -> None:
         field = WorkspaceChainItemField()
         with self.assertRaisesRegex(
-            ValidationError,
-            r"'invalid' should be in" "`scopename/workspacename` format",
+            ValidationError, r"Workspace with id \d+ is not named 'invalid'"
         ):
-            field.to_internal_value("invalid"),
+            field.to_internal_value(
+                {"id": self.scenario.workspace.id, "workspace": "invalid"}
+            )
+        with self.assertRaisesRegex(
+            ValidationError, r"Workspace with id \d+ is not in scope 'invalid'"
+        ):
+            field.to_internal_value(
+                {"id": self.scenario.workspace.id, "scope": "invalid"}
+            )
 
     def test_from_str_cannot_display(self) -> None:
         field = WorkspaceChainItemField()
@@ -916,19 +941,27 @@ class WorkspaceChainItemFieldTests(TestCase):
             ValidationError,
             r"Workspace 'hidden' does not exist in scope 'debusine'",
         ):
-            field.to_internal_value("debusine/hidden"),
+            field.to_internal_value({"workspace": "hidden"}),
 
     def test_from_invalid(self) -> None:
         field = WorkspaceChainItemField()
         value: Any
-        for value in (None, [], 3.14, False, True, {}):
+        for value in (None, [], 3.14, False, True, "invalid", 42):
             with self.subTest(value=value):
                 with self.assertRaisesRegex(
                     ValidationError,
-                    r"Workspace indicator should be an int or string",
+                    r"Workspace indicator should be a dict",
                 ):
                     field.to_internal_value(value)
 
+    def test_from_empty_dict(self) -> None:
+        field = WorkspaceChainItemField()
+        with self.assertRaisesRegex(
+            ValidationError,
+            r"at least id or workspace need to be set",
+        ):
+            field.to_internal_value({})
+
 
 class WorkspaceChainSerializerTests(TestCase):
     """Tests for :py:class:`WorkspaceChainSerializer`."""
@@ -941,9 +974,16 @@ class WorkspaceChainSerializerTests(TestCase):
 
         for data, expected in (
             ([], []),
-            ([first.pk, second.pk], [first, second]),
-            (["debusine/first"], [first]),
-            ([second.pk, "debusine/first"], [second, first]),
+            ([{"id": first.pk}, {"id": second.pk}], [first, second]),
+            ([{"workspace": "first"}], [first]),
+            ([{"scope": "debusine", "workspace": "first"}], [first]),
+            (
+                [
+                    {"id": second.pk},
+                    {"scope": "debusine", "workspace": "first"},
+                ],
+                [second, first],
+            ),
         ):
             with self.subTest(data=data):
                 serializer = WorkspaceChainSerializer(data={"chain": data})
