@@ -8,10 +8,12 @@
 # contained in the LICENSE file.
 
 """Views for the server application: base infrastructure."""
+import copy
 import logging
-from collections.abc import Callable, Iterable
+from collections.abc import Callable
 from typing import (
     Any,
+    Generic,
     Protocol,
     Self,
     TYPE_CHECKING,
@@ -19,10 +21,10 @@ from typing import (
     runtime_checkable,
 )
 
+from django.core.exceptions import ObjectDoesNotExist
 from django.db.models import Model, QuerySet
-from django.http import JsonResponse
+from django.http import Http404
 from rest_framework import status
-from rest_framework.exceptions import PermissionDenied as DRFPermissionDenied
 from rest_framework.filters import BaseFilterBackend
 from rest_framework.generics import (
     CreateAPIView,
@@ -32,13 +34,13 @@ from rest_framework.generics import (
     RetrieveAPIView,
     UpdateAPIView,
 )
-from rest_framework.permissions import BasePermission, DjangoModelPermissions
 from rest_framework.request import Request
 from rest_framework.response import Response
+from rest_framework.serializers import BaseSerializer
 from rest_framework.views import APIView
 
 from debusine.db.context import ContextConsistencyError, context
-from debusine.db.models import Token, Workspace
+from debusine.db.models import Workspace
 from debusine.db.models.permissions import (
     PermissionUser,
     format_permission_check_error,
@@ -93,124 +95,6 @@ else:
     UpdateAPIViewBase = _UpdateAPIViewBase
 
 
-class ProblemResponse(JsonResponse):
-    """
-    Holds a title and other optional fields to return problems to the client.
-
-    Follows RFC7807 (https://www.rfc-editor.org/rfc/rfc7807#section-6.1)
-    """
-
-    def __init__(
-        self,
-        title: str,
-        detail: str | None = None,
-        validation_errors: Iterable[Any] | None = None,
-        status_code: int = status.HTTP_400_BAD_REQUEST,
-    ) -> None:
-        """
-        Initialize object.
-
-        :param title: included in the response data.
-        :param detail: if not None, included in the response data.
-        :param validation_errors: if not None, included in the response data.
-        :param status_code: HTTP status code for the response.
-        """
-        data: dict[str, Any] = {"title": title}
-
-        if detail is not None:
-            data["detail"] = detail
-
-        if validation_errors is not None:
-            data["validation_errors"] = validation_errors
-
-        super().__init__(
-            data, status=status_code, content_type="application/problem+json"
-        )
-
-
-class IsTokenAuthenticated(BasePermission):
-    """Allow access to requests with a valid token."""
-
-    def has_permission(
-        self, request: Request, view: APIView  # noqa: U100
-    ) -> bool:
-        """Return True if the request is authenticated with a Token."""
-        return isinstance(request.auth, Token) and request.auth.enabled
-
-
-class IsTokenUserAuthenticated(IsTokenAuthenticated):
-    """Allow access if the request has an enabled Token with associated user."""
-
-    def has_permission(
-        self, request: Request, view: APIView  # noqa: U100
-    ) -> bool:
-        """Return True if valid token has a User assigned."""
-        return (
-            isinstance(request.auth, Token)
-            and request.auth.enabled
-            and request.auth.user is not None
-        )
-
-
-class IsTokenUserAuthenticatedDjangoModelPermissions(DjangoModelPermissions):
-    """
-    Allow access if a Token-authenticated user has model permissions.
-
-    This is similar to :py:class:`DjangoModelPermissions`, except that it
-    checks Token authentication.
-    """
-
-    def has_permission(self, request: Request, view: APIView) -> bool:
-        """Return True if the user has the right permissions."""
-        token = request.auth
-        if not token or not token.user:
-            return False
-
-        queryset = self._queryset(view)
-        assert request.method is not None
-        perms = self.get_required_permissions(request.method, queryset.model)
-
-        if not token.user.has_perms(perms):
-            # Raise this rather than the default NotAuthenticated exception
-            # when an authenticator fails; in this case the user is
-            # authenticated, but doesn't have the right permissions.
-            raise DRFPermissionDenied()
-
-        return True
-
-
-class IsWorkerAuthenticated(IsTokenAuthenticated):
-    """Allow access to requests with a token assigned to a worker."""
-
-    def has_permission(
-        self, request: Request, view: APIView  # noqa: U100
-    ) -> bool:
-        """
-        Return True if the request is an authenticated worker.
-
-        The Token must exist in the database and have a Worker.
-        """
-        if not super().has_permission(request, view):
-            # No token authenticated: no Worker Authenticated
-            return False
-
-        if not hasattr(request.auth, "worker"):
-            # request.auth is None; or it's a Token without a "worker"
-            return False
-
-        return True
-
-
-class IsGet(BasePermission):
-    """Allow access if the request's method is GET."""
-
-    def has_permission(
-        self, request: Request, view: APIView  # noqa: U100
-    ) -> bool:
-        """Return True if request.method == "GET"."""
-        return request.method == "GET"
-
-
 class BaseAPIView(APIView):
     """Common base for API views."""
 
@@ -247,6 +131,59 @@ class BaseAPIView(APIView):
 _M = TypeVar("_M", bound=Model, covariant=True)
 
 
+class GetOrCreateAPIView(CreateAPIViewBase[_M], BaseAPIView, Generic[_M]):
+    """Class to create a new object (return 201 created) or return 200."""
+
+    def get_object(self) -> _M:
+        """Override to return more API-friendly errors."""
+        try:
+            return super().get_object()
+        except Http404 as exc:
+            raise DebusineAPIException(
+                title=str(exc), status_code=status.HTTP_404_NOT_FOUND
+            )
+
+    def get_existing_object(self, serializer: BaseSerializer[_M]) -> _M:
+        """
+        If there is an existing object matching Serializer, return it.
+
+        If not, raise ObjectDoesNotExist.
+        """
+        return self.get_queryset().get(**serializer.validated_data)
+
+    def create(self, request: Request, *args: Any, **kwargs: Any) -> Response:
+        """
+        Return the object with HTTP 201 (created) or HTTP 200 (already existed).
+
+        It is a modification of CreateAPIView.create().
+
+        The default behaviour in rest_framework when an object cannot be
+        inserted because it already exists is to return HTTP 400. In
+        GetOrCreateApiView it returns HTTP 200 and the client knows that
+        the object that wanted to create was already in the database.
+        """
+        serializer = self.get_serializer(data=request.data)
+        serializer.is_valid(raise_exception=True)
+
+        response_status: int
+        try:
+            obj = self.get_existing_object(serializer)
+            response_status = status.HTTP_200_OK
+            pk = obj.pk
+        except ObjectDoesNotExist:
+            self.perform_create(serializer)
+            response_status = status.HTTP_201_CREATED
+            assert serializer.instance is not None
+            pk = serializer.instance.pk
+
+        headers = self.get_success_headers(serializer.data)
+
+        response_data = copy.deepcopy(serializer.data)
+        response_data["id"] = pk
+
+        return Response(response_data, status=response_status, headers=headers)
+
+
 @runtime_checkable
 class SupportsCanDisplay(Protocol[_M]):
     """A query set that supports `can_display` filtering."""
@@ -266,7 +203,7 @@ class CanDisplayFilterBackend(BaseFilterBackend):
     ) -> QuerySet[_M]:
         """Keep only objects that can be displayed."""
         assert isinstance(queryset, SupportsCanDisplay)
-        return queryset.can_display(context.require_user())
+        return queryset.can_display(context.user)
 
 
 class Whoami(BaseAPIView):
