@@ -14,6 +14,7 @@ from typing import cast
 
 from debusine.server.workflows.autopkgtest import AutopkgtestWorkflow
 from debusine.server.workflows.base import Workflow, WorkflowValidationError
+from debusine.server.workflows.blhc import BlhcWorkflow
 from debusine.server.workflows.create_experiment_workspace import (
     CreateExperimentWorkspaceWorkflow,
 )
@@ -42,6 +43,7 @@ __doc__ = cast(str, sys.modules[Workflow.__module__].__doc__)
 
 __all__ = [
     "AutopkgtestWorkflow",
+    "BlhcWorkflow",
     "DebDiffWorkflow",
     "DebianPipelineWorkflow",
     "CreateExperimentWorkspaceWorkflow",
