@@ -393,6 +393,13 @@ class Workflow(BaseServerTask[WD, DTD], metaclass=ABCMeta):
         """Unused abstract method from BaseTask: populate() is used instead."""
         raise NotImplementedError()
 
+    def get_label(self) -> str:
+        """Return the workflow label."""
+        return (
+            self.work_request.workflow_display_name_parameters
+            or self.work_request.workflow_display_name
+        )
+
 
 def orchestrate_workflow(work_request: WorkRequest) -> None:
     """
