@@ -468,7 +468,3 @@ class DebDiffWorkflow(Workflow[DebDiffWorkflowData, BaseDynamicTaskData]):
         self.requires_artifact(wr, news)
 
         return wr
-
-    def get_label(self) -> str:
-        """Return the workflow label."""
-        return "run debdiff"
