@@ -38,7 +38,7 @@ from debusine.server.workflows.models import (
 )
 from debusine.tasks.models import (
     BackendType,
-    BaseDynamicTaskData,
+    DebianPipelineWorkflowDynamicData,
     LintianFailOnSeverity,
     LookupMultiple,
     LookupSingle,
@@ -48,7 +48,7 @@ from debusine.tasks.server import TaskDatabaseInterface
 
 
 class DebianPipelineWorkflow(
-    Workflow[DebianPipelineWorkflowData, BaseDynamicTaskData]
+    Workflow[DebianPipelineWorkflowData, DebianPipelineWorkflowDynamicData]
 ):
     """Debian pipeline workflow."""
 
@@ -70,18 +70,39 @@ class DebianPipelineWorkflow(
 
     def build_dynamic_data(
         self, task_database: TaskDatabaseInterface  # noqa: U100
-    ) -> BaseDynamicTaskData:
+    ) -> DebianPipelineWorkflowDynamicData:
         """
         Compute dynamic data for this workflow.
 
         :subject: package name of ``source_artifact``
         """
         source_data = workflow_utils.source_package_data(self)
-        return BaseDynamicTaskData(
+
+        source_artifact_id = lookup_single(
+            self.data.source_artifact,
+            self.workspace,
+            user=self.work_request.created_by,
+            workflow_root=self.work_request.get_workflow_root(),
+            expect_type=LookupChildType.ARTIFACT,
+        ).artifact.id
+
+        return DebianPipelineWorkflowDynamicData(
             subject=source_data.name,
             parameter_summary=f"{source_data.name}_{source_data.version}",
+            input_source_artifact_id=source_artifact_id,
         )
 
+    def get_input_artifacts_ids(self) -> list[int]:
+        """Return a list with the source package (or empty list)."""
+        if (
+            self.dynamic_data is None
+            or (artifact_id := self.dynamic_data.input_source_artifact_id)
+            is None
+        ):
+            return []
+
+        return [artifact_id]
+
     def populate(self) -> None:
         """Create work requests."""
         if (data_archs := self.data.architectures) is not None:
@@ -123,6 +144,15 @@ class DebianPipelineWorkflow(
             }
         )
 
+        # sbuild workflow promises the buildlog-{arch} artifacts
+        package_build_logs = LookupMultiple.parse_obj(
+            {
+                "collection": "internal@collections",
+                "child_type": LookupChildType.ARTIFACT_OR_PROMISE,
+                "name__startswith": "buildlog-",
+            }
+        )
+
         to_upload = [(self.data.source_artifact, binary_artifacts, False)]
 
         if (
@@ -131,10 +161,12 @@ class DebianPipelineWorkflow(
             or self.data.enable_lintian
             or self.data.enable_piuparts
             or self.data.enable_debdiff
+            or self.data.enable_blhc
         ):
             self._populate_qa(
                 source_artifact=self.data.source_artifact,
                 binary_artifacts=binary_artifacts,
+                package_build_logs=package_build_logs,
                 vendor=self.data.vendor,
                 codename=self.data.codename,
                 architectures=effective_architectures,
@@ -152,6 +184,7 @@ class DebianPipelineWorkflow(
                 piuparts_backend=self.data.piuparts_backend,
                 piuparts_environment=self.data.piuparts_environment,
                 enable_debdiff=self.data.enable_debdiff,
+                enable_blhc=self.data.enable_blhc,
             )
 
         if (
@@ -247,6 +280,7 @@ class DebianPipelineWorkflow(
         *,
         source_artifact: LookupSingle,
         binary_artifacts: LookupMultiple,
+        package_build_logs: LookupMultiple,
         vendor: str,
         codename: str,
         architectures: list[str],
@@ -262,11 +296,13 @@ class DebianPipelineWorkflow(
         piuparts_backend: BackendType,
         piuparts_environment: LookupSingle | None,
         enable_debdiff: bool,
+        enable_blhc: bool,
     ) -> None:
         """Create work request for qa workflow."""
         data = QAWorkflowData(
             source_artifact=source_artifact,
             binary_artifacts=binary_artifacts,
+            package_build_logs=package_build_logs,
             vendor=vendor,
             codename=codename,
             architectures=architectures,
@@ -284,6 +320,7 @@ class DebianPipelineWorkflow(
             enable_piuparts=enable_piuparts,
             piuparts_backend=piuparts_backend,
             enable_debdiff=enable_debdiff,
+            enable_blhc=enable_blhc,
         )
         if piuparts_environment:
             data.piuparts_environment = piuparts_environment
@@ -439,7 +476,3 @@ class DebianPipelineWorkflow(
         if wr.status == WorkRequest.Statuses.PENDING:
             wr.mark_running()
             orchestrate_workflow(wr)
-
-    def get_label(self) -> str:
-        """Return the task label."""
-        return "run Debian pipeline"
