@@ -29,6 +29,7 @@ from debusine.tasks.models import (
     AutopkgtestTimeout,
     BackendType,
     BaseTaskData,
+    BlhcFlags,
     DebDiffFlags,
     ExtraRepository,
     LintianFailOnSeverity,
@@ -360,11 +361,19 @@ class DebDiffWorkflowData(BaseWorkflowData):
     arch_all_host_architecture: str = "amd64"
 
 
+class BlhcWorkflowData(BaseWorkflowData):
+    """`blhc` workflow data."""
+
+    package_build_logs: LookupMultiple
+    extra_flags: list[BlhcFlags] = pydantic.Field(default_factory=list)
+
+
 class QAWorkflowData(RegressionTrackingWorkflowData):
     """`qa` workflow data."""
 
     source_artifact: LookupSingle
     binary_artifacts: LookupMultiple
+    package_build_logs: LookupMultiple | None = None
 
     vendor: str
     codename: str
@@ -392,6 +401,7 @@ class QAWorkflowData(RegressionTrackingWorkflowData):
     piuparts_environment: LookupSingle | None = None
 
     enable_debdiff: bool = False
+    enable_blhc: bool = False
 
     @pydantic.root_validator(allow_reuse=True)
     @classmethod
@@ -421,6 +431,19 @@ class QAWorkflowData(RegressionTrackingWorkflowData):
             )
         return values
 
+    @pydantic.root_validator(allow_reuse=True)
+    @classmethod
+    def enable_blhc_consistency(cls, values: dict[str, Any]) -> dict[str, Any]:
+        """Check consistency of enable_blhc options."""
+        if (
+            values.get("enable_blhc")
+            and values.get("package_build_logs") is None
+        ):
+            raise ValueError(
+                '"package_build_logs" is required if "enable_blhc" is set'
+            )
+        return values
+
 
 class DebianPipelineWorkflowData(BaseWorkflowData):
     """`debian_pipeline` workflow data."""
@@ -460,6 +483,8 @@ class DebianPipelineWorkflowData(BaseWorkflowData):
 
     enable_debdiff: bool = False
 
+    enable_blhc: bool = True
+
     enable_make_signed_source: bool = False
     make_signed_source_purpose: KeyPurpose | None = None
     make_signed_source_key: str | None = None
