@@ -10,10 +10,14 @@
 
 from debusine.server.workflows.base import Workflow
 from debusine.server.workflows.models import BaseWorkflowData
+from debusine.tasks import DefaultDynamicData
 from debusine.tasks.models import BaseDynamicTaskData
 
 
-class NoopWorkflow(Workflow[BaseWorkflowData, BaseDynamicTaskData]):
+class NoopWorkflow(
+    Workflow[BaseWorkflowData, BaseDynamicTaskData],
+    DefaultDynamicData[BaseWorkflowData],
+):
     """Workflow that does nothing."""
 
     TASK_NAME = "noop"
