@@ -312,6 +312,12 @@ class SbuildWorkflow(Workflow[SbuildWorkflowData, BaseDynamicTaskData]):
                 ),
             },
         )
+        self.provides_artifact(
+            wr,
+            ArtifactCategory.PACKAGE_BUILD_LOG,
+            f"{self.data.prefix}buildlog-{architecture}",
+            data={"architecture": architecture},
+        )
 
         try:
             build_logs_collection_id = self.lookup_singleton_collection(
@@ -372,9 +378,3 @@ class SbuildWorkflow(Workflow[SbuildWorkflowData, BaseDynamicTaskData]):
                     "data__deb_fields__Package": binary_package_name
                 },
             )
-
-    def get_label(self) -> str:
-        """Return the task label."""
-        # TODO: copy the source package information in dynamic task data and
-        # use them here if available
-        return f"{self.data.prefix}build a package"
