@@ -934,4 +934,4 @@ class DebDiffWorkflowTests(TestCase):
     @preserve_task_registry()
     def test_get_label(self) -> None:
         w = self.create_debdiff_workflow()
-        self.assertEqual(w.get_label(), "run debdiff")
+        self.assertEqual(w.get_label(), "debdiff")
