@@ -34,8 +34,8 @@ from debusine.server.workflows.models import DebianPipelineWorkflowData
 from debusine.tasks import TaskConfigError
 from debusine.tasks.models import (
     BackendType,
-    BaseDynamicTaskData,
     CollectionItemMatcherKind,
+    DebianPipelineWorkflowDynamicData,
     LintianFailOnSeverity,
 )
 from debusine.test.django import TestCase
@@ -82,6 +82,7 @@ class DebianPipelineWorkflowTests(TestCase):
             "enable_autopkgtest": False,
             "enable_lintian": False,
             "enable_piuparts": False,
+            "enable_blhc": False,
             **extra_data,
         }
 
@@ -134,8 +135,10 @@ class DebianPipelineWorkflowTests(TestCase):
 
         self.assertEqual(
             workflow.compute_dynamic_data(TaskDatabase(wr)),
-            BaseDynamicTaskData(
-                subject="hello", parameter_summary="hello_1.0-1"
+            DebianPipelineWorkflowDynamicData(
+                subject="hello",
+                parameter_summary="hello_1.0-1",
+                input_source_artifact_id=source_artifact.id,
             ),
         )
 
@@ -159,6 +162,35 @@ class DebianPipelineWorkflowTests(TestCase):
         ):
             workflow.compute_dynamic_data(TaskDatabase(wr))
 
+    def test_get_input_artifacts_ids(self) -> None:
+        """Return source artifact id."""
+        upload_artifacts = self.playground.create_upload_artifacts()
+        wr = self.playground.create_workflow(
+            task_name="debian_pipeline",
+            task_data=DebianPipelineWorkflowData(
+                source_artifact=upload_artifacts.upload.id,
+                vendor="debian",
+                codename="sid",
+            ),
+        )
+        workflow = DebianPipelineWorkflow(wr)
+
+        # workflow.dynamic_data is None: return []
+        self.assertIsNone(workflow.dynamic_data)
+        self.assertEqual(workflow.get_input_artifacts_ids(), [])
+
+        # workflow.dynamic_data.source_artifact_id is None: return []
+        workflow.dynamic_data = DebianPipelineWorkflowDynamicData()
+        self.assertEqual(workflow.get_input_artifacts_ids(), [])
+
+        # workflow.dynamic_data.source_artifact_id is the id: return it
+        workflow.dynamic_data.input_source_artifact_id = (
+            upload_artifacts.upload.id
+        )
+        self.assertEqual(
+            workflow.get_input_artifacts_ids(), [upload_artifacts.upload.id]
+        )
+
     def test_source_package_upload(self) -> None:
         """`source_artifact` can be an upload."""
         upload_artifacts = self.playground.create_upload_artifacts()
@@ -218,6 +250,7 @@ class DebianPipelineWorkflowTests(TestCase):
             extra_data={
                 "architectures": ["amd64"],
                 "source_artifact": self.source_artifact.id,
+                "enable_blhc": False,
             }
         )
 
@@ -345,6 +378,16 @@ class DebianPipelineWorkflowTests(TestCase):
                         },
                     }
                 ],
+                "package_build_logs": [
+                    {
+                        "collection": "internal@collections",
+                        "child_type": LookupChildType.ARTIFACT_OR_PROMISE,
+                        "name_matcher": {
+                            "kind": CollectionItemMatcherKind.STARTSWITH,
+                            "value": "buildlog-",
+                        },
+                    }
+                ],
                 "codename": "bookworm",
                 "extra_repositories": [
                     {
@@ -358,6 +401,7 @@ class DebianPipelineWorkflowTests(TestCase):
                 "enable_piuparts": False,
                 "enable_reverse_dependencies_autopkgtest": False,
                 "enable_debdiff": False,
+                "enable_blhc": False,
                 "lintian_backend": BackendType.AUTO,
                 "lintian_fail_on_severity": LintianFailOnSeverity.ERROR,
                 "piuparts_backend": BackendType.AUTO,
@@ -392,6 +436,16 @@ class DebianPipelineWorkflowTests(TestCase):
                         },
                     }
                 ],
+                "package_build_logs": [
+                    {
+                        "collection": "internal@collections",
+                        "child_type": LookupChildType.ARTIFACT_OR_PROMISE,
+                        "name_matcher": {
+                            "kind": CollectionItemMatcherKind.STARTSWITH,
+                            "value": "buildlog-",
+                        },
+                    }
+                ],
                 "codename": "bookworm",
                 "extra_repositories": None,
                 "enable_autopkgtest": False,
@@ -399,6 +453,7 @@ class DebianPipelineWorkflowTests(TestCase):
                 "enable_piuparts": False,
                 "enable_reverse_dependencies_autopkgtest": False,
                 "enable_debdiff": False,
+                "enable_blhc": False,
                 "lintian_backend": BackendType.AUTO,
                 "lintian_fail_on_severity": LintianFailOnSeverity.ERROR,
                 "piuparts_backend": BackendType.AUTO,
@@ -434,6 +489,16 @@ class DebianPipelineWorkflowTests(TestCase):
                         },
                     }
                 ],
+                "package_build_logs": [
+                    {
+                        "collection": "internal@collections",
+                        "child_type": LookupChildType.ARTIFACT_OR_PROMISE,
+                        "name_matcher": {
+                            "kind": CollectionItemMatcherKind.STARTSWITH,
+                            "value": "buildlog-",
+                        },
+                    }
+                ],
                 "codename": "bookworm",
                 "extra_repositories": None,
                 "enable_autopkgtest": True,
@@ -441,6 +506,7 @@ class DebianPipelineWorkflowTests(TestCase):
                 "enable_piuparts": False,
                 "enable_reverse_dependencies_autopkgtest": False,
                 "enable_debdiff": False,
+                "enable_blhc": False,
                 "lintian_backend": BackendType.AUTO,
                 "lintian_fail_on_severity": LintianFailOnSeverity.ERROR,
                 "piuparts_backend": BackendType.AUTO,
@@ -476,6 +542,16 @@ class DebianPipelineWorkflowTests(TestCase):
                         },
                     }
                 ],
+                "package_build_logs": [
+                    {
+                        "collection": "internal@collections",
+                        "child_type": LookupChildType.ARTIFACT_OR_PROMISE,
+                        "name_matcher": {
+                            "kind": CollectionItemMatcherKind.STARTSWITH,
+                            "value": "buildlog-",
+                        },
+                    }
+                ],
                 "codename": "bookworm",
                 "extra_repositories": None,
                 "enable_autopkgtest": False,
@@ -483,6 +559,7 @@ class DebianPipelineWorkflowTests(TestCase):
                 "enable_piuparts": True,
                 "enable_reverse_dependencies_autopkgtest": False,
                 "enable_debdiff": False,
+                "enable_blhc": False,
                 "lintian_backend": BackendType.AUTO,
                 "lintian_fail_on_severity": LintianFailOnSeverity.ERROR,
                 "piuparts_backend": BackendType.AUTO,
@@ -526,6 +603,16 @@ class DebianPipelineWorkflowTests(TestCase):
                         },
                     }
                 ],
+                "package_build_logs": [
+                    {
+                        "collection": "internal@collections",
+                        "child_type": LookupChildType.ARTIFACT_OR_PROMISE,
+                        "name_matcher": {
+                            "kind": CollectionItemMatcherKind.STARTSWITH,
+                            "value": "buildlog-",
+                        },
+                    }
+                ],
                 "codename": "bookworm",
                 "extra_repositories": None,
                 "enable_autopkgtest": False,
@@ -533,6 +620,7 @@ class DebianPipelineWorkflowTests(TestCase):
                 "enable_piuparts": False,
                 "enable_reverse_dependencies_autopkgtest": False,
                 "enable_debdiff": True,
+                "enable_blhc": False,
                 "lintian_backend": BackendType.AUTO,
                 "lintian_fail_on_severity": LintianFailOnSeverity.ERROR,
                 "piuparts_backend": BackendType.AUTO,
@@ -579,6 +667,16 @@ class DebianPipelineWorkflowTests(TestCase):
                         },
                     }
                 ],
+                "package_build_logs": [
+                    {
+                        "collection": "internal@collections",
+                        "child_type": LookupChildType.ARTIFACT_OR_PROMISE,
+                        "name_matcher": {
+                            "kind": CollectionItemMatcherKind.STARTSWITH,
+                            "value": "buildlog-",
+                        },
+                    }
+                ],
                 "codename": "bookworm",
                 "extra_repositories": None,
                 "enable_autopkgtest": False,
@@ -586,6 +684,7 @@ class DebianPipelineWorkflowTests(TestCase):
                 "enable_piuparts": True,
                 "enable_reverse_dependencies_autopkgtest": False,
                 "enable_debdiff": False,
+                "enable_blhc": False,
                 "lintian_backend": BackendType.AUTO,
                 "lintian_fail_on_severity": LintianFailOnSeverity.ERROR,
                 "piuparts_backend": BackendType.INCUS_LXC,
@@ -1292,4 +1391,4 @@ class DebianPipelineWorkflowTests(TestCase):
     def test_get_label(self) -> None:
         """Test get_label."""
         w = self.create_debian_pipeline_workflow()
-        self.assertEqual(w.get_label(), "run Debian pipeline")
+        self.assertEqual(w.get_label(), "debian_pipeline-template")
