@@ -21,6 +21,7 @@ from debusine.artifacts.models import (
     TaskTypes,
 )
 from debusine.db.models import Artifact, TaskDatabase, WorkRequest
+from debusine.server.collections import DebianSuiteManager
 from debusine.server.scheduler import schedule
 from debusine.server.workflows import (
     QAWorkflow,
@@ -61,6 +62,7 @@ class QAWorkflowTests(TestCase):
         task_data = {
             "source_artifact": 2,
             "binary_artifacts": [3],
+            "package_build_logs": [4],
             "vendor": "debian",
             "codename": "bookworm",
         }
@@ -125,10 +127,21 @@ class QAWorkflowTests(TestCase):
                                 "source_package_name": "hello",
                             },
                         )
+                        self.provides_artifact(
+                            child,
+                            ArtifactCategory.PACKAGE_BUILD_LOG,
+                            f"log-{arch}",
+                            data={
+                                "architecture": arch,
+                                "source_package_name": "hello",
+                            },
+                        )
 
                 data = {
                     "source_artifact": source_artifact_id,
                     "binary_artifacts": [30, 31],
+                    "package_build_logs": [32, 33],
+                    "enable_blhc": False,
                     "vendor": "debian",
                     "codename": "bookworm",
                 }
@@ -204,6 +217,9 @@ class QAWorkflowTests(TestCase):
                 binary_artifacts=LookupMultiple.parse_obj(
                     ["internal@collections/name:build-amd64"]
                 ),
+                package_build_logs=LookupMultiple.parse_obj(
+                    ["internal@collections/name:log-amd64"]
+                ),
                 vendor="debian",
                 codename="sid",
             ),
@@ -229,6 +245,9 @@ class QAWorkflowTests(TestCase):
                 binary_artifacts=LookupMultiple.parse_obj(
                     ["internal@collections/name:build-amd64"]
                 ),
+                package_build_logs=LookupMultiple.parse_obj(
+                    ["internal@collections/name:log-amd64"]
+                ),
                 vendor="debian",
                 codename="sid",
             ),
@@ -252,6 +271,7 @@ class QAWorkflowTests(TestCase):
                 binary_artifacts=LookupMultiple.parse_obj(
                     ["internal@collections/name:build-amd64"]
                 ),
+                enable_blhc=False,
                 vendor="debian",
                 codename="sid",
             ),
@@ -1185,6 +1205,53 @@ class QAWorkflowTests(TestCase):
             },
         )
 
+    @preserve_task_registry()
+    def test_populate_blhc(self) -> None:
+        """Populate enables blhc."""
+        architectures = ["amd64"]
+        source_artifact = self.playground.create_source_artifact(name="hello")
+        sid = self.playground.create_collection(
+            name="sid", category=CollectionCategory.SUITE
+        )
+        assert isinstance(sid.manager, DebianSuiteManager)
+
+        workflow = self.orchestrate(
+            extra_data={
+                "source_artifact": source_artifact.id,
+                "binary_artifacts": ["internal@collections/name:build-amd64"],
+                "package_build_logs": ["internal@collections/name:log-amd64"],
+                "architectures": ["amd64"],
+                "qa_suite": f"sid@{CollectionCategory.SUITE}",
+                "enable_blhc": True,
+            },
+            architectures=architectures,
+        )
+
+        blhc_workflow = workflow.children.get(
+            task_name="blhc", task_type=TaskTypes.WORKFLOW
+        )
+
+        self.assertEqual(blhc_workflow.children.count(), 1)
+        self.assertEqual(blhc_workflow.status, WorkRequest.Statuses.RUNNING)
+
+        [blhc_work_request] = blhc_workflow.children.all()
+
+        self.assertEqual(blhc_work_request.task_name, "blhc")
+        self.assertEqual(
+            blhc_work_request.workflow_data_json,
+            {
+                "display_name": "build log hardening check for amd64",
+                "step": "blhc-amd64",
+            },
+        )
+        self.assertEqual(
+            blhc_work_request.task_data,
+            {
+                "input": {"artifact": "internal@collections/name:log-amd64"},
+                "extra_flags": [],
+            },
+        )
+
     def test_get_label(self) -> None:
         """Test get_label()."""
         w = self.create_qa_workflow()
