@@ -355,7 +355,19 @@ class SbuildWorkflowTests(CollectionTestMixin, TestCase):
                             "source_package_name": "hello",
                             "architecture": expected_arch,
                         },
-                    }
+                    },
+                    {
+                        "action": ActionTypes.UPDATE_COLLECTION_WITH_ARTIFACTS,
+                        "artifact_filters": {
+                            "category": ArtifactCategory.PACKAGE_BUILD_LOG
+                        },
+                        "collection": "internal@collections",
+                        "name_template": f"buildlog-{expected_arch}",
+                        "variables": {
+                            "architecture": expected_arch,
+                        },
+                        "created_at": None,
+                    },
                 ],
             )
             self.assertQuerySetEqual(child.dependencies.all(), [])
@@ -553,6 +565,18 @@ class SbuildWorkflowTests(CollectionTestMixin, TestCase):
                         "source_package_name": "hello",
                     },
                 },
+                {
+                    "action": ActionTypes.UPDATE_COLLECTION_WITH_ARTIFACTS,
+                    "artifact_filters": {
+                        "category": ArtifactCategory.PACKAGE_BUILD_LOG
+                    },
+                    "collection": "internal@collections",
+                    "name_template": "buildlog-amd64",
+                    "variables": {
+                        "architecture": "amd64",
+                    },
+                    "created_at": None,
+                },
                 {
                     "action": ActionTypes.UPDATE_COLLECTION_WITH_ARTIFACTS,
                     "collection": build_logs.id,
@@ -888,16 +912,21 @@ class SbuildWorkflowTests(CollectionTestMixin, TestCase):
 
     def test_label(self) -> None:
         """Test get_label."""
-        wr = WorkRequest(
+        source = self.create_source_package(
+            "hello", "1.0", dsc_fields={"Architecture": "all"}
+        )
+        self.playground.create_debian_environment(codename="bookworm")
+        template = self.create_sbuild_template()
+        wr = self.playground.create_workflow(
+            template,
             task_data={
-                "prefix": "signed-source|",
                 "input": {
-                    "source_artifact": 42,
+                    "source_artifact": source.id,
                 },
                 "target_distribution": "debian:bookworm",
                 "architectures": ["all", "amd64"],
             },
-            workspace=default_workspace(),
         )
-        w = SbuildWorkflow(wr)
-        self.assertEqual(w.get_label(), "signed-source|build a package")
+
+        workflow = SbuildWorkflow(wr)
+        self.assertEqual(workflow.get_label(), "sbuild")
