@@ -409,3 +409,24 @@ class WorkflowTests(TestCase):
 
         # Returned the same as had been created
         self.assertEqual(wr_created, wr_returned)
+
+    @preserve_task_registry()
+    def test_get_label(self) -> None:
+        workflow_root = self.playground.create_workflow()
+
+        class ExampleWorkflow(
+            SampleWorkflow[BaseWorkflowData, BaseDynamicTaskData]
+        ):
+            """Concrete workflow instance to use for tests."""
+
+            def populate(self) -> None:
+                """Unused abstract method from Workflow."""
+                raise NotImplementedError()
+
+        workflow = ExampleWorkflow(workflow_root)
+
+        self.assertEqual(workflow.get_label(), "noop-template")
+
+        workflow_root.workflow_data_json = {"workflow_template_name": None}
+
+        self.assertEqual(workflow.get_label(), "noop")
