@@ -16,25 +16,37 @@ from typing import Any, TYPE_CHECKING
 from debusine.artifacts import SourcePackage
 from debusine.artifacts.models import (
     ArtifactCategory,
+    BareDataCategory,
     CollectionCategory,
     DebianBinaryPackage,
     DebianBinaryPackages,
     DebianPackageBuildLog,
     DebianSourcePackage,
+    DebianSystemTarball,
     DebianUpload,
+    DebusinePromise,
     get_source_package_name,
 )
 from debusine.client.models import LookupChildType
-from debusine.db.models import Artifact, ArtifactRelation, CollectionItem
+from debusine.db.models import (
+    Artifact,
+    ArtifactRelation,
+    CollectionItem,
+    TaskDatabase,
+)
 from debusine.server.collections.lookup import (
     LookupResult,
     lookup_multiple,
     lookup_single,
     reconstruct_lookup,
 )
-from debusine.server.workflows.base import ArtifactHasNoArchitecture
-from debusine.tasks import BaseTask, TaskConfigError
-from debusine.tasks.models import LookupMultiple, LookupSingle
+from debusine.tasks import BaseTask, TaskConfigError, get_environment
+from debusine.tasks.models import (
+    BackendType,
+    ExtraRepository,
+    LookupMultiple,
+    LookupSingle,
+)
 
 if TYPE_CHECKING:
     from debusine.server.workflows import Workflow
@@ -49,8 +61,8 @@ def source_package(workflow: "Workflow[Any, Any]") -> Artifact:
     ``workflow.data.input.source_artifact``, otherwise
     ``workflow.data.source_artifact``.
 
-    If the source artifact is a :ref:`debian:upload <artifact-upload>`,
-    returns its :ref:`debian:source-package <artifact-source-package>`.
+    If the source artifact is a :artifact:`debian:upload`, returns its
+    :artifact:`debian:source-package`.
     """
     if hasattr(workflow.data, "input"):
         lookup = workflow.data.input.source_artifact
@@ -76,6 +88,33 @@ def source_package_data(workflow: "Workflow[Any, Any]") -> DebianSourcePackage:
     return SourcePackage.create_data(source_package(workflow).data)
 
 
+def lookup_result_artifact_category(result: LookupResult) -> str:
+    """
+    Get artifact category from result of looking up an artifact.
+
+    The result may be either an artifact or a promise.
+    """
+    if (
+        result.result_type == CollectionItem.Types.ARTIFACT
+        and result.artifact is not None
+    ):
+        return result.artifact.category
+    elif (
+        result.result_type == CollectionItem.Types.BARE
+        and result.collection_item is not None
+        and result.collection_item.category == BareDataCategory.PROMISE
+    ):
+        return DebusinePromise(**result.collection_item.data).promise_category
+    else:
+        raise ValueError(
+            f"Cannot determine artifact category for lookup result: {result}"
+        )
+
+
+class ArtifactHasNoArchitecture(Exception):
+    """Raised if it's not possible to determine the artifact's architecture."""
+
+
 def lookup_result_architecture(result: LookupResult) -> str:
     """Get architecture from result of looking up an artifact."""
     architecture: str | None
@@ -83,7 +122,7 @@ def lookup_result_architecture(result: LookupResult) -> str:
     if result.artifact is not None:
         artifact_data = result.artifact.create_data()
         match artifact_data:
-            case DebianBinaryPackages():
+            case DebianBinaryPackages() | DebianPackageBuildLog():
                 architecture = artifact_data.architecture
             case DebianBinaryPackage():
                 architecture = artifact_data.deb_fields.get("Architecture")
@@ -91,23 +130,51 @@ def lookup_result_architecture(result: LookupResult) -> str:
                 architecture = artifact_data.changes_fields.get("Architecture")
             case _:
                 raise ArtifactHasNoArchitecture(f"{type(artifact_data)}")
-
-        assert isinstance(architecture, str)
-        return architecture
-
     elif result.collection_item is not None:
         architecture = result.collection_item.data.get("architecture")
-        if isinstance(architecture, str):
-            return architecture
-
     else:
         raise ValueError(
             "Unexpected result: must have collection_item or artifact"
         )
 
-    raise ValueError(
-        f"Cannot determine architecture for lookup result: {result}"
-    )
+    if not isinstance(architecture, str):
+        raise ValueError(
+            f"Cannot determine architecture for lookup result: {result}"
+        )
+
+    return architecture
+
+
+class ArtifactHasNoBinaryPackageName(Exception):
+    """Raised if it's not possible to determine the artifact's binary name."""
+
+
+def lookup_result_binary_package_name(result: LookupResult) -> str:
+    """Get binary package name from result of looking up an artifact."""
+    binary_package_name: str | None
+
+    if result.artifact is not None:
+        artifact_data = result.artifact.create_data()
+        match artifact_data:
+            case DebianBinaryPackage():
+                binary_package_name = artifact_data.deb_fields.get("Package")
+            case _:
+                raise ArtifactHasNoBinaryPackageName(f"{type(artifact_data)}")
+    elif result.collection_item is not None:
+        binary_package_name = result.collection_item.data.get(
+            "binary_package_name"
+        )
+    else:
+        raise ValueError(
+            "Unexpected result: must have collection_item or artifact"
+        )
+
+    if not isinstance(binary_package_name, str):
+        raise ValueError(
+            f"Cannot determine binary package name for lookup result: {result}"
+        )
+
+    return binary_package_name
 
 
 def filter_artifact_lookup_by_arch(
@@ -124,7 +191,7 @@ def filter_artifact_lookup_by_arch(
         workflow_root=workflow_root,
         expect_type=LookupChildType.ARTIFACT_OR_PROMISE,
     )
-    relevant: list[LookupSingle] = []
+    relevant: list[str] = []
     for result in results:
         arch_in_lookup = lookup_result_architecture(result)
         if arch_in_lookup in architectures:
@@ -161,7 +228,7 @@ def locate_debian_source_package(
     """
     Accept a debian:upload or debian:source-package in a workflow.
 
-    Resolve to the :ref:`debian:source-package <artifact-source-package>`.
+    Resolve to the :artifact:`debian:source-package`.
     """
     BaseTask.ensure_artifact_categories(
         configuration_key=configuration_key,
@@ -211,11 +278,9 @@ def locate_debian_source_package_lookup(
     """
     Return a lookup to a debian:source-package.
 
-    If the specified lookup returns a :ref:`debian:source-package
-    <artifact-source-package>`, return it.  If it returns a
-    :ref:`debian:upload <artifact-upload>`, find the related
-    :ref:`debian:source-package <artifact-source-package>` and return a
-    lookup to it.
+    If the specified lookup returns a :artifact:`debian:source-package`,
+    return it.  If it returns a :artifact:`debian:upload`, find the related
+    :artifact:`debian:source-package` and return a lookup to it.
     """
     artifact = lookup_single(
         lookup,
@@ -331,6 +396,54 @@ def get_available_architectures(
     ):
         architectures.add(result.artifact.data.get("architecture"))
 
+    if not architectures:
+        raise TaskConfigError(
+            f"Unable to find any environments for {vendor}:{codename}"
+        )
+
     architectures.add("all")
 
     return architectures
+
+
+def configure_for_overlay_suite(
+    workflow: "Workflow[Any, Any]",
+    *,
+    extra_repositories: list[ExtraRepository] | None,
+    vendor: str,
+    codename: str,
+    environment: LookupSingle,
+    backend: BackendType,
+    architecture: str,
+    try_variant: str,
+) -> list[ExtraRepository] | None:
+    """Return any needed extra repository to use an overlay suite."""
+    match (vendor, codename):
+        case ("debian", "experimental"):
+            pass
+        case _:
+            return extra_repositories
+
+    if extra_repositories is None:
+        extra_repositories = []
+
+    task_database = TaskDatabase(workflow.work_request)
+    environment_artifact = get_environment(
+        task_database,
+        environment,
+        architecture=architecture,
+        backend=backend,
+        default_category=CollectionCategory.ENVIRONMENTS,
+        try_variant=try_variant,
+    )
+    assert isinstance(environment_artifact.data, DebianSystemTarball)
+    mirror = environment_artifact.data.mirror
+    components = environment_artifact.data.components
+
+    return extra_repositories + [
+        ExtraRepository(
+            url=mirror,
+            suite=codename,
+            components=components,
+        )
+    ]
