@@ -23,17 +23,21 @@ from nacl.public import PrivateKey
 
 from debusine.artifacts import (
     LocalArtifact,
-    SigningKeyArtifact,
     SourcePackage,
     Upload,
     WorkRequestDebugLogs,
 )
 from debusine.artifacts.models import (
     ArtifactCategory,
+    DebianUpload,
     EmptyArtifactData,
-    KeyPurpose,
 )
-from debusine.client.models import ArtifactResponse, RemoteArtifact
+from debusine.assets.models import AssetCategory
+from debusine.client.models import (
+    ArtifactResponse,
+    AssetPermissionCheckResponse,
+    RemoteArtifact,
+)
 from debusine.signing.db.models import Key
 from debusine.signing.gnupg import gpg_ephemeral_context
 from debusine.signing.models import SigningMode
@@ -41,8 +45,10 @@ from debusine.signing.tasks import Debsign
 from debusine.signing.tasks.models import DebsignDynamicData
 from debusine.signing.tasks.tests.test_sign import SignTestMixin
 from debusine.tasks.models import WorkerType
+from debusine.tasks.server import ArtifactInfo
 from debusine.tasks.tests.helper_mixin import FakeTaskDatabase
 from debusine.test import TestCase
+from debusine.test.test_utils import create_remote_artifact
 
 
 class DebsignTests(SignTestMixin, TestCase, DjangoTestCase):
@@ -103,22 +109,36 @@ class DebsignTests(SignTestMixin, TestCase, DjangoTestCase):
 
     def test_compute_dynamic_data(self) -> None:
         """Dynamic data receives relevant artifact IDs."""
-        key_lookup = "bookworm@debian:suite-signing-keys/key:openpgp"
         task_db = FakeTaskDatabase(
             single_lookups={
-                # unsigned
-                (2, None): 2,
-                # key
-                (key_lookup, None): 1,
+                (2, None): ArtifactInfo(
+                    id=2,
+                    category=ArtifactCategory.UPLOAD,
+                    data=DebianUpload(
+                        type="dpkg",
+                        changes_fields={
+                            "Architecture": "source",
+                            "Files": [{"name": "hello_1.0.dsc"}],
+                        },
+                    ),
+                ),
             }
         )
 
-        task = Debsign(task_data={"unsigned": 2, "key": key_lookup})
+        task = Debsign(task_data={"unsigned": 2, "key": "ABC123"})
         self.assertEqual(
             task.compute_dynamic_data(task_db),
-            DebsignDynamicData(unsigned_id=2, key_id=1),
+            DebsignDynamicData(unsigned_id=2),
         )
 
+    def test_get_input_artifacts_ids(self) -> None:
+        """Test get_input_artifacts_ids."""
+        task = Debsign(task_data={"unsigned": 1, "key": "ABC123"})
+        self.assertEqual(task.get_input_artifacts_ids(), [])
+
+        task.dynamic_data = DebsignDynamicData(unsigned_id=1)
+        self.assertEqual(task.get_input_artifacts_ids(), [1])
+
     def test_fetch_input_wrong_category(self) -> None:
         """fetch_input checks the category of the unsigned artifact."""
         temp_path = self.create_temporary_directory()
@@ -128,21 +148,18 @@ class DebsignTests(SignTestMixin, TestCase, DjangoTestCase):
             name="foo", version="1.0", files=[dsc_path]
         )
         source_package_id = 1
-        response = self.make_artifact_response(
+        source_package_response = self.make_artifact_response(
             source_package, source_package_id
         )
 
-        def download_artifact(
-            artifact_id: int, destination: Path, **kwargs: Any  # noqa: U100
-        ) -> ArtifactResponse:
-            return response
-
         task = Debsign(
-            task_data={"unsigned": source_package_id, "key": 2},
-            dynamic_task_data={"unsigned_id": source_package_id, "key_id": 2},
+            task_data={"unsigned": source_package_id, "key": "ABC123"},
+            dynamic_task_data={"unsigned_id": source_package_id},
         )
+        task.work_request_id = 1
+        task.workspace_name = "workspace"
         debusine_mock = self.mock_debusine(task)
-        debusine_mock.download_artifact.side_effect = download_artifact
+        debusine_mock.download_artifact.return_value = source_package_response
         debug_log_files_directory = TemporaryDirectory(prefix="debusine-tests-")
         self.addCleanup(debug_log_files_directory.cleanup)
         task._debug_log_files_directory = debug_log_files_directory
@@ -155,41 +172,28 @@ class DebsignTests(SignTestMixin, TestCase, DjangoTestCase):
             "Expected unsigned artifact of category debian:upload; got "
             "debian:source-package\n",
         )
+        debusine_mock.download_artifact.assert_called_once_with(
+            source_package_id, destination, tarball=mock.ANY
+        )
 
     def test_fetch_input_missing_key(self) -> None:
         """fetch_input requires the signing key to exist in the database."""
         fingerprint = "0" * 64
-        signing_key = SigningKeyArtifact.create(
-            KeyPurpose.OPENPGP, fingerprint=fingerprint, public_key=b""
-        )
         directory = self.create_temporary_directory()
         changes_file = directory / "foo.changes"
         self.write_changes_file(changes_file, [])
         upload = Upload.create(changes_file=changes_file)
-        signing_key_id = 1
         upload_id = 2
-        responses = {
-            response.id: response
-            for response in (
-                self.make_artifact_response(signing_key, signing_key_id),
-                self.make_artifact_response(upload, upload_id),
-            )
-        }
-
-        def download_artifact(
-            artifact_id: int, destination: Path, **kwargs: Any  # noqa: U100
-        ) -> ArtifactResponse:
-            return responses[artifact_id]
+        upload_response = self.make_artifact_response(upload, upload_id)
 
         task = Debsign(
-            task_data={"unsigned": upload_id, "key": signing_key_id},
-            dynamic_task_data={
-                "unsigned_id": upload_id,
-                "key_id": signing_key_id,
-            },
+            task_data={"unsigned": upload_id, "key": fingerprint},
+            dynamic_task_data={"unsigned_id": upload_id},
         )
+        task.work_request_id = 1
+        task.workspace_name = "workspace"
         debusine_mock = self.mock_debusine(task)
-        debusine_mock.download_artifact.side_effect = download_artifact
+        debusine_mock.download_artifact.return_value = upload_response
         debug_log_files_directory = TemporaryDirectory(prefix="debusine-tests-")
         self.addCleanup(debug_log_files_directory.cleanup)
         task._debug_log_files_directory = debug_log_files_directory
@@ -201,6 +205,55 @@ class DebsignTests(SignTestMixin, TestCase, DjangoTestCase):
             Path(debug_log_files_directory.name, "fetch_input.log").read_text(),
             f"Signing key openpgp:{fingerprint} does not exist\n",
         )
+        debusine_mock.download_artifact.assert_called_once_with(
+            upload_id, destination, tarball=mock.ANY
+        )
+
+    def test_fetch_input_permission_denied(self) -> None:
+        key = Key.objects.create(
+            purpose=Key.Purpose.OPENPGP,
+            fingerprint="0" * 64,
+            private_key={},
+            public_key=b"",
+        )
+        directory = self.create_temporary_directory()
+        changes_file = directory / "foo.changes"
+        self.write_changes_file(changes_file, [])
+        upload = Upload.create(changes_file=changes_file)
+        upload_id = 2
+        upload_response = self.make_artifact_response(upload, upload_id)
+
+        task = Debsign(
+            task_data={"unsigned": upload_id, "key": key.fingerprint},
+            dynamic_task_data={"unsigned_id": upload_id},
+        )
+        task.work_request_id = 1
+        task.workspace_name = "workspace"
+        debusine_mock = self.mock_debusine(task)
+        debusine_mock.download_artifact.return_value = upload_response
+        debusine_mock.asset_permission_check.return_value = (
+            AssetPermissionCheckResponse(
+                has_permission=False,
+                username="testuser",
+                user_id=123,
+                resource={},
+            )
+        )
+        debug_log_files_directory = TemporaryDirectory(prefix="debusine-tests-")
+        self.addCleanup(debug_log_files_directory.cleanup)
+        task._debug_log_files_directory = debug_log_files_directory
+        destination = self.create_temporary_directory()
+
+        self.assertFalse(task.fetch_input(destination))
+
+        self.assertEqual(
+            Path(debug_log_files_directory.name, "fetch_input.log").read_text(),
+            f"User testuser does not have permission to sign "
+            f"{upload_id} with openpgp:{key.fingerprint}.\n",
+        )
+        debusine_mock.download_artifact.assert_called_once_with(
+            upload_id, destination, tarball=mock.ANY
+        )
 
     def test_run_error(self) -> None:
         """run() raises an exception if signing fails."""
@@ -213,17 +266,23 @@ class DebsignTests(SignTestMixin, TestCase, DjangoTestCase):
         download_directory = self.create_temporary_directory()
         execute_directory = self.create_temporary_directory()
         (changes_file := download_directory / "foo.changes").touch()
-        task = Debsign(task_data={"unsigned": 2, "key": 1})
+        task = Debsign(task_data={"unsigned": 2, "key": key.fingerprint})
         task.work_request_id = 1
         task._files = self.make_files_response({"foo.changes": changes_file})
         task._key = key
+        task._permission_check = AssetPermissionCheckResponse(
+            has_permission=True, username="testuser", user_id=123, resource={}
+        )
         debug_log_files_directory = TemporaryDirectory(prefix="debusine-tests-")
         self.addCleanup(debug_log_files_directory.cleanup)
         task._debug_log_files_directory = debug_log_files_directory
         task.prepare_to_run(download_directory, execute_directory)
 
         with (
-            mock.patch.object(key, "sign", side_effect=ValueError("Boom")),
+            mock.patch(
+                "debusine.signing.tasks.debsign.sign",
+                side_effect=ValueError("Boom"),
+            ),
             self.assertRaisesRegex(ValueError, "Boom"),
         ):
             task.run(execute_directory)
@@ -241,16 +300,22 @@ class DebsignTests(SignTestMixin, TestCase, DjangoTestCase):
         download_directory = self.create_temporary_directory()
         execute_directory = self.create_temporary_directory()
         (changes_file := download_directory / "foo.changes").touch()
-        task = Debsign(task_data={"unsigned": 2, "key": 1})
+        task = Debsign(task_data={"unsigned": 2, "key": key.fingerprint})
         task.work_request_id = 1
         task._files = self.make_files_response({"foo.changes": changes_file})
         task._key = key
+        task._permission_check = AssetPermissionCheckResponse(
+            has_permission=True,
+            username="testuser",
+            user_id=123,
+            resource={"package": "foo"},
+        )
         debug_log_files_directory = TemporaryDirectory(prefix="debusine-tests-")
         self.addCleanup(debug_log_files_directory.cleanup)
         task._debug_log_files_directory = debug_log_files_directory
         task.prepare_to_run(download_directory, execute_directory)
 
-        with mock.patch.object(key, "sign") as mock_sign:
+        with mock.patch("debusine.signing.tasks.debsign.sign") as mock_sign:
             self.assertTrue(task.run(execute_directory))
 
         self.assertEqual(
@@ -258,10 +323,14 @@ class DebsignTests(SignTestMixin, TestCase, DjangoTestCase):
             execute_directory / "output" / "foo.changes",
         )
         mock_sign.assert_called_once_with(
+            [key],
             execute_directory / "input" / "foo.changes",
             execute_directory / "output" / "foo.changes",
             SigningMode.DEBSIGN,
             1,
+            username="testuser",
+            user_id=123,
+            resource={"package": "foo"},
             log_file=mock.ANY,
         )
 
@@ -269,9 +338,7 @@ class DebsignTests(SignTestMixin, TestCase, DjangoTestCase):
 class DebsignIntegrationTests(SignTestMixin, TestCase, TransactionTestCase):
     """Integration tests for the :py:class:`Debsign` task."""
 
-    def generate_signing_key(
-        self, service_private_key: PrivateKey
-    ) -> tuple[Key, SigningKeyArtifact]:
+    def generate_signing_key(self, service_private_key: PrivateKey) -> Key:
         """Generate a signing key."""
         with (
             override_settings(
@@ -282,12 +349,7 @@ class DebsignIntegrationTests(SignTestMixin, TestCase, TransactionTestCase):
             key = Key.objects.generate(
                 Key.Purpose.OPENPGP, "An OpenPGP key", 1, log_file
             )
-        signing_key = SigningKeyArtifact.create(
-            KeyPurpose.OPENPGP,
-            fingerprint=key.fingerprint,
-            public_key=key.public_key,
-        )
-        return key, signing_key
+        return key
 
     def make_upload(self) -> tuple[Path, Path, Upload]:
         """Prepare a minimal upload; return (dsc, changes, artifact)."""
@@ -302,17 +364,10 @@ class DebsignIntegrationTests(SignTestMixin, TestCase, TransactionTestCase):
     def test_execute_success(self) -> None:
         """Integration test: sign using a real OpenPGP key."""
         service_private_key = PrivateKey.generate()
-        key, signing_key = self.generate_signing_key(service_private_key)
-        signing_key_id = 1
+        key = self.generate_signing_key(service_private_key)
         dsc_path, changes_path, upload = self.make_upload()
         upload_id = 2
-        responses = {
-            response.id: response
-            for response in (
-                self.make_artifact_response(signing_key, signing_key_id),
-                self.make_artifact_response(upload, upload_id),
-            )
-        }
+        upload_response = self.make_artifact_response(upload, upload_id)
         uploaded_paths: list[Path] = []
         uploaded_upload: Upload | None = None
         output_path = self.create_temporary_directory()
@@ -324,15 +379,14 @@ class DebsignIntegrationTests(SignTestMixin, TestCase, TransactionTestCase):
         def download_artifact(
             artifact_id: int, destination: Path, **kwargs: Any  # noqa: U100
         ) -> ArtifactResponse:
-            if artifact_id == upload_id:
-                shutil.copy(dsc_path, destination)
-                shutil.copy(changes_path, destination)
-            return responses[artifact_id]
+            assert artifact_id == upload_id
+            shutil.copy(dsc_path, destination)
+            shutil.copy(changes_path, destination)
+            return upload_response
 
         def upload_artifact(
             local_artifact: LocalArtifact[Any], **kwargs: Any
         ) -> RemoteArtifact:
-            nonlocal uploaded_paths
             nonlocal uploaded_upload
             for path in local_artifact.files.values():
                 uploaded_paths.append(path)
@@ -344,21 +398,28 @@ class DebsignIntegrationTests(SignTestMixin, TestCase, TransactionTestCase):
                     if path.name.endswith(".changes")
                 ]
                 uploaded_upload = Upload.create(changes_file=changes_path)
-            return RemoteArtifact(
+            return create_remote_artifact(
                 id=uploaded_artifact_ids[local_artifact.category],
                 workspace="System",
             )
 
         task = Debsign(
-            task_data={"unsigned": upload_id, "key": signing_key_id},
+            task_data={"unsigned": upload_id, "key": key.fingerprint},
             dynamic_task_data={
                 "unsigned_id": upload_id,
-                "key_id": signing_key_id,
             },
         )
         debusine_mock = self.mock_debusine(task)
         debusine_mock.download_artifact.side_effect = download_artifact
         debusine_mock.upload_artifact.side_effect = upload_artifact
+        debusine_mock.asset_permission_check.return_value = (
+            AssetPermissionCheckResponse(
+                has_permission=True,
+                username="testuser",
+                user_id=123,
+                resource={},
+            )
+        )
         task.work_request_id = 1
         task.workspace_name = "System"
         debug_log_files_directory = TemporaryDirectory(prefix="debusine-tests-")
@@ -370,6 +431,15 @@ class DebsignIntegrationTests(SignTestMixin, TestCase, TransactionTestCase):
         ):
             self.assertTrue(task.execute())
 
+        debusine_mock.asset_permission_check.assert_called_once_with(
+            asset_category=AssetCategory.SIGNING_KEY,
+            asset_slug=f"openpgp:{key.fingerprint}",
+            permission_name="sign_with",
+            artifact_id=upload_id,
+            work_request_id=task.work_request_id,
+            workspace=task.workspace_name,
+        )
+
         # The task uploaded an artifact with the expected properties.
         self.assertEqual(len(uploaded_paths), 3)
         debusine_mock.upload_artifact.assert_has_calls(
@@ -385,25 +455,19 @@ class DebsignIntegrationTests(SignTestMixin, TestCase, TransactionTestCase):
                                 / "cmd-output.log"
                             )
                         },
+                        content_types={
+                            "cmd-output.log": "text/plain; charset=utf-8"
+                        },
                     ),
                     workspace="System",
                     work_request=1,
                 ),
             ]
         )
-        debusine_mock.relation_create.assert_has_calls(
-            [
-                mock.call(
-                    uploaded_artifact_ids[ArtifactCategory.UPLOAD],
-                    upload_id,
-                    "relates-to",
-                ),
-                mock.call(
-                    uploaded_artifact_ids[ArtifactCategory.UPLOAD],
-                    signing_key_id,
-                    "relates-to",
-                ),
-            ]
+        debusine_mock.relation_create.assert_any_call(
+            uploaded_artifact_ids[ArtifactCategory.UPLOAD],
+            upload_id,
+            "relates-to",
         )
 
         # The file was properly signed.
@@ -428,31 +492,26 @@ class DebsignIntegrationTests(SignTestMixin, TestCase, TransactionTestCase):
     def test_execute_failure(self) -> None:
         """If execution fails, the task does not upload an output artifact."""
         service_private_key = PrivateKey.generate()
-        _, signing_key = self.generate_signing_key(service_private_key)
-        signing_key_id = 1
+        key = self.generate_signing_key(service_private_key)
         dsc_path, changes_path, upload = self.make_upload()
         upload_id = 2
-        responses = {
-            response.id: response
-            for response in (
-                self.make_artifact_response(signing_key, signing_key_id),
-                self.make_artifact_response(upload, upload_id),
-            )
-        }
+        upload_response = self.make_artifact_response(upload, upload_id)
 
         def download_artifact(
             artifact_id: int, destination: Path, **kwargs: Any  # noqa: U100
         ) -> ArtifactResponse:
-            if artifact_id == upload_id:
-                shutil.copy(dsc_path, destination)
-                shutil.copy(changes_path, destination)
-            return responses[artifact_id]
+            assert artifact_id == upload_id
+            shutil.copy(dsc_path, destination)
+            shutil.copy(changes_path, destination)
+            return upload_response
 
         task = Debsign(
-            task_data={"unsigned": upload_id, "key": signing_key_id},
+            task_data={
+                "unsigned": upload_id,
+                "key": key.fingerprint,
+            },
             dynamic_task_data={
                 "unsigned_id": upload_id,
-                "key_id": signing_key_id,
             },
         )
         debusine_mock = self.mock_debusine(task)
@@ -463,17 +522,21 @@ class DebsignIntegrationTests(SignTestMixin, TestCase, TransactionTestCase):
         self.addCleanup(debug_log_files_directory.cleanup)
         task._debug_log_files_directory = debug_log_files_directory
 
-        with mock.patch.object(
-            Key, "sign", side_effect=Exception("Signing failed")
+        with mock.patch(
+            "debusine.signing.tasks.debsign.sign",
+            side_effect=Exception("Signing failed"),
         ):
             self.assertFalse(task.execute())
 
-        # Only a debug logs artifact was uploaded, not a signing key artifact.
+        # Only a debug logs artifact was uploaded, not any signing output
         debusine_mock.upload_artifact.assert_called_once_with(
             WorkRequestDebugLogs(
                 category=WorkRequestDebugLogs._category,
                 data=EmptyArtifactData(),
                 files={
+                    "stages.log": (
+                        Path(debug_log_files_directory.name) / "stages.log"
+                    ),
                     "cmd-output.log": (
                         Path(debug_log_files_directory.name) / "cmd-output.log"
                     ),
@@ -481,6 +544,11 @@ class DebsignIntegrationTests(SignTestMixin, TestCase, TransactionTestCase):
                         Path(debug_log_files_directory.name) / "execution.log"
                     ),
                 },
+                content_types={
+                    "stages.log": "text/plain; charset=utf-8",
+                    "cmd-output.log": "text/plain; charset=utf-8",
+                    "execution.log": "text/plain; charset=utf-8",
+                },
             ),
             workspace="System",
             work_request=1,
