@@ -9,24 +9,41 @@
 
 """Unit tests for the signing no-operation task."""
 
-from unittest import TestCase
+from tempfile import TemporaryDirectory
+from unittest import mock
+
+from django.test import TestCase
 
 from debusine.signing.tasks import SigningNoop
 from debusine.tasks import TaskConfigError
 
 
-class SigningNoopTaskTests(TestCase):
-    """Test the noop task."""
+class SigningNoopTests(TestCase):
+    """Test the SigningNoop task."""
 
     def test_execute_result_true(self) -> None:
         """If result=True, the task returns True."""
         noop = SigningNoop({"result": True})
-        self.assertTrue(noop.execute())
+        noop._debug_log_files_directory = TemporaryDirectory(
+            prefix="debusine-tests-"
+        )
+        self.addCleanup(noop._debug_log_files_directory.cleanup)
+        with mock.patch.object(
+            noop, "_upload_work_request_debug_logs", autospec=True
+        ):
+            self.assertTrue(noop.execute())
 
     def test_execute_result_false(self) -> None:
         """If result=False, the task returns False."""
         noop = SigningNoop({"result": False})
-        self.assertFalse(noop.execute())
+        noop._debug_log_files_directory = TemporaryDirectory(
+            prefix="debusine-tests-"
+        )
+        self.addCleanup(noop._debug_log_files_directory.cleanup)
+        with mock.patch.object(
+            noop, "_upload_work_request_debug_logs", autospec=True
+        ):
+            self.assertFalse(noop.execute())
 
     def test_no_additional_properties(self) -> None:
         """Assert no additional properties."""
