@@ -308,6 +308,24 @@ class BaseTask(Generic[TD, DTD], metaclass=ABCMeta):
                 f"Valid categories: {valid_categories}"
             )
 
+    @staticmethod
+    def ensure_collection_category(
+        *, configuration_key: str, category: str, expected: CollectionCategory
+    ) -> None:
+        """
+        Validate that the collection's category is as expected.
+
+        :param configuration_key: Optional key to identify the source of the
+          artifact. Provides additional context in error messages.
+        :param category: The category to validate.
+        :param expected: The expected collection category.
+        """
+        if category != expected:
+            raise TaskConfigError(
+                f"{configuration_key}: unexpected collection category: "
+                f"'{category}'. Expected: '{expected}'"
+            )
+
     @overload
     def open_debug_log_file(
         self, filename: str, *, mode: "OpenTextModeWriting" = "a"
