@@ -30,11 +30,12 @@ class QemuExecutorTests(ExternalTaskHelperMixin[Noop], TestCase):
 
     def setUp(self) -> None:
         """Mock the Debusine API for tests."""
+        super().setUp()
         self.debusine_api = MagicMock(spec=Debusine)
         self.image_artifact = self.mock_image_download(
             self.debusine_api, system_image=True
         )
-        self.image_artifact.data["pkglist"]["python3"] = "3.0"
+        self.image_artifact.data["pkglist"]["python3-minimal"] = "3.0"
 
         self.executor = QemuExecutor(self.debusine_api, 42)
 
@@ -85,11 +86,14 @@ class QemuExecutorTests(ExternalTaskHelperMixin[Noop], TestCase):
         self.assertEqual(response, str(expected_path))
 
     def test_download_image_requires_python3(self) -> None:
-        """Test that download_image requires python3 in the image."""
-        del self.image_artifact.data["pkglist"]["python3"]
+        """Test that download_image requires python3-minimal in the image."""
+        del self.image_artifact.data["pkglist"]["python3-minimal"]
         with self.assertRaisesRegex(
             ImageImportError,
-            "Image doesn't contain an python3, required by qemu backend.",
+            (
+                "Image doesn't contain a python3-minimal, required by qemu "
+                "backend."
+            ),
         ):
             self.executor.download_image()
 
