@@ -145,6 +145,16 @@ class ExtractForSigning(
             subject=binary_package_name,
         )
 
+    def get_input_artifacts_ids(self) -> list[int]:
+        """Return the list of input artifact IDs used by this task."""
+        if not self.dynamic_data:
+            return []
+        return [
+            self.dynamic_data.environment_id,
+            self.dynamic_data.input_template_artifact_id,
+            *self.dynamic_data.input_binary_artifacts_ids,
+        ]
+
     def fetch_input(self, destination: Path) -> bool:
         """Download the required artifacts."""
         assert self.dynamic_data is not None
