@@ -15,7 +15,7 @@ from datetime import datetime
 from enum import StrEnum
 from itertools import groupby
 from pathlib import Path
-from typing import Annotated, Any, Literal, TypeAlias, Union
+from typing import Annotated, Any, ClassVar, Literal, TypeAlias, Union
 
 import debian.deb822 as deb822
 
@@ -544,8 +544,12 @@ class BaseTaskData(BaseTaskDataModel):
     validation.
     """
 
+    DEFAULT_TASK_CONFIGURATION_LOOKUP: ClassVar[str] = (
+        "default@debusine:task-configuration"
+    )
+
     #: debusine:task-configuration collection to use to configure the task
-    task_configuration: LookupSingle | None = None
+    task_configuration: LookupSingle | None = DEFAULT_TASK_CONFIGURATION_LOOKUP
 
 
 class BaseTaskDataWithExecutor(BaseTaskData):
@@ -1334,6 +1338,12 @@ class MergeUploadsDynamicData(BaseDynamicTaskData):
     input_uploads_ids: list[int]
 
 
+class DebianPipelineWorkflowDynamicData(BaseDynamicTaskData):
+    """Dynamic data for the DebianPipeline workflow."""
+
+    input_source_artifact_id: int | None = None
+
+
 # Workarounds for https://github.com/yaml/pyyaml/issues/722
 yaml.SafeDumper.add_multi_representer(
     StrEnum,
