@@ -11,11 +11,14 @@
 
 from pathlib import Path
 
-from debusine.tasks import BaseExternalTask
+from debusine.tasks import BaseExternalTask, DefaultDynamicData
 from debusine.tasks.models import BaseDynamicTaskData, NoopData
 
 
-class Noop(BaseExternalTask[NoopData, BaseDynamicTaskData]):
+class Noop(
+    BaseExternalTask[NoopData, BaseDynamicTaskData],
+    DefaultDynamicData[NoopData],
+):
     """
     Task that returns a boolean (execute() depending on the result field).
 
