@@ -10,6 +10,7 @@
 """Unit tests for the mergeuploads task support on the worker."""
 
 import hashlib
+from collections.abc import Iterable
 from pathlib import Path
 from unittest import mock
 from unittest.mock import call
@@ -22,11 +23,7 @@ from debusine.artifacts.models import (
     EmptyArtifactData,
 )
 from debusine.artifacts.playground import ArtifactPlayground
-from debusine.client.models import (
-    LookupResultType,
-    LookupSingleResponse,
-    RemoteArtifact,
-)
+from debusine.client.models import LookupResultType, LookupSingleResponse
 from debusine.tasks import MergeUploads, TaskConfigError
 from debusine.tasks.mergeuploads import MergeUploadsError
 from debusine.tasks.models import LookupMultiple, MergeUploadsDynamicData
@@ -36,7 +33,10 @@ from debusine.tasks.tests.helper_mixin import (
     FakeTaskDatabase,
 )
 from debusine.test import TestCase
-from debusine.test.test_utils import create_artifact_response
+from debusine.test.test_utils import (
+    create_artifact_response,
+    create_remote_artifact,
+)
 
 
 class MergeUploadsTests(ExternalTaskHelperMixin[MergeUploads], TestCase):
@@ -44,13 +44,45 @@ class MergeUploadsTests(ExternalTaskHelperMixin[MergeUploads], TestCase):
 
     SAMPLE_TASK_DATA = {"input": {"uploads": [421, 666]}}
 
-    def setUp(self) -> None:  # noqa: D102
+    def setUp(self) -> None:
+        super().setUp()
         self.configure_task()
 
     def tearDown(self) -> None:
         """Delete directory to avoid ResourceWarning with python -m unittest."""
         if self.task._debug_log_files_directory is not None:
             self.task._debug_log_files_directory.cleanup()
+        super().tearDown()
+
+    @staticmethod
+    def _make_files_field(paths: Iterable[Path]) -> list[dict[str, str]]:
+        """Make a valid ``Files`` field."""
+        return [
+            {
+                "md5sum": hashlib.md5(path.read_bytes()).hexdigest(),
+                "name": path.name,
+                "priority": "optional",
+                "section": "unknown",
+                "size": str(path.stat().st_size),
+            }
+            for path in paths
+        ]
+
+    @staticmethod
+    def _make_checksums_field(
+        hash_name: str, paths: Iterable[Path]
+    ) -> list[dict[str, str]]:
+        """Make a valid ``Checksums-*`` field."""
+        return [
+            {
+                "name": path.name,
+                hash_name: hashlib.new(
+                    hash_name, path.read_bytes()
+                ).hexdigest(),
+                "size": str(path.stat().st_size),
+            }
+            for path in paths
+        ]
 
     def test_compute_dynamic_data(self) -> None:
         """Dynamic data receives relevant artifact IDs."""
@@ -168,7 +200,16 @@ class MergeUploadsTests(ExternalTaskHelperMixin[MergeUploads], TestCase):
         ):
             self.task.compute_dynamic_data(task_db)
 
-    def test_configure_fails_with_missing_required_data(  # noqa: D102
+    def test_get_input_artifacts_ids(self) -> None:
+        """Test get_input_artifacts_ids."""
+        self.assertEqual(self.task.get_input_artifacts_ids(), [])
+
+        self.task.dynamic_data = MergeUploadsDynamicData(
+            input_uploads_ids=[2, 3],
+        )
+        self.assertEqual(self.task.get_input_artifacts_ids(), [2, 3])
+
+    def test_configure_fails_with_missing_required_data(
         self,
     ) -> None:
         with self.assertRaises(TaskConfigError):
@@ -179,6 +220,7 @@ class MergeUploadsTests(ExternalTaskHelperMixin[MergeUploads], TestCase):
         directory = self.create_temporary_directory()
         artifact1 = self.fake_debian_upload_artifact()
         artifact2 = self.fake_debian_upload_artifact()
+        artifact2.id += 1
         self.configure_task()
         self.task.work_request_id = 5
         self.task.dynamic_data = MergeUploadsDynamicData(
@@ -195,8 +237,8 @@ class MergeUploadsTests(ExternalTaskHelperMixin[MergeUploads], TestCase):
         self.assertTrue(result)
         fetch_artifact_mocked.assert_has_calls(
             [
-                mock.call(artifact1.id, directory),
-                mock.call(artifact2.id, directory),
+                mock.call(artifact1.id, directory / f"upload-{artifact1.id}"),
+                mock.call(artifact2.id, directory / f"upload-{artifact2.id}"),
             ]
         )
 
@@ -234,17 +276,37 @@ class MergeUploadsTests(ExternalTaskHelperMixin[MergeUploads], TestCase):
 
         fetch_artifact_mocked.assert_not_called()
 
+    def test_configure_for_execution_groups_changes_by_upload(self) -> None:
+        directory = self.create_temporary_directory()
+        changes_source = directory / "upload-9" / "foo_1.0_source.changes"
+        changes_amd64 = directory / "upload-10" / "foo_1.0_amd64.changes"
+        changes_i386 = directory / "upload-11" / "foo_1.0_i386.changes"
+        for changes in (changes_source, changes_amd64, changes_i386):
+            changes.parent.mkdir()
+            changes.touch()
+
+        self.task.configure_for_execution(directory)
+
+        self.assertEqual(
+            self.task._changes_paths,
+            [changes_source, changes_amd64, changes_i386],
+        )
+
     def test_merge_changes_unknown_format(self) -> None:
         """Only ``Format: 1.8`` is supported."""
+        execute_directory = self.create_temporary_directory()
         with self.assertRaisesRegex(
             MergeUploadsError, "Unknown .changes format: 1.7"
         ):
-            self.task.merge_changes([deb822.Changes({"Format": "1.7"})])
+            self.task.merge_changes(
+                execute_directory, [deb822.Changes({"Format": "1.7"})]
+            )
 
     def test_merge_changes_mismatched_simple_fields(self) -> None:
         """All ``Format``, ``Source``, and ``Version`` fields must match."""
         for field in ("Format", "Source", "Version"):
             with self.subTest(field=field):
+                execute_directory = self.create_temporary_directory()
                 all_changes = [
                     deb822.Changes(
                         {"Format": "1.8", "Source": "hello", "Version": "1.0"}
@@ -257,15 +319,17 @@ class MergeUploadsTests(ExternalTaskHelperMixin[MergeUploads], TestCase):
                     fr"{field} fields do not match: "
                     fr"\[{all_changes[0][field]!r}, 'mismatch'\]",
                 ):
-                    self.task.merge_changes(all_changes)
+                    self.task.merge_changes(execute_directory, all_changes)
 
     def test_merge_changes_mismatched_descriptions(self) -> None:
         """The descriptions for each binary package must match."""
+        execute_directory = self.create_temporary_directory()
         with self.assertRaisesRegex(
             MergeUploadsError,
             "Descriptions for mismatch do not match: 'left' != 'right'",
         ):
             self.task.merge_changes(
+                execute_directory,
                 [
                     deb822.Changes(
                         {
@@ -295,11 +359,12 @@ class MergeUploadsTests(ExternalTaskHelperMixin[MergeUploads], TestCase):
                             ),
                         }
                     ),
-                ]
+                ],
             )
 
     def test_merge_changes_mismatched_files(self) -> None:
         """The entries for a given file in ``Files`` must match."""
+        execute_directory = self.create_temporary_directory()
         with self.assertRaisesRegex(
             MergeUploadsError,
             r"Entries in Files for mismatch_1\.0_all.deb do not match: "
@@ -309,6 +374,7 @@ class MergeUploadsTests(ExternalTaskHelperMixin[MergeUploads], TestCase):
             r"'priority': 'optional', 'name': 'mismatch_1\.0_all.deb'\}",
         ):
             self.task.merge_changes(
+                execute_directory,
                 [
                     deb822.Changes(
                         {
@@ -336,7 +402,7 @@ class MergeUploadsTests(ExternalTaskHelperMixin[MergeUploads], TestCase):
                             ),
                         }
                     ),
-                ]
+                ],
             )
 
     def test_merge_changes_mismatched_checksums(self) -> None:
@@ -346,6 +412,7 @@ class MergeUploadsTests(ExternalTaskHelperMixin[MergeUploads], TestCase):
             ("Checksums-Sha256", "sha256"),
         ):
             with self.subTest(field=field):
+                execute_directory = self.create_temporary_directory()
                 with self.assertRaisesRegex(
                     MergeUploadsError,
                     fr"Entries in {field} for mismatch_1\.0_all.deb do not "
@@ -356,6 +423,7 @@ class MergeUploadsTests(ExternalTaskHelperMixin[MergeUploads], TestCase):
                     fr"'name': 'mismatch_1\.0_all.deb'\}}",
                 ):
                     self.task.merge_changes(
+                        execute_directory,
                         [
                             deb822.Changes(
                                 {
@@ -383,16 +451,18 @@ class MergeUploadsTests(ExternalTaskHelperMixin[MergeUploads], TestCase):
                                     ),
                                 }
                             ),
-                        ]
+                        ],
                     )
 
     def test_merge_changes_unsupported_checksum(self) -> None:
         """Unsupported ``Checksums-*`` fields are an error."""
+        execute_directory = self.create_temporary_directory()
         with self.assertRaisesRegex(
             MergeUploadsError,
             "Unsupported checksum field: Checksums-Unsupported",
         ):
             self.task.merge_changes(
+                execute_directory,
                 [
                     deb822.Changes(
                         {
@@ -402,9 +472,79 @@ class MergeUploadsTests(ExternalTaskHelperMixin[MergeUploads], TestCase):
                             "Checksums-Unsupported": "",
                         }
                     )
-                ]
+                ],
             )
 
+    def test_merge_changes(self) -> None:
+        """``merge_changes`` merges everything and symlinks referenced files."""
+        download_directory = self.create_temporary_directory()
+
+        (upload_source := download_directory / "upload-1").mkdir()
+        f_dsc = upload_source / "hello_1.0.dsc"
+        f_dsc.write_bytes(f_dsc.name.encode())
+        f_tar = upload_source / "hello_1.0.tar.xz"
+        f_tar.write_bytes(f_tar.name.encode())
+        f_changes_source = upload_source / "hello_1.0_source.changes"
+        ArtifactPlayground.write_deb822_file(
+            deb822.Changes,
+            f_changes_source,
+            [f_dsc, f_tar],
+            source="hello",
+            version="1.0",
+        )
+
+        (upload_all := download_directory / "upload-2").mkdir()
+        f_deb = upload_all / "hello_1.0_all.deb"
+        f_deb.write_bytes(f_deb.name.encode())
+        f_changes_all = upload_all / "hello_1.0_all.changes"
+        changes_all = ArtifactPlayground.write_deb822_file(
+            deb822.Changes,
+            f_changes_all,
+            [f_deb],
+            source="hello",
+            binaries=["hello"],
+            version="1.0",
+        )
+
+        self.task._changes_paths = [f_changes_source, f_changes_all]
+        execute_directory = self.create_temporary_directory()
+
+        merged_changes_path = self.task.merge_changes(
+            execute_directory,
+            [
+                deb822.Changes(f_changes_source.read_bytes()),
+                deb822.Changes(f_changes_all.read_bytes()),
+            ],
+        )
+
+        self.assertEqual(
+            deb822.Changes(merged_changes_path.read_bytes()),
+            {
+                "Architecture": "all source",
+                "Binary": "hello",
+                "Changes": changes_all["Changes"],
+                "Checksums-Sha1": self._make_checksums_field(
+                    "sha1", [f_dsc, f_tar, f_deb]
+                ),
+                "Checksums-Sha256": self._make_checksums_field(
+                    "sha256", [f_dsc, f_tar, f_deb]
+                ),
+                "Date": changes_all["Date"],
+                "Description": "\n hello - A Description",
+                "Distribution": changes_all["Distribution"],
+                "Files": self._make_files_field([f_dsc, f_tar, f_deb]),
+                "Format": "1.8",
+                "Maintainer": changes_all["Maintainer"],
+                "Source": "hello",
+                "Urgency": changes_all["Urgency"],
+                "Version": "1.0",
+            },
+        )
+        for target in (f_dsc, f_tar, f_deb):
+            link = merged_changes_path.with_name(target.name)
+            self.assertTrue(link.is_symlink())
+            self.assertEqual(link.readlink(), target)
+
     def test_merge_descriptions_left_only(self) -> None:
         """Merge ``Description`` where it is only on the left side."""
         merged = deb822.Changes({"Description": "\n foo - package foo"})
@@ -466,13 +606,14 @@ class MergeUploadsTests(ExternalTaskHelperMixin[MergeUploads], TestCase):
             result_type=LookupResultType.ARTIFACT, artifact=1
         )
         debusine_mock.download_artifact.return_value = True
-        debusine_mock.upload_artifact.return_value = RemoteArtifact(
+        debusine_mock.upload_artifact.return_value = create_remote_artifact(
             id=2, workspace=self.task.workspace_name
         )
 
-        f_deb_all = download_directory / "meritous-data_1.5-1.1_all.deb"
-        f_deb_all.write_bytes(f_deb_all_contents := f_deb_all.name.encode())
-        f_changes_all = download_directory / "meritous_1.5-1.1_all.changes"
+        (upload_all := download_directory / "upload-1").mkdir()
+        f_deb_all = upload_all / "meritous-data_1.5-1.1_all.deb"
+        f_deb_all.write_bytes(f_deb_all.name.encode())
+        f_changes_all = upload_all / "meritous_1.5-1.1_all.changes"
         changes_all = ArtifactPlayground.write_deb822_file(
             deb822.Changes,
             f_changes_all,
@@ -481,11 +622,13 @@ class MergeUploadsTests(ExternalTaskHelperMixin[MergeUploads], TestCase):
             binaries=["meritous-data"],
             version="1.5-1.1",
         )
-        f_deb_lib = download_directory / "libmeritous1_1.5-1.1_amd64.deb"
-        f_deb_lib.write_bytes(f_deb_lib_contents := f_deb_lib.name.encode())
-        f_deb_prog = download_directory / "meritous_1.5-1.1_amd64.deb"
-        f_deb_prog.write_bytes(f_deb_prog_contents := f_deb_prog.name.encode())
-        f_changes_amd64 = download_directory / "meritous_1.5-1.1_amd64.changes"
+
+        (upload_amd64 := download_directory / "upload-2").mkdir()
+        f_deb_lib = upload_amd64 / "libmeritous1_1.5-1.1_amd64.deb"
+        f_deb_lib.write_bytes(f_deb_lib.name.encode())
+        f_deb_prog = upload_amd64 / "meritous_1.5-1.1_amd64.deb"
+        f_deb_prog.write_bytes(f_deb_prog.name.encode())
+        f_changes_amd64 = upload_amd64 / "meritous_1.5-1.1_amd64.changes"
         ArtifactPlayground.write_deb822_file(
             deb822.Changes,
             f_changes_amd64,
@@ -513,30 +656,12 @@ class MergeUploadsTests(ExternalTaskHelperMixin[MergeUploads], TestCase):
                     "Architecture": "all amd64",
                     "Binary": "libmeritous1 meritous meritous-data",
                     "Changes": changes_all["Changes"],
-                    "Checksums-Sha1": [
-                        {
-                            "name": path.name,
-                            "sha1": hashlib.sha1(contents).hexdigest(),
-                            "size": str(path.stat().st_size),
-                        }
-                        for path, contents in (
-                            (f_deb_all, f_deb_all_contents),
-                            (f_deb_lib, f_deb_lib_contents),
-                            (f_deb_prog, f_deb_prog_contents),
-                        )
-                    ],
-                    "Checksums-Sha256": [
-                        {
-                            "name": path.name,
-                            "sha256": hashlib.sha256(contents).hexdigest(),
-                            "size": str(path.stat().st_size),
-                        }
-                        for path, contents in (
-                            (f_deb_all, f_deb_all_contents),
-                            (f_deb_lib, f_deb_lib_contents),
-                            (f_deb_prog, f_deb_prog_contents),
-                        )
-                    ],
+                    "Checksums-Sha1": self._make_checksums_field(
+                        "sha1", [f_deb_all, f_deb_lib, f_deb_prog]
+                    ),
+                    "Checksums-Sha256": self._make_checksums_field(
+                        "sha256", [f_deb_all, f_deb_lib, f_deb_prog]
+                    ),
                     "Date": changes_all["Date"],
                     "Description": (
                         "\n"
@@ -545,20 +670,9 @@ class MergeUploadsTests(ExternalTaskHelperMixin[MergeUploads], TestCase):
                         " meritous - A Description"
                     ),
                     "Distribution": changes_all["Distribution"],
-                    "Files": [
-                        {
-                            "md5sum": hashlib.md5(contents).hexdigest(),
-                            "name": path.name,
-                            "priority": "optional",
-                            "section": "unknown",
-                            "size": str(path.stat().st_size),
-                        }
-                        for path, contents in (
-                            (f_deb_all, f_deb_all_contents),
-                            (f_deb_lib, f_deb_lib_contents),
-                            (f_deb_prog, f_deb_prog_contents),
-                        )
-                    ],
+                    "Files": self._make_files_field(
+                        [f_deb_all, f_deb_lib, f_deb_prog]
+                    ),
                     "Format": "1.8",
                     "Maintainer": changes_all["Maintainer"],
                     "Source": "meritous",
@@ -569,8 +683,11 @@ class MergeUploadsTests(ExternalTaskHelperMixin[MergeUploads], TestCase):
         )
         debusine_mock.upload_artifact.assert_called_once_with(
             Upload.create(
-                changes_file=download_directory
-                / "meritous_1.5-1.1_multi.changes"
+                changes_file=(
+                    execute_directory
+                    / "merged"
+                    / "meritous_1.5-1.1_multi.changes"
+                )
             ),
             workspace=self.task.workspace_name,
             work_request=self.task.work_request_id,
@@ -601,7 +718,14 @@ class MergeUploadsTests(ExternalTaskHelperMixin[MergeUploads], TestCase):
         f_merged = self.create_temporary_file()
         self.write_changes_file(f_merged, [f_deb])
         merged = deb822.Changes(f_merged.read_text())
-        self.task._upload_artifact = self.task.make_upload_artifact(merged)
+        merged_changes_path = (
+            download_directory / "meritous_1.5-1.1_multi.changes"
+        )
+        with open(merged_changes_path, "w") as f:
+            merged.dump(f, text_mode=True)
+        self.task._upload_artifact = self.task.make_upload_artifact(
+            merged_changes_path
+        )
 
         # Debusine.upload_artifact is mocked to verify the call only
         debusine_mock = self.mock_debusine()
@@ -609,7 +733,7 @@ class MergeUploadsTests(ExternalTaskHelperMixin[MergeUploads], TestCase):
         workspace_name = "testing"
 
         uploaded_artifacts = [
-            RemoteArtifact(id=10, workspace=workspace_name),
+            create_remote_artifact(id=10, workspace=workspace_name),
         ]
 
         debusine_mock.upload_artifact.side_effect = uploaded_artifacts
