@@ -9,23 +9,38 @@
 
 """Unit tests for the noop task."""
 
-from unittest import TestCase
+from tempfile import TemporaryDirectory
+from unittest import TestCase, mock
 
 from debusine.tasks import Noop, TaskConfigError
 
 
-class NoopTaskTests(TestCase):
-    """Test the noop task."""
+class NoopTests(TestCase):
+    """Test the Noop class."""
 
     def test_execute_true(self) -> None:
         """Assert execute() return True (result is True)."""
         noop = Noop({"result": True})
-        self.assertTrue(noop.execute())
+        noop._debug_log_files_directory = TemporaryDirectory(
+            prefix="debusine-tests-"
+        )
+        self.addCleanup(noop._debug_log_files_directory.cleanup)
+        with mock.patch.object(
+            noop, "_upload_work_request_debug_logs", autospec=True
+        ):
+            self.assertTrue(noop.execute())
 
     def test_execute_false(self) -> None:
         """Assert execute() return False (result is False)."""
         noop = Noop({"result": False})
-        self.assertFalse(noop.execute())
+        noop._debug_log_files_directory = TemporaryDirectory(
+            prefix="debusine-tests-"
+        )
+        self.addCleanup(noop._debug_log_files_directory.cleanup)
+        with mock.patch.object(
+            noop, "_upload_work_request_debug_logs", autospec=True
+        ):
+            self.assertFalse(noop.execute())
 
     def test_no_additional_properties(self) -> None:
         """Assert no additional properties."""
