@@ -36,6 +36,7 @@ except ImportError:
 from debusine.artifacts import WorkRequestDebugLogs
 from debusine.artifacts.models import (
     ArtifactCategory,
+    CollectionCategory,
     EmptyArtifactData,
     TaskTypes,
 )
@@ -706,6 +707,26 @@ class BaseTaskTests(TestCase):
                 ],
             )
 
+    def test_ensure_collection_category_ok(self) -> None:
+        BaseTask.ensure_collection_category(
+            configuration_key="source_collection",
+            category=CollectionCategory.TEST,
+            expected=CollectionCategory.TEST,
+        )
+
+    def test_ensure_collection_category_raise(self) -> None:
+        with self.assertRaisesRegex(
+            TaskConfigError,
+            fr"^source_collection: unexpected collection category: "
+            fr"'{CollectionCategory.TEST}'\. "
+            fr"Expected: '{CollectionCategory.SUITE}'$",
+        ):
+            BaseTask.ensure_collection_category(
+                configuration_key="source_collection",
+                category=CollectionCategory.TEST,
+                expected=CollectionCategory.SUITE,
+            )
+
     def test_instantiate_with_new_data(self) -> None:
         task = self.task2.instantiate_with_new_data({"foo": "baz"})
         self.assertEqual(self.task2.data.foo, "bar")
@@ -1522,40 +1543,38 @@ class RunCommandTaskTests(
         # Block SIGUSR1 signal in case that it is delivered before
         # _do_wait_for_sigusr1 collects it
 
-        signal.pthread_sigmask(signal.SIG_BLOCK, {signal.SIGUSR1})
-
-        self.addCleanup(
-            signal.pthread_sigmask, signal.SIG_UNBLOCK, {signal.SIGUSR1}
-        )
+        old_mask = signal.pthread_sigmask(signal.SIG_BLOCK, {signal.SIGUSR1})
 
         script_path = self.write_signal_logger()
 
-        return task.run_cmd(
-            [
-                str(script_path),
-                str(self.temp_directory),
-                str(os.getpid()),
-                *options,
-            ],
-            self.temp_directory,
-        )
+        try:
+            return task.run_cmd(
+                [
+                    str(script_path),
+                    str(self.temp_directory),
+                    str(os.getpid()),
+                    *options,
+                ],
+                self.temp_directory,
+            )
+        finally:
+            signal.pthread_sigmask(signal.SIG_SETMASK, old_mask)
 
     def run_success_failure(self, task: RunCommandTaskForTesting) -> int | None:
         """Run success/failure script in task."""
         # Block SIGUSR1 signal in case that it is delivered before
         # _do_wait_for_sigusr1 collects it
 
-        signal.pthread_sigmask(signal.SIG_BLOCK, {signal.SIGUSR1})
-
-        self.addCleanup(
-            signal.pthread_sigmask, signal.SIG_UNBLOCK, {signal.SIGUSR1}
-        )
+        old_mask = signal.pthread_sigmask(signal.SIG_BLOCK, {signal.SIGUSR1})
 
         script_path = self.write_success_failure()
 
-        return task.run_cmd(
-            [str(script_path), str(os.getpid())], self.temp_directory
-        )
+        try:
+            return task.run_cmd(
+                [str(script_path), str(os.getpid())], self.temp_directory
+            )
+        finally:
+            signal.pthread_sigmask(signal.SIG_SETMASK, old_mask)
 
     @staticmethod
     def pid_exist_not_zombie(pid: int) -> bool:
