@@ -486,12 +486,12 @@ class TestCase(unittest.TestCase):
         errors: list[str] = []
         for error in error_log:
             match (error.domain, error.type_name):
-                case Domains.HTML, "HTML_UNKNOWN_TAG":
+                # Nice to have if available, but libxml2 >= 2.14.0 no longer
+                # provides unknown-tag checks at all so we can't reliably
+                # cover this.
+                case Domains.HTML, "HTML_UNKNOWN_TAG":  # pragma: no cover
                     if self.re_lxml_false_positive_tags.match(error.message):
                         continue
-                    # Without this pass, python coverage is currently unable to
-                    # detect that code does flow through here
-                    pass
 
             errors.append(f"{error.line}:{error.type_name}:{error.message}")
         return errors
