@@ -30,4 +30,9 @@ urlpatterns = [
         views.MembershipDeleteView.as_view(),
         name="remove-member",
     ),
+    path(
+        "<group>/audit-log/",
+        views.GroupAuditLogView.as_view(),
+        name="audit-log",
+    ),
 ]
