@@ -10,11 +10,13 @@
 """Views for debusine."""
 
 from debusine.web.views.autopkgtest import AutopkgtestView
+from debusine.web.views.debdiff import DebDiffView
 from debusine.web.views.lintian import LintianView
 from debusine.web.views.views import HomepageView
 
 __all__ = [
     "AutopkgtestView",
+    "DebDiffView",
     "HomepageView",
     "LintianView",
 ]
