@@ -655,6 +655,8 @@ class UserTokenDeleteViewTests(TestCase):
     def test_get_delete_token_authenticated_user(self) -> None:
         """Get request to delete a token. Token information is returned."""
         token = self.scenario.user_token
+        token.last_seen_at = timezone.now()
+        token.save()
         self.client.force_login(self.scenario.user)
 
         response = self.client.get(
@@ -672,6 +674,7 @@ class UserTokenDeleteViewTests(TestCase):
             f"<li>Comment: {token.comment}</li>"
             f"<li>Enabled: {html_check_icon(token.enabled)}</li>"
             f"<li>Created at: {date_format(token.created_at)}</li>"
+            f"<li>Last seen at: {date_format(token.last_seen_at)}</li>"
             "</ul>"
         )
 
