@@ -14,7 +14,7 @@ from unittest import mock
 
 from django.contrib.auth.models import AnonymousUser
 from django.core.exceptions import PermissionDenied
-from django.http import HttpResponse
+from django.http import Http404, HttpResponse
 from django.template.response import TemplateResponse
 from django.test import RequestFactory
 from django.views.generic import TemplateView
@@ -76,6 +76,14 @@ class TestBaseUI(ViewTestMixin, TestCase):
         ):
             view.enforce(scope.can_display)
 
+    def test_set_current_workspace_wrong_name(self) -> None:
+        """Test setting the current workspace to an invalid name."""
+        context.set_scope(self.scenario.scope)
+        context.set_user(self.scenario.user)
+        view = BaseUIView()
+        with self.assertRaises(Http404):
+            view.set_current_workspace("does-not-exist")
+
     def _call_init_view(self, method: str, add_method: bool) -> bool:
         """Check if init_view is called for the given method."""
         factory = RequestFactory()
@@ -177,13 +185,14 @@ class TestBaseUI(ViewTestMixin, TestCase):
         context.set_scope(self.scenario.scope)
         context.set_user(AnonymousUser())
         response = self.render_base()
-        tree = self.assertHTMLValid(response)
+        tree = self.assertResponseHTML(response)
 
         self.assertNavCommonElements(tree)
         self.assertNavNoUser(tree)
         self.assertNavNoWorkspaces(tree)
         self.assertNavNoWorkflows(tree)
         self.assertNavNoCollections(tree)
+        self.assertNavNoPlumbing(tree)
 
     def test_navbar_no_workspace(self) -> None:
         """Test the navbar with no user and no current workspace."""
@@ -191,26 +200,28 @@ class TestBaseUI(ViewTestMixin, TestCase):
         context.set_user(self.scenario.user)
 
         response = self.render_base()
-        tree = self.assertHTMLValid(response)
+        tree = self.assertResponseHTML(response)
 
         self.assertNavCommonElements(tree)
         self.assertNavHasUser(tree, self.scenario.user)
         self.assertNavNoWorkspaces(tree)
         self.assertNavNoWorkflows(tree)
         self.assertNavNoCollections(tree)
+        self.assertNavNoPlumbing(tree)
 
     def test_navbar_with_workspace(self) -> None:
         """Test the navbar with user and current workspace."""
         self.scenario.set_current()
 
         response = self.render_base()
-        tree = self.assertHTMLValid(response)
+        tree = self.assertResponseHTML(response)
 
         self.assertNavCommonElements(tree)
         self.assertNavHasCollections(tree, self.scenario.workspace)
         self.assertNavHasWorkflows(tree, self.scenario.workspace, [])
         self.assertNavHasWorkspaces(tree, self.scenario.workspace, [])
         self.assertNavHasUser(tree, self.scenario.user)
+        self.assertNavHasPlumbing(tree)
 
     def test_navbar_with_workflow_templates(self) -> None:
         """Test the navbar with user and current workspace."""
@@ -221,7 +232,7 @@ class TestBaseUI(ViewTestMixin, TestCase):
         wt_baz = self.playground.create_workflow_template("baz", "noop")
 
         response = self.render_base()
-        tree = self.assertHTMLValid(response)
+        tree = self.assertResponseHTML(response)
 
         self.assertNavCommonElements(tree)
         self.assertNavHasCollections(tree, self.scenario.workspace)
@@ -241,8 +252,8 @@ class TestBaseUI(ViewTestMixin, TestCase):
         self.playground.create_workspace(name="c")
 
         response = self.render_base()
+        tree = self.assertResponseHTML(response)
         self.assertEqual(response.context["other_workspaces"], [a, b])
-        tree = self.assertHTMLValid(response)
 
         self.assertNavCommonElements(tree)
         self.assertNavHasCollections(tree, self.scenario.workspace)
@@ -257,7 +268,7 @@ class TestBaseUI(ViewTestMixin, TestCase):
             "debusine.web.views.base.version", return_value="0.1.0"
         ):
             response = self.render_base()
-        tree = self.assertHTMLValid(response)
+        tree = self.assertResponseHTML(response)
 
         self.assertTextContentEqual(
             tree.xpath("//footer/p")[-1],
@@ -272,9 +283,47 @@ class TestBaseUI(ViewTestMixin, TestCase):
             "debusine.web.views.base.version", side_effect=PackageNotFoundError
         ):
             response = self.render_base()
-        tree = self.assertHTMLValid(response)
+        tree = self.assertResponseHTML(response)
 
         self.assertTextContentEqual(
             tree.xpath("//footer/p")[-1],
             "Documentation • Bugs • Code • Contributing",
         )
+
+    def test_footer_custom_replace(self) -> None:
+        """Footer replaced by templates installed by site admins."""
+        context.set_scope(self.scenario.scope)
+        context.set_user(AnonymousUser())
+        tree = self.assertResponseHTML(self.render_base())
+        footer = self.assertHasElement(tree, "//footer")
+        self.assertTextContentEqual(
+            footer.p[0], "Status: Worker pools • Workers • Task status"
+        )
+
+        with self.custom_template("web/footer.html") as template:
+            template.write_text("CUSTOM")
+            tree = self.assertResponseHTML(self.render_base())
+            footer = self.assertHasElement(tree, "//footer")
+            self.assertTextContentEqual(footer, "CUSTOM")
+
+    def test_footer_custom_extend(self) -> None:
+        """Footer extended by templates installed by site admins."""
+        context.set_scope(self.scenario.scope)
+        context.set_user(AnonymousUser())
+        tree = self.assertResponseHTML(self.render_base())
+        footer = self.assertHasElement(tree, "//footer")
+        self.assertTextContentEqual(
+            footer.p[0], "Status: Worker pools • Workers • Task status"
+        )
+
+        with self.custom_template("web/footer.html") as template:
+            template.write_text(
+                "{% extends 'web/footer.html' %}"
+                "{% block status %}{{block.super}} • Testing{% endblock %}"
+            )
+            tree = self.assertResponseHTML(self.render_base())
+            footer = self.assertHasElement(tree, "//footer")
+            self.assertTextContentEqual(
+                footer.p[0],
+                "Status: Worker pools • Workers • Task status • Testing",
+            )
