@@ -218,7 +218,7 @@ class WorkRequestDetailViewTests(ViewTestMixin, TestCase):
 
         self.assertNotIn("task_data_original-work_request", response.context)
 
-        self.assertHasElement(tree, "//p[@id='task_data_not_configured']")
+        self.assertFalse(tree.xpath("//p[@id='task_data_not_configured']"))
 
     def test_get_work_request_no_input_output_artifacts(self) -> None:
         work_request = self.playground.create_work_request()
@@ -268,22 +268,42 @@ class WorkRequestDetailViewTests(ViewTestMixin, TestCase):
         )
 
     def test_get_work_request_task_data_not_configured(self) -> None:
+        self.work_request.configured_task_data = None
+        self.work_request.save()
         response = self.client.get(self.work_request.get_absolute_url())
         tree = self.assertResponseHTML(response)
 
         task_data_not_configured_p = self.assertHasElement(
             tree, "//p[@id='task_data_not_configured']"
         )
-        dynamic_data_not_configured_p = self.assertHasElement(
-            tree, "//p[@id='dynamic_data_not_configured']"
-        )
-
         self.assertTextContentEqual(
             task_data_not_configured_p, "Work request is not configured."
         )
-        self.assertTextContentEqual(
-            dynamic_data_not_configured_p, "Work request is not configured."
+        self.assertFalse(tree.xpath("//p[@id='dynamic_data_not_available']"))
+
+    def test_get_work_request_dynamic_data_not_available(self) -> None:
+        self.work_request.dynamic_task_data = None
+        self.work_request.save()
+
+        response = self.client.get(self.work_request.get_absolute_url())
+        tree = self.assertResponseHTML(response)
+
+        task_dynamic_data_not_available_p = self.assertHasElement(
+            tree, "//p[@id='dynamic_data_not_available']"
         )
+        self.assertTextContentEqual(
+            task_dynamic_data_not_available_p, "No dynamic task data."
+        )
+
+    def test_get_work_request_dynamic_data(self) -> None:
+        response = self.client.get(self.work_request.get_absolute_url())
+        tree = self.assertResponseHTML(response)
+
+        dynamic_task_data = self.assertHasCardYaml(
+            tree, div_id="card-dynamic_data"
+        )
+
+        self.assertEqual(dynamic_task_data, self.work_request.dynamic_task_data)
 
     def test_get_work_request_incomplete(self) -> None:
         """Artifacts with incomplete files are marked as incomplete."""
@@ -367,6 +387,7 @@ class WorkRequestDetailViewTests(ViewTestMixin, TestCase):
         """Show the retry link if a work request can be retried."""
         self.work_request.task_name = "noop"
         self.work_request.task_data = {}
+        self.work_request.configured_task_data = {}
         self.work_request.dynamic_task_data = {}
         self.work_request.status = WorkRequest.Statuses.ABORTED
         self.work_request.save()
