@@ -688,6 +688,11 @@ class Worker:
             # work request
             self._task_lock.release()
 
+    @staticmethod
+    def _exit_worker() -> NoReturn:  # pragma: no cover
+        """Exit the worker, mocked in tests."""
+        raise SystemExit(1)
+
     async def _send_task_result(
         self, work_request_id: int, task_exec_future: Future[bool]
     ) -> None:
@@ -748,8 +753,11 @@ class Worker:
                 # will retry it when this worker next manages to connect and
                 # request a new work request to run.
                 logging.exception(
-                    "Cannot reach server to report work request completed."
+                    "Cannot reach server to report work request completed. "
+                    "Exiting."
                 )
+                # Exit so that the worker will try to reconnect.
+                self._exit_worker()
         finally:
             self._reset_state()
 
