@@ -29,6 +29,12 @@ def cpu_count() -> int:
     return count
 
 
+def cpu_time() -> float:
+    """Return total CPU time (user + system) used so far."""
+    cpu_times = psutil.cpu_times()
+    return cpu_times.user + cpu_times.system
+
+
 @cache
 def host_architecture() -> str | None:
     """Return the architecture of the system or None if dpkg not available."""
