@@ -88,7 +88,7 @@ class TestDebusineAsyncHttpClient(server.DebusineAioHTTPTestCase, TestCase):
             )
 
     async def make_http_request(
-        self, method: str, path: str
+        self, method: str, path: str, token: str | None = None
     ) -> dict[str, str] | None:
         """
         Make HTTP request, return data sent (or None if no data was sent).
@@ -102,7 +102,7 @@ class TestDebusineAsyncHttpClient(server.DebusineAioHTTPTestCase, TestCase):
             data = None
 
         await self.debusine_async_http_client._make_http_request(
-            method, path, json=data
+            method, path, json=data, token=token
         )
 
         return data
@@ -154,6 +154,21 @@ class TestDebusineAsyncHttpClient(server.DebusineAioHTTPTestCase, TestCase):
                     method, path, include_token=include_token, data=data
                 )
 
+    async def test_make_http_request_explicit_token(self) -> None:
+        """_make_http_request sends an explicitly-requested token."""
+        path = '/collect_request_information/'
+        token = 'a84b92e3c3379733bf0d2b25700c4c'
+
+        for method, config_has_token in itertools.product(
+            ["PUT", "POST", "GET"], [True, False]
+        ):
+            with self.subTest(method=method):
+                self.setup_valid_token(config_has_token)
+                data = await self.make_http_request(method, path, token=token)
+                await self.assert_http_request(
+                    method, path, include_token=True, data=data
+                )
+
     async def test_make_http_request_get_with_data_raises_value_error(
         self,
     ) -> None:
